@@ -14,25 +14,25 @@
 namespace fst {
 namespace script {
 
-string LoadArcTypeFromFar(const string &far_fname) {
+std::string LoadArcTypeFromFar(const std::string &far_source) {
   FarHeader hdr;
-  if (!hdr.Read(far_fname)) {
-    LOG(ERROR) << "Error reading FAR: " << far_fname;
+  if (!hdr.Read(far_source)) {
+    LOG(ERROR) << "Error reading FAR: " << far_source;
     return "";
   }
-  string atype = hdr.ArcType();
+  std::string atype = hdr.ArcType();
   if (atype == "unknown") {
-    LOG(ERROR) << "Empty FST archive: " << far_fname;
+    LOG(ERROR) << "Empty FST archive: " << far_source;
     return "";
   }
   return atype;
 }
 
-string LoadArcTypeFromFst(const string &fst_fname) {
+std::string LoadArcTypeFromFst(const std::string &fst_source) {
   FstHeader hdr;
-  std::ifstream in(fst_fname, std::ios_base::in | std::ios_base::binary);
-  if (!hdr.Read(in, fst_fname)) {
-    LOG(ERROR) << "Error reading FST: " << fst_fname;
+  std::ifstream in(fst_source, std::ios_base::in | std::ios_base::binary);
+  if (!hdr.Read(in, fst_source)) {
+    LOG(ERROR) << "Error reading FST: " << fst_source;
     return "";
   }
   return hdr.ArcType();
