@@ -1,13 +1,14 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#include <fstream>
 #include <fst/extensions/far/sttable.h>
 
+#include <fstream>
+
 namespace fst {
 
-bool IsSTTable(const string &filename) {
-  std::ifstream strm(filename);
+bool IsSTTable(const std::string &source) {
+  std::ifstream strm(source);
   if (!strm.good()) return false;
 
   int32 magic_number = 0;
