@@ -5,11 +5,12 @@
 
 #include <sys/types.h>
 
-#include <fst/fstlib.h>
+#include <fst/arc.h>
+#include <fst/register.h>
 
+using fst::LogArc;
 using fst::NGramFst;
 using fst::StdArc;
-using fst::LogArc;
 
 REGISTER_FST(NGramFst, StdArc);
 REGISTER_FST(NGramFst, LogArc);
