@@ -7,12 +7,15 @@
 // See comments in nlp/fst/script/script-impl.h for how the registration
 // mechanism allows these to work with various arc types.
 
+#include <fst/extensions/pdt/pdtscript.h>
+
 #include <string>
+#include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/extensions/pdt/compose.h>
 #include <fst/extensions/pdt/expand.h>
-#include <fst/extensions/pdt/pdtscript.h>
 #include <fst/extensions/pdt/replace.h>
 #include <fst/extensions/pdt/reverse.h>
 #include <fst/extensions/pdt/shortest-path.h>
@@ -22,23 +25,28 @@ namespace fst {
 namespace script {
 
 void PdtCompose(const FstClass &ifst1, const FstClass &ifst2,
-                const std::vector<LabelPair> &parens,
+                const std::vector<std::pair<int64, int64>> &parens,
                 MutableFstClass *ofst, const PdtComposeOptions &copts,
                 bool left_pdt) {
-  if (!ArcTypesMatch(ifst1, ifst2, "PdtCompose") ||
-      !ArcTypesMatch(ifst1, *ofst, "PdtCompose"))
+  if (!internal::ArcTypesMatch(ifst1, ifst2, "PdtCompose") ||
+      !internal::ArcTypesMatch(ifst1, *ofst, "PdtCompose")) {
     return;
+  }
   PdtComposeArgs args(ifst1, ifst2, parens, ofst, copts, left_pdt);
   Apply<Operation<PdtComposeArgs>>("PdtCompose", ifst1.ArcType(), &args);
 }
 
+REGISTER_FST_OPERATION_3ARCS(PdtCompose, PdtComposeArgs);
+
 void PdtExpand(const FstClass &ifst,
-               const std::vector<LabelPair> &parens,
+               const std::vector<std::pair<int64, int64>> &parens,
                MutableFstClass *ofst, const PdtExpandOptions &opts) {
   PdtExpandArgs args(ifst, parens, ofst, opts);
   Apply<Operation<PdtExpandArgs>>("PdtExpand", ifst.ArcType(), &args);
 }
 
+REGISTER_FST_OPERATION_3ARCS(PdtExpand, PdtExpandArgs);
+
 void PdtExpand(const FstClass &ifst,
                const std::vector<std::pair<int64, int64>> &parens,
                MutableFstClass *ofst, bool connect, bool keep_parentheses,
@@ -47,31 +55,37 @@ void PdtExpand(const FstClass &ifst,
             PdtExpandOptions(connect, keep_parentheses, weight_threshold));
 }
 
-void PdtReplace(const std::vector<LabelFstClassPair> &pairs,
-                MutableFstClass *ofst, std::vector<LabelPair> *parens,
-                int64 root, PdtParserType parser_type, int64 start_paren_labels,
-                const string &left_paren_prefix,
-                const string &right_paren_prefix) {
-  for (auto i = 0; i < pairs.size() - 1; ++i) {
-    if (!ArcTypesMatch(*pairs[i].second, *pairs[i + 1].second, "PdtReplace"))
+void PdtReplace(const std::vector<std::pair<int64, const FstClass *>> &pairs,
+                MutableFstClass *ofst,
+                std::vector<std::pair<int64, int64>> *parens, int64 root,
+                PdtParserType parser_type, int64 start_paren_labels,
+                const std::string &left_paren_prefix,
+                const std::string &right_paren_prefix) {
+  for (size_t i = 1; i < pairs.size(); ++i) {
+    if (!internal::ArcTypesMatch(*pairs[i - 1].second, *pairs[i].second,
+                                 "PdtReplace"))
       return;
   }
-  if (!ArcTypesMatch(*pairs[0].second, *ofst, "PdtReplace")) return;
+  if (!internal::ArcTypesMatch(*pairs[0].second, *ofst, "PdtReplace")) return;
   PdtReplaceArgs args(pairs, ofst, parens, root, parser_type,
                       start_paren_labels, left_paren_prefix,
                       right_paren_prefix);
   Apply<Operation<PdtReplaceArgs>>("PdtReplace", ofst->ArcType(), &args);
 }
 
+REGISTER_FST_OPERATION_3ARCS(PdtReplace, PdtReplaceArgs);
+
 void PdtReverse(const FstClass &ifst,
-                const std::vector<LabelPair> &parens,
+                const std::vector<std::pair<int64, int64>> &parens,
                 MutableFstClass *ofst) {
   PdtReverseArgs args(ifst, parens, ofst);
   Apply<Operation<PdtReverseArgs>>("PdtReverse", ifst.ArcType(), &args);
 }
 
+REGISTER_FST_OPERATION_3ARCS(PdtReverse, PdtReverseArgs);
+
 void PdtShortestPath(const FstClass &ifst,
-                     const std::vector<LabelPair> &parens,
+                     const std::vector<std::pair<int64, int64>> &parens,
                      MutableFstClass *ofst,
                      const PdtShortestPathOptions &opts) {
   PdtShortestPathArgs args(ifst, parens, ofst, opts);
@@ -79,17 +93,15 @@ void PdtShortestPath(const FstClass &ifst,
                                         &args);
 }
 
+REGISTER_FST_OPERATION_3ARCS(PdtShortestPath, PdtShortestPathArgs);
+
 void PrintPdtInfo(const FstClass &ifst,
-                  const std::vector<LabelPair> &parens) {
+                  const std::vector<std::pair<int64, int64>> &parens) {
   PrintPdtInfoArgs args(ifst, parens);
   Apply<Operation<PrintPdtInfoArgs>>("PrintPdtInfo", ifst.ArcType(), &args);
 }
 
-// Register operations for common arc types.
-
-REGISTER_FST_PDT_OPERATIONS(StdArc);
-REGISTER_FST_PDT_OPERATIONS(LogArc);
-REGISTER_FST_PDT_OPERATIONS(Log64Arc);
+REGISTER_FST_OPERATION_3ARCS(PrintPdtInfo, PrintPdtInfoArgs);
 
 }  // namespace script
 }  // namespace fst
