@@ -3,7 +3,9 @@
 
 #include <fst/extensions/special/sigma-fst.h>
 
+#include <fst/arc.h>
 #include <fst/fst.h>
+#include <fst/register.h>
 
 DEFINE_int64(sigma_fst_sigma_label, 0,
              "Label of transitions to be interpreted as sigma ('any') "
@@ -18,16 +20,16 @@ const char sigma_fst_type[] = "sigma";
 const char input_sigma_fst_type[] = "input_sigma";
 const char output_sigma_fst_type[] = "output_sigma";
 
-static FstRegisterer<StdSigmaFst> SigmaFst_StdArc_registerer;
-static FstRegisterer<LogSigmaFst> SigmaFst_LogArc_registerer;
-static FstRegisterer<Log64SigmaFst> SigmaFst_Log64Arc_registerer;
+REGISTER_FST(SigmaFst, StdArc);
+REGISTER_FST(SigmaFst, LogArc);
+REGISTER_FST(SigmaFst, Log64Arc);
 
-static FstRegisterer<StdInputSigmaFst> InputSigmaFst_StdArc_registerer;
-static FstRegisterer<LogInputSigmaFst> InputSigmaFst_LogArc_registerer;
-static FstRegisterer<Log64InputSigmaFst> InputSigmaFst_Log64Arc_registerer;
+REGISTER_FST(InputSigmaFst, StdArc);
+REGISTER_FST(InputSigmaFst, LogArc);
+REGISTER_FST(InputSigmaFst, Log64Arc);
 
-static FstRegisterer<StdOutputSigmaFst> OutputSigmaFst_StdArc_registerer;
-static FstRegisterer<LogOutputSigmaFst> OutputSigmaFst_LogArc_registerer;
-static FstRegisterer<Log64OutputSigmaFst> OutputSigmaFst_Log64Arc_registerer;
+REGISTER_FST(OutputSigmaFst, StdArc);
+REGISTER_FST(OutputSigmaFst, LogArc);
+REGISTER_FST(OutputSigmaFst, Log64Arc);
 
 }  // namespace fst
