@@ -3,8 +3,8 @@
 //
 // Classes to accumulate arc weights. Useful for weight lookahead.
 
-#ifndef FST_LIB_ACCUMULATOR_H_
-#define FST_LIB_ACCUMULATOR_H_
+#ifndef FST_ACCUMULATOR_H_
+#define FST_ACCUMULATOR_H_
 
 #include <algorithm>
 #include <functional>
@@ -22,6 +22,7 @@
 namespace fst {
 
 // This class accumulates arc weights using the semiring Plus().
+// Sum(w, aiter, begin, end) has time complexity O(begin - end).
 template <class A>
 class DefaultAccumulator {
  public:
@@ -56,6 +57,7 @@ class DefaultAccumulator {
 
 // This class accumulates arc weights using the log semiring Plus() assuming an
 // arc weight has a WeightConvert specialization to and from log64 weights.
+// Sum(w, aiter, begin, end) has time complexity O(begin - end).
 template <class A>
 class LogAccumulator {
  public:
@@ -87,6 +89,9 @@ class LogAccumulator {
 
  private:
   Weight LogPlus(Weight w, Weight v) {
+    if (w == Weight::Zero()) {
+      return v;
+    }
     const auto f1 = to_log_weight_(w).Value();
     const auto f2 = to_log_weight_(v).Value();
     if (f1 > f2) {
@@ -96,8 +101,8 @@ class LogAccumulator {
     }
   }
 
-  WeightConvert<Weight, Log64Weight> to_log_weight_;
-  WeightConvert<Log64Weight, Weight> to_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
+  const WeightConvert<Log64Weight, Weight> to_weight_{};
 
   LogAccumulator &operator=(const LogAccumulator &) = delete;
 };
@@ -191,6 +196,9 @@ class MutableFastLogAccumulatorData : public FastLogAccumulatorData {
 // arc weight has a WeightConvert specialization to and from log64 weights. The
 // member function Init(fst) has to be called to setup pre-computed weight
 // information.
+// Sum(w, aiter, begin, end) has time complexity O(arc_limit_) or O(arc_period_)
+// depending on whether the state has more than arc_limit_ arcs
+// Space complexity is O(CountStates(fst) + CountArcs(fst) / arc_period_).
 template <class A>
 class FastLogAccumulator {
  public:
@@ -217,7 +225,7 @@ class FastLogAccumulator {
         state_weights_(nullptr),
         error_(false) {}
 
-  FastLogAccumulator(const FastLogAccumulator<Arc> &acc, bool safe = false)
+  FastLogAccumulator(const FastLogAccumulator &acc, bool safe = false)
       : to_log_weight_(),
         to_weight_(),
         arc_limit_(acc.arc_limit_),
@@ -335,6 +343,9 @@ class FastLogAccumulator {
   }
 
   Weight LogPlus(Weight w, Weight v) const {
+    if (w == Weight::Zero()) {
+      return v;
+    }
     const auto f1 = to_log_weight_(w).Value();
     const auto f2 = to_log_weight_(v).Value();
     if (f1 > f2) {
@@ -364,8 +375,8 @@ class FastLogAccumulator {
     }
   }
 
-  const WeightConvert<Weight, Log64Weight> to_log_weight_;
-  const WeightConvert<Log64Weight, Weight> to_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
+  const WeightConvert<Log64Weight, Weight> to_weight_{};
   const ssize_t arc_limit_;   // Minimum number of arcs to pre-compute state.
   const ssize_t arc_period_;  // Saves cumulative weights per arc_period_.
   std::shared_ptr<FastLogAccumulatorData> data_;
@@ -405,7 +416,7 @@ class CacheLogAccumulatorData {
 
   void AddWeights(StateId s, std::vector<double> *weights) {
     if (cache_gc_ && cache_size_ >= cache_limit_) GC(false);
-    cache_.insert(std::make_pair(s, CacheState(weights, true)));
+    cache_.emplace(s, CacheState(weights, true));
     if (cache_gc_) cache_size_ += weights->capacity() * sizeof(double);
   }
 
@@ -451,7 +462,7 @@ class CacheLogAccumulatorData {
 // WeightConvert specialization to and from log64 weights. It is similar to the
 // FastLogAccumator. However here, the accumulated weights are pre-computed and
 // stored only for the states that are visited. The member function Init(fst)
-// has to be called to setup this accumulator.
+// has to be called to setup this accumulator.  Space complexity is O(gc_limit).
 template <class Arc>
 class CacheLogAccumulator {
  public:
@@ -465,7 +476,7 @@ class CacheLogAccumulator {
         s_(kNoStateId),
         error_(false) {}
 
-  CacheLogAccumulator(const CacheLogAccumulator<Arc> &acc, bool safe = false)
+  CacheLogAccumulator(const CacheLogAccumulator &acc, bool safe = false)
       : arc_limit_(acc.arc_limit_),
         fst_(acc.fst_ ? acc.fst_->Copy() : nullptr),
         data_(safe ? std::make_shared<CacheLogAccumulatorData<Arc>>(*acc.data_)
@@ -547,9 +558,9 @@ class CacheLogAccumulator {
     auto pos = aiter->Position();
     if (weights_) {
       Extend(fst_->NumArcs(s_), aiter);
-      return std::lower_bound(weights_->begin() + pos + 1, weights_->end(),
-                              f, std::greater<double>()) -
-          weights_->begin() - 1;
+      return std::lower_bound(weights_->begin() + pos + 1, weights_->end(), f,
+                              std::greater<double>()) -
+             weights_->begin() - 1;
     } else {
       size_t n = 0;
       auto x = FloatLimits<double>::PosInfinity();
@@ -575,6 +586,9 @@ class CacheLogAccumulator {
   }
 
   Weight LogPlus(Weight w, Weight v) {
+    if (w == Weight::Zero()) {
+      return v;
+    }
     const auto f1 = to_log_weight_(w).Value();
     const auto f2 = to_log_weight_(v).Value();
     if (f1 > f2) {
@@ -615,9 +629,8 @@ class CacheLogAccumulator {
     }
   }
 
-
-  WeightConvert<Weight, Log64Weight> to_log_weight_;
-  WeightConvert<Log64Weight, Weight> to_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
+  const WeightConvert<Log64Weight, Weight> to_weight_{};
   ssize_t arc_limit_;                    // Minimum # of arcs to cache a state.
   std::vector<double> *weights_;         // Accumulated weights for cur. state.
   std::unique_ptr<const Fst<Arc>> fst_;  // Input FST.
@@ -653,7 +666,7 @@ class ReplaceAccumulatorData {
     accumulators_.resize(fst_tuples.size());
     for (Label i = 0; i < accumulators_.size(); ++i) {
       if (!accumulators_[i]) {
-        accumulators_[i].reset(new Accumulator());
+        accumulators_[i] = fst::make_unique<Accumulator>();
         accumulators_[i]->Init(*(fst_tuples[i].second));
       }
       fst_array_.emplace_back(fst_tuples[i].second->Copy());
@@ -865,7 +878,7 @@ class SafeReplaceAccumulator {
     ArcIteratorPtr(const ArcIteratorPtr &copy) {}
 
     void Set(const Fst<Arc> &fst, StateId state_id) {
-      ptr_.reset(new ArcIterator<Fst<Arc>>(fst, state_id));
+      ptr_ = fst::make_unique<ArcIterator<Fst<Arc>>>(fst, state_id);
     }
 
     ArcIterator<Fst<Arc>> *get() { return ptr_.get(); }
@@ -891,4 +904,4 @@ class SafeReplaceAccumulator {
 
 }  // namespace fst
 
-#endif  // FST_LIB_ACCUMULATOR_H_
+#endif  // FST_ACCUMULATOR_H_
