@@ -5,13 +5,14 @@
 // implement project/invert. Consider using when operation does
 // not change the number of arcs (except possibly superfinal arcs).
 
-#ifndef FST_LIB_ARC_MAP_H_
-#define FST_LIB_ARC_MAP_H_
+#ifndef FST_ARC_MAP_H_
+#define FST_ARC_MAP_H_
 
 #include <string>
 #include <unordered_map>
 #include <utility>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/cache.h>
@@ -47,8 +48,8 @@ enum MapSymbolsAction {
 };
 
 // The ArcMapper interfaces defines how arcs and final weights are mapped.
-// This is useful for implementing operations that do not change the number of
-// arcs (expect possibly superfinal arcs).
+// This is useful for implementing operations that apply to each arc separately
+// and do not change the number of arcs (except possibly superfinal arcs).
 //
 // template <class A, class B>
 // class ArcMapper {
@@ -94,7 +95,6 @@ template <class A, class C>
 void ArcMap(MutableFst<A> *fst, C *mapper) {
   using FromArc = A;
   using ToArc = A;
-  using StateId = typename FromArc::StateId;
   using Weight = typename FromArc::Weight;
   if (mapper->InputSymbolsAction() == MAP_CLEAR_SYMBOLS) {
     fst->SetInputSymbols(nullptr);
@@ -108,7 +108,7 @@ void ArcMap(MutableFst<A> *fst, C *mapper) {
   auto superfinal = kNoStateId;
   if (final_action == MAP_REQUIRE_SUPERFINAL) {
     superfinal = fst->AddState();
-    fst->SetFinal(superfinal, Weight::One());
+    fst->SetFinal(superfinal);
   }
   for (StateIterator<MutableFst<FromArc>> siter(*fst); !siter.Done();
        siter.Next()) {
@@ -138,10 +138,10 @@ void ArcMap(MutableFst<A> *fst, C *mapper) {
             // Add a superfinal state if not already done.
             if (superfinal == kNoStateId) {
               superfinal = fst->AddState();
-              fst->SetFinal(superfinal, Weight::One());
+              fst->SetFinal(superfinal);
             }
             final_arc.nextstate = superfinal;
-            fst->AddArc(state, final_arc);
+            fst->AddArc(state, std::move(final_arc));
             fst->SetFinal(state, Weight::Zero());
           } else {
             fst->SetFinal(state, final_arc.weight);
@@ -181,7 +181,6 @@ template <class A, class B, class C>
 void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C *mapper) {
   using FromArc = A;
   using StateId = typename FromArc::StateId;
-  using Weight = typename FromArc::Weight;
   ofst->DeleteStates();
   if (mapper->InputSymbolsAction() == MAP_COPY_SYMBOLS) {
     ofst->SetInputSymbols(ifst.InputSymbols());
@@ -200,8 +199,8 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C *mapper) {
   }
   const auto final_action = mapper->FinalAction();
   if (ifst.Properties(kExpanded, false)) {
-    ofst->ReserveStates(
-        CountStates(ifst) + final_action == MAP_NO_SUPERFINAL ? 0 : 1);
+    ofst->ReserveStates(CountStates(ifst) +
+                        (final_action == MAP_NO_SUPERFINAL ? 0 : 1));
   }
   // Adds all states.
   for (StateIterator<Fst<A>> siter(ifst); !siter.Done(); siter.Next()) {
@@ -210,12 +209,13 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C *mapper) {
   StateId superfinal = kNoStateId;
   if (final_action == MAP_REQUIRE_SUPERFINAL) {
     superfinal = ofst->AddState();
-    ofst->SetFinal(superfinal, B::Weight::One());
+    ofst->SetFinal(superfinal);
   }
   for (StateIterator<Fst<A>> siter(ifst); !siter.Done(); siter.Next()) {
     StateId s = siter.Value();
     if (s == ifst.Start()) ofst->SetStart(s);
-    ofst->ReserveArcs(s, ifst.NumArcs(s));
+    ofst->ReserveArcs(
+        s, ifst.NumArcs(s) + (final_action != MAP_NO_SUPERFINAL ? 1 : 0));
     for (ArcIterator<Fst<A>> aiter(ifst, s); !aiter.Done(); aiter.Next()) {
       ofst->AddArc(s, (*mapper)(aiter.Value()));
     }
@@ -236,10 +236,10 @@ void ArcMap(const Fst<A> &ifst, MutableFst<B> *ofst, C *mapper) {
           // Add a superfinal state if not already done.
           if (superfinal == kNoStateId) {
             superfinal = ofst->AddState();
-            ofst->SetFinal(superfinal, B::Weight::One());
+            ofst->SetFinal(superfinal);
           }
           final_arc.nextstate = superfinal;
-          ofst->AddArc(s, final_arc);
+          ofst->AddArc(s, std::move(final_arc));
           ofst->SetFinal(s, B::Weight::Zero());
         } else {
           ofst->SetFinal(s, final_arc.weight);
@@ -296,10 +296,11 @@ class ArcMapFstImpl : public CacheImpl<B> {
   using FstImpl<B>::SetInputSymbols;
   using FstImpl<B>::SetOutputSymbols;
 
-  using CacheImpl<B>::PushArc;
+  using CacheImpl<B>::EmplaceArc;
   using CacheImpl<B>::HasArcs;
   using CacheImpl<B>::HasFinal;
   using CacheImpl<B>::HasStart;
+  using CacheImpl<B>::PushArc;
   using CacheImpl<B>::SetArcs;
   using CacheImpl<B>::SetFinal;
   using CacheImpl<B>::SetStart;
@@ -362,7 +363,7 @@ class ArcMapFstImpl : public CacheImpl<B> {
         }
         case MAP_ALLOW_SUPERFINAL: {
           if (s == superfinal_) {
-            SetFinal(s, Weight::One());
+            SetFinal(s);
           } else {
             const auto final_arc =
                 (*mapper_)(A(0, 0, fst_->Final(FindIState(s)), kNoStateId));
@@ -424,8 +425,7 @@ class ArcMapFstImpl : public CacheImpl<B> {
          aiter.Next()) {
       auto aarc = aiter.Value();
       aarc.nextstate = FindOState(aarc.nextstate);
-      const auto &barc = (*mapper_)(aarc);
-      PushArc(s, barc);
+      PushArc(s, (*mapper_)(aarc));
     }
 
     // Check for superfinal arcs.
@@ -440,7 +440,7 @@ class ArcMapFstImpl : public CacheImpl<B> {
           if (final_arc.ilabel != 0 || final_arc.olabel != 0) {
             if (superfinal_ == kNoStateId) superfinal_ = nstates_++;
             final_arc.nextstate = superfinal_;
-            PushArc(s, final_arc);
+            PushArc(s, std::move(final_arc));
           }
           break;
         }
@@ -449,8 +449,8 @@ class ArcMapFstImpl : public CacheImpl<B> {
               (*mapper_)(A(0, 0, fst_->Final(FindIState(s)), kNoStateId));
           if (final_arc.ilabel != 0 || final_arc.olabel != 0 ||
               final_arc.weight != B::Weight::Zero()) {
-            PushArc(s, B(final_arc.ilabel, final_arc.olabel, final_arc.weight,
-                         superfinal_));
+            EmplaceArc(s, final_arc.ilabel, final_arc.olabel, final_arc.weight,
+                       superfinal_);
           }
           break;
         }
@@ -526,27 +526,21 @@ class ArcMapFst : public ImplToFst<internal::ArcMapFstImpl<A, B, C>> {
   friend class ArcIterator<ArcMapFst<A, B, C>>;
   friend class StateIterator<ArcMapFst<A, B, C>>;
 
-  ArcMapFst(const Fst<A> &fst, const C &mapper, const ArcMapFstOptions &opts)
+  ArcMapFst(const Fst<A> &fst, const C &mapper,
+            const ArcMapFstOptions &opts = ArcMapFstOptions())
       : ImplToFst<Impl>(std::make_shared<Impl>(fst, mapper, opts)) {}
 
-  ArcMapFst(const Fst<A> &fst, C *mapper, const ArcMapFstOptions &opts)
+  ArcMapFst(const Fst<A> &fst, C *mapper,
+            const ArcMapFstOptions &opts = ArcMapFstOptions())
       : ImplToFst<Impl>(std::make_shared<Impl>(fst, mapper, opts)) {}
 
-  ArcMapFst(const Fst<A> &fst, const C &mapper)
-      : ImplToFst<Impl>(
-            std::make_shared<Impl>(fst, mapper, ArcMapFstOptions())) {}
-
-  ArcMapFst(const Fst<A> &fst, C *mapper)
-      : ImplToFst<Impl>(
-            std::make_shared<Impl>(fst, mapper, ArcMapFstOptions())) {}
-
   // See Fst<>::Copy() for doc.
-  ArcMapFst(const ArcMapFst<A, B, C> &fst, bool safe = false)
+  ArcMapFst(const ArcMapFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
   // Get a copy of this ArcMapFst. See Fst<>::Copy() for further doc.
-  ArcMapFst<A, B, C> *Copy(bool safe = false) const override {
-    return new ArcMapFst<A, B, C>(*this, safe);
+  ArcMapFst *Copy(bool safe = false) const override {
+    return new ArcMapFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<B> *data) const override;
@@ -635,6 +629,26 @@ inline void ArcMapFst<A, B, C>::InitStateIterator(
   data->base = new StateIterator<ArcMapFst<A, B, C>>(*this);
 }
 
+// Constructs and returns an ArcMapFst.  This allows constructing ArcMapFsts
+// without specifying all the types. The template argument is typically
+// not specified, so a call looks like: MakeArcMapFst(fst, Mapper(...)).
+template <class ArcMapper>
+ArcMapFst<typename ArcMapper::FromArc, typename ArcMapper::ToArc, ArcMapper>
+MakeArcMapFst(const Fst<typename ArcMapper::FromArc> &fst,
+              const ArcMapper &mapper) {
+  return ArcMapFst<typename ArcMapper::FromArc, typename ArcMapper::ToArc,
+                   ArcMapper>(fst, mapper);
+}
+
+// Constructs and returns an ArcMapFst.  As above, but using the
+// ArcMapFst(..., ArcMapper *) constructor.
+template <class ArcMapper>
+ArcMapFst<typename ArcMapper::FromArc, typename ArcMapper::ToArc, ArcMapper>
+MakeArcMapFst(const Fst<typename ArcMapper::FromArc> &fst, ArcMapper *mapper) {
+  return ArcMapFst<typename ArcMapper::FromArc, typename ArcMapper::ToArc,
+                   ArcMapper>(fst, mapper);
+}
+
 // Utility Mappers.
 
 // Mapper that returns its input.
@@ -644,7 +658,7 @@ class IdentityArcMapper {
   using FromArc = A;
   using ToArc = A;
 
-  ToArc operator()(const FromArc &arc) const { return arc; }
+  constexpr ToArc operator()(const FromArc &arc) const { return arc; }
 
   constexpr MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
 
@@ -656,7 +670,7 @@ class IdentityArcMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const { return props; }
+  constexpr uint64 Properties(uint64 props) const { return props; }
 };
 
 // Mapper that converts all input symbols to epsilon.
@@ -666,7 +680,7 @@ class InputEpsilonMapper {
   using FromArc = A;
   using ToArc = A;
 
-  ToArc operator()(const FromArc &arc) const {
+  constexpr ToArc operator()(const FromArc &arc) const {
     return ToArc(0, arc.olabel, arc.weight, arc.nextstate);
   }
 
@@ -680,8 +694,8 @@ class InputEpsilonMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const {
-    return (props & kSetArcProperties) | kIEpsilons;
+  constexpr uint64 Properties(uint64 props) const {
+    return (props & kSetArcProperties) | kIEpsilons | kILabelSorted;
   }
 };
 
@@ -692,7 +706,7 @@ class OutputEpsilonMapper {
   using FromArc = A;
   using ToArc = A;
 
-  ToArc operator()(const FromArc &arc) const {
+  constexpr ToArc operator()(const FromArc &arc) const {
     return ToArc(arc.ilabel, 0, arc.weight, arc.nextstate);
   }
 
@@ -706,8 +720,8 @@ class OutputEpsilonMapper {
     return MAP_CLEAR_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const {
-    return (props & kSetArcProperties) | kOEpsilons;
+  constexpr uint64 Properties(uint64 props) const {
+    return (props & kSetArcProperties) | kOEpsilons | kOLabelSorted;
   }
 };
 
@@ -718,8 +732,21 @@ class SuperFinalMapper {
  public:
   using FromArc = A;
   using ToArc = A;
+  using Label = typename FromArc::Label;
+  using Weight = typename FromArc::Weight;
 
-  ToArc operator()(const FromArc &arc) const { return arc; }
+  // Arg allows setting super-final label.
+  constexpr explicit SuperFinalMapper(Label final_label = 0)
+      : final_label_(final_label) {}
+
+  ToArc operator()(const FromArc &arc) const {
+    // Super-final arc.
+    if (arc.nextstate == kNoStateId && arc.weight != Weight::Zero()) {
+      return ToArc(final_label_, final_label_, arc.weight, kNoStateId);
+    } else {
+      return arc;
+    }
+  }
 
   constexpr MapFinalAction FinalAction() const {
     return MAP_REQUIRE_SUPERFINAL;
@@ -734,8 +761,16 @@ class SuperFinalMapper {
   }
 
   uint64 Properties(uint64 props) const {
-    return props & kAddSuperFinalProperties;
+    if (final_label_ == 0) {
+      return props & kAddSuperFinalProperties;
+    } else {
+      return props & kAddSuperFinalProperties & kILabelInvariantProperties &
+             kOLabelInvariantProperties;
+    }
   }
+
+ private:
+  const Label final_label_;
 };
 
 // Mapper that leaves labels and nextstate unchanged and constructs a new weight
@@ -752,10 +787,10 @@ class WeightConvertMapper {
   using FromWeight = typename FromArc::Weight;
   using ToWeight = typename ToArc::Weight;
 
-  explicit WeightConvertMapper(const Converter &c = Converter())
+  constexpr explicit WeightConvertMapper(const Converter &c = Converter())
       : convert_weight_(c) {}
 
-  ToArc operator()(const FromArc &arc) const {
+  constexpr ToArc operator()(const FromArc &arc) const {
     return ToArc(arc.ilabel, arc.olabel, convert_weight_(arc.weight),
                  arc.nextstate);
   }
@@ -770,10 +805,10 @@ class WeightConvertMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const { return props; }
+  constexpr uint64 Properties(uint64 props) const { return props; }
 
  private:
-  Converter convert_weight_;
+  const Converter convert_weight_;
 };
 
 // Non-precision-changing weight conversions; consider using more efficient
@@ -808,14 +843,14 @@ class ToGallicMapper {
     // Super-final arc.
     if (arc.nextstate == kNoStateId && arc.weight != AW::Zero()) {
       return ToArc(0, 0, GW(SW::One(), arc.weight), kNoStateId);
-    // Super-non-final arc.
+      // Super-non-final arc.
     } else if (arc.nextstate == kNoStateId) {
       return ToArc(0, 0, GW::Zero(), kNoStateId);
-    // Epsilon label.
+      // Epsilon label.
     } else if (arc.olabel == 0) {
       return ToArc(arc.ilabel, arc.ilabel, GW(SW::One(), arc.weight),
                    arc.nextstate);
-    // Regular label.
+      // Regular label.
     } else {
       return ToArc(arc.ilabel, arc.ilabel, GW(SW(arc.olabel), arc.weight),
                    arc.nextstate);
@@ -857,7 +892,7 @@ class FromGallicMapper {
       return A(arc.ilabel, 0, AW::Zero(), kNoStateId);
     }
     Label l = kNoLabel;
-    AW weight;
+    AW weight = AW::Zero();
     if (!Extract(arc.weight, &weight, &l) || arc.ilabel != arc.olabel) {
       FSTERROR() << "FromGallicMapper: Unrepresentable weight: " << arc.weight
                  << " for arc with ilabel = " << arc.ilabel
@@ -941,9 +976,9 @@ class GallicToNewSymbolsMapper {
     fst_->DeleteStates();
     state_ = fst_->AddState();
     fst_->SetStart(state_);
-    fst_->SetFinal(state_, AW::One());
+    fst_->SetFinal(state_);
     if (osymbols_) {
-      string name = osymbols_->Name() + "_from_gallic";
+      std::string name = osymbols_->Name() + "_from_gallic";
       fst_->SetInputSymbols(new SymbolTable(name));
       isymbols_ = fst_->MutableInputSymbols();
       const int64 zero = 0;
@@ -964,7 +999,7 @@ class GallicToNewSymbolsMapper {
     if (w1.Size() == 0) {
       l = 0;
     } else {
-      auto insert_result = map_.insert(std::make_pair(w1, kNoLabel));
+      auto insert_result = map_.emplace(w1, kNoLabel);
       if (!insert_result.second) {
         l = insert_result.first->second;
       } else {
@@ -972,11 +1007,11 @@ class GallicToNewSymbolsMapper {
         insert_result.first->second = l;
         StringWeightIterator<SW> iter1(w1);
         StateId n;
-        string s;
+        std::string s;
         for (size_t i = 0, p = state_; i < w1.Size();
              ++i, iter1.Next(), p = n) {
           n = i == w1.Size() - 1 ? state_ : fst_->AddState();
-          fst_->AddArc(p, ToArc(i ? 0 : l, iter1.Value(), AW::One(), n));
+          fst_->AddArc(p, ToArc(i ? 0 : l, iter1.Value(), n));
           if (isymbols_) {
             if (i) s = s + "_";
             s = s + osymbols_->Find(iter1.Value());
@@ -1026,6 +1061,9 @@ class GallicToNewSymbolsMapper {
   mutable bool error_;
 };
 
+// TODO(kbg): Add common base class for those mappers which do nothing except
+// mutate their weights.
+
 // Mapper to add a constant to all weights.
 template <class A>
 class PlusMapper {
@@ -1034,7 +1072,7 @@ class PlusMapper {
   using ToArc = A;
   using Weight = typename FromArc::Weight;
 
-  explicit PlusMapper(Weight weight) : weight_(std::move(weight)) {}
+  constexpr explicit PlusMapper(Weight weight) : weight_(std::move(weight)) {}
 
   ToArc operator()(const FromArc &arc) const {
     if (arc.weight == Weight::Zero()) return arc;
@@ -1052,7 +1090,7 @@ class PlusMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const {
+  constexpr uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
   }
 
@@ -1068,7 +1106,7 @@ class TimesMapper {
   using ToArc = A;
   using Weight = typename FromArc::Weight;
 
-  explicit TimesMapper(Weight weight) : weight_(std::move(weight)) {}
+  constexpr explicit TimesMapper(Weight weight) : weight_(std::move(weight)) {}
 
   ToArc operator()(const FromArc &arc) const {
     if (arc.weight == Weight::Zero()) return arc;
@@ -1086,7 +1124,7 @@ class TimesMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const {
+  constexpr uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
   }
 
@@ -1094,7 +1132,11 @@ class TimesMapper {
   const Weight weight_;
 };
 
-// Mapper to take all arc-weights to a fixed power.
+// Mapper to take all weights to a constant power. The power argument is stored
+// as a double, so if there is a floating-point power implementation for this
+// weight type, it will take precedence. Otherwise, the power argument's 53 bits
+// of integer precision will be implicitly converted to a size_t and the default
+// power implementation (iterated multiplication) will be used instead.
 template <class A>
 class PowerMapper {
  public:
@@ -1102,25 +1144,29 @@ class PowerMapper {
   using ToArc = A;
   using Weight = typename FromArc::Weight;
 
-  explicit PowerMapper(size_t power) : power_(power) {}
+  explicit PowerMapper(double power) : power_(power) {}
 
   ToArc operator()(const FromArc &arc) const {
     return ToArc(arc.ilabel, arc.olabel, Power(arc.weight, power_),
                  arc.nextstate);
   }
 
-  MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
+  constexpr MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
 
-  MapSymbolsAction InputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
+  constexpr MapSymbolsAction InputSymbolsAction() const {
+    return MAP_COPY_SYMBOLS;
+  }
 
-  MapSymbolsAction OutputSymbolsAction() const { return MAP_COPY_SYMBOLS; }
+  constexpr MapSymbolsAction OutputSymbolsAction() const {
+    return MAP_COPY_SYMBOLS;
+  }
 
-  uint64 Properties(uint64 props) const {
+  constexpr uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
   }
 
  private:
-  size_t power_;
+  const double power_;
 };
 
 // Mapper to reciprocate all non-Zero() weights.
@@ -1147,7 +1193,7 @@ class InvertWeightMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const {
+  constexpr uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
   }
 };
@@ -1162,10 +1208,10 @@ class RmWeightMapper {
   using ToWeight = typename ToArc::Weight;
 
   ToArc operator()(const FromArc &arc) const {
-    return ToArc(arc.ilabel, arc.olabel,
-                 arc.weight != FromWeight::Zero() ?
-                 ToWeight::One() : ToWeight::Zero(),
-                 arc.nextstate);
+    return ToArc(
+        arc.ilabel, arc.olabel,
+        arc.weight != FromWeight::Zero() ? ToWeight::One() : ToWeight::Zero(),
+        arc.nextstate);
   }
 
   constexpr MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
@@ -1178,7 +1224,7 @@ class RmWeightMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const {
+  constexpr uint64 Properties(uint64 props) const {
     return (props & kWeightInvariantProperties) | kUnweighted;
   }
 };
@@ -1211,7 +1257,7 @@ class QuantizeMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const {
+  constexpr uint64 Properties(uint64 props) const {
     return props & kWeightInvariantProperties;
   }
 
@@ -1232,7 +1278,7 @@ class ReverseWeightMapper {
   using FromArc = A;
   using ToArc = B;
 
-  ToArc operator()(const FromArc &arc) const {
+  constexpr ToArc operator()(const FromArc &arc) const {
     return ToArc(arc.ilabel, arc.olabel, arc.weight.Reverse(), arc.nextstate);
   }
 
@@ -1246,9 +1292,9 @@ class ReverseWeightMapper {
     return MAP_COPY_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) const { return props; }
+  constexpr uint64 Properties(uint64 props) const { return props; }
 };
 
 }  // namespace fst
 
-#endif  // FST_LIB_ARC_MAP_H_
+#endif  // FST_ARC_MAP_H_
