@@ -3,18 +3,19 @@
 //
 // Class to complement an FST.
 
-#ifndef FST_LIB_COMPLEMENT_H_
-#define FST_LIB_COMPLEMENT_H_
+#ifndef FST_COMPLEMENT_H_
+#define FST_COMPLEMENT_H_
 
 #include <algorithm>
 #include <string>
 #include <vector>
+
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/fst.h>
 #include <fst/test-properties.h>
 
-
 namespace fst {
 
 template <class Arc>
@@ -49,7 +50,7 @@ class ComplementFstImpl : public FstImpl<A> {
 
   explicit ComplementFstImpl(const Fst<Arc> &fst) : fst_(fst.Copy()) {
     SetType("complement");
-    uint64 props = fst.Properties(kILabelSorted, false);
+    const auto props = fst.Properties(kILabelSorted, false);
     SetProperties(ComplementProperties(props), kCopyProperties);
     SetInputSymbols(fst.InputSymbols());
     SetOutputSymbols(fst.OutputSymbols());
@@ -65,7 +66,7 @@ class ComplementFstImpl : public FstImpl<A> {
 
   StateId Start() const {
     if (Properties(kError)) return kNoStateId;
-    auto start = fst_->Start();
+    const auto start = fst_->Start();
     return start != kNoStateId ? start + 1 : 0;
   }
 
@@ -135,12 +136,12 @@ class ComplementFst : public ImplToFst<internal::ComplementFstImpl<A>> {
   }
 
   // See Fst<>::Copy() for doc.
-  ComplementFst(const ComplementFst<Arc> &fst, bool safe = false)
+  ComplementFst(const ComplementFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
   // Gets a copy of this FST. See Fst<>::Copy() for further doc.
-  ComplementFst<Arc> *Copy(bool safe = false) const override {
-    return new ComplementFst<Arc>(*this, safe);
+  ComplementFst *Copy(bool safe = false) const override {
+    return new ComplementFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -198,7 +199,8 @@ class ArcIterator<ComplementFst<Arc>> : public ArcIteratorBase<Arc> {
 
   ArcIterator(const ComplementFst<Arc> &fst, StateId s) : s_(s), pos_(0) {
     if (s_ != 0) {
-      aiter_.reset(new ArcIterator<Fst<Arc>>(*fst.GetImpl()->fst_, s - 1));
+      aiter_ =
+          fst::make_unique<ArcIterator<Fst<Arc>>>(*fst.GetImpl()->fst_, s - 1);
     }
   }
 
@@ -246,9 +248,9 @@ class ArcIterator<ComplementFst<Arc>> : public ArcIteratorBase<Arc> {
     pos_ = a;
   }
 
-  uint32 Flags() const final { return kArcValueFlags; }
+  uint8 Flags() const final { return kArcValueFlags; }
 
-  void SetFlags(uint32, uint32) final {}
+  void SetFlags(uint8, uint8) final {}
 
  private:
   std::unique_ptr<ArcIterator<Fst<Arc>>> aiter_;
@@ -264,8 +266,8 @@ inline void ComplementFst<Arc>::InitStateIterator(
 }
 
 template <class Arc>
-inline void ComplementFst<Arc>::InitArcIterator(StateId s,
-    ArcIteratorData<Arc> *data) const {
+inline void ComplementFst<Arc>::InitArcIterator(
+    StateId s, ArcIteratorData<Arc> *data) const {
   data->base = new ArcIterator<ComplementFst<Arc>>(*this, s);
 }
 
@@ -274,4 +276,4 @@ using StdComplementFst = ComplementFst<StdArc>;
 
 }  // namespace fst
 
-#endif  // FST_LIB_COMPLEMENT_H_
+#endif  // FST_COMPLEMENT_H_
