@@ -3,12 +3,15 @@
 //
 // Functions and classes to compute the concatenation of two FSTs.
 
-#ifndef FST_LIB_CONCAT_H_
-#define FST_LIB_CONCAT_H_
+#ifndef FST_CONCAT_H_
+#define FST_CONCAT_H_
 
 #include <algorithm>
 #include <vector>
 
+#include <fst/types.h>
+
+#include <fst/expanded-fst.h>
 #include <fst/mutable-fst.h>
 #include <fst/rational.h>
 
@@ -30,7 +33,6 @@ namespace fst {
 // FST.
 template <class Arc>
 void Concat(MutableFst<Arc> *fst1, const Fst<Arc> &fst2) {
-  using Label = typename Arc::Label;
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
   // Checks that the symbol table are compatible.
@@ -78,6 +80,13 @@ void Concat(MutableFst<Arc> *fst1, const Fst<Arc> &fst2) {
   }
 }
 
+// Computes the concatentation of two FSTs. This version modifies its
+// RationalFst input (in first position).
+template <class Arc>
+void Concat(RationalFst<Arc> *fst1, const Fst<Arc> &fst2) {
+  fst1->GetMutableImpl()->AddConcat(fst2, true);
+}
+
 // Computes the concatentation of two FSTs.  This version modifies its
 // MutableFst argument (in second position).
 //
@@ -90,8 +99,6 @@ void Concat(MutableFst<Arc> *fst1, const Fst<Arc> &fst2) {
 // FST.
 template <class Arc>
 void Concat(const Fst<Arc> &fst1, MutableFst<Arc> *fst2) {
-  using Label = typename Arc::Label;
-  using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
   // Checks that the symbol table are compatible.
   if (!CompatSymbols(fst1.InputSymbols(), fst2->InputSymbols()) ||
@@ -137,11 +144,12 @@ void Concat(const Fst<Arc> &fst1, MutableFst<Arc> *fst2) {
   }
 }
 
-// Computes the concatentation of two FSTs. This version modifies its
-// RationalFst input (in first position).
+// Same as the above but can handle arbitrarily many left-hand-side FSTs,
+// preallocating the states.
 template <class Arc>
-void Concat(RationalFst<Arc> *fst1, const Fst<Arc> &fst2) {
-  fst1->GetMutableImpl()->AddConcat(fst2, true);
+void Concat(const std::vector<const Fst<Arc> *> &fsts1, MutableFst<Arc> *fst2) {
+  fst2->ReserveStates(CountStates(fsts1) + fst2->NumStates());
+  for (const auto *fst1 : fsts1) Concat(*fst1, fst2);
 }
 
 // Computes the concatentation of two FSTs. This version modifies its
@@ -184,12 +192,12 @@ class ConcatFst : public RationalFst<A> {
   }
 
   // See Fst<>::Copy() for doc.
-  ConcatFst(const ConcatFst<Arc> &fst, bool safe = false)
+  ConcatFst(const ConcatFst &fst, bool safe = false)
       : RationalFst<Arc>(fst, safe) {}
 
   // Get a copy of this ConcatFst. See Fst<>::Copy() for further doc.
-  ConcatFst<Arc> *Copy(bool safe = false) const override {
-    return new ConcatFst<Arc>(*this, safe);
+  ConcatFst *Copy(bool safe = false) const override {
+    return new ConcatFst(*this, safe);
   }
 
  private:
@@ -220,4 +228,4 @@ using StdConcatFst = ConcatFst<StdArc>;
 
 }  // namespace fst
 
-#endif  // FST_LIB_CONCAT_H_
+#endif  // FST_CONCAT_H_
