@@ -3,16 +3,18 @@
 //
 // Functions and classes to determinize an FST.
 
-#ifndef FST_LIB_DETERMINIZE_H_
-#define FST_LIB_DETERMINIZE_H_
+#ifndef FST_DETERMINIZE_H_
+#define FST_DETERMINIZE_H_
 
 #include <algorithm>
 #include <climits>
 #include <forward_list>
 #include <map>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/arc-map.h>
@@ -23,7 +25,6 @@
 #include <fst/prune.h>
 #include <fst/test-properties.h>
 
-
 namespace fst {
 
 // Common divisors are used in determinization to compute transition weights.
@@ -230,14 +231,14 @@ class DefaultDeterminizeFilter {
   // Filters transition, possibly modifying label map. Returns true if arc is
   // added to the label map.
   bool FilterArc(const Arc &arc, const Element &src_element,
-                 const Element &dest_element, LabelMap *label_map) const {
+                 Element &&dest_element, LabelMap *label_map) const {
     // Adds element to unique state tuple for arc label.
     auto &det_arc = (*label_map)[arc.ilabel];
     if (det_arc.label == kNoLabel) {
       det_arc = internal::DeterminizeArc<StateTuple>(arc);
       det_arc.dest_tuple->filter_state = FilterState(0);
     }
-    det_arc.dest_tuple->subset.push_front(dest_element);
+    det_arc.dest_tuple->subset.push_front(std::move(dest_element));
     return true;
   }
 
@@ -460,7 +461,7 @@ class DeterminizeFstImplBase : public CacheImpl<Arc> {
     SetOutputSymbols(fst.OutputSymbols());
   }
 
-  DeterminizeFstImplBase(const DeterminizeFstImplBase<Arc> &impl)
+  DeterminizeFstImplBase(const DeterminizeFstImplBase &impl)
       : CacheImpl<Arc>(impl), fst_(impl.fst_->Copy(true)) {
     SetType("determinize");
     SetProperties(impl.Properties(), kCopyProperties);
@@ -468,7 +469,7 @@ class DeterminizeFstImplBase : public CacheImpl<Arc> {
     SetOutputSymbols(impl.OutputSymbols());
   }
 
-  virtual DeterminizeFstImplBase<Arc> *Copy() const = 0;
+  virtual DeterminizeFstImplBase *Copy() const = 0;
 
   StateId Start() {
     if (!HasStart()) {
@@ -555,8 +556,7 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
     if (out_dist_) out_dist_->clear();
   }
 
-  DeterminizeFsaImpl(
-      const DeterminizeFsaImpl<Arc, CommonDivisor, Filter, StateTable> &impl)
+  DeterminizeFsaImpl(const DeterminizeFsaImpl &impl)
       : DeterminizeFstImplBase<Arc>(impl),
         delta_(impl.delta_),
         in_dist_(nullptr),
@@ -569,10 +569,8 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
     }
   }
 
-  DeterminizeFsaImpl<Arc, CommonDivisor, Filter, StateTable> *Copy()
-      const override {
-    return new DeterminizeFsaImpl<Arc, CommonDivisor, Filter, StateTable>(
-        *this);
+  DeterminizeFsaImpl *Copy() const override {
+    return new DeterminizeFsaImpl(*this);
   }
 
   uint64 Properties() const override { return Properties(kFstProperties); }
@@ -588,9 +586,8 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
   StateId ComputeStart() override {
     const auto s = GetFst().Start();
     if (s == kNoStateId) return kNoStateId;
-    const Element element(s, Weight::One());
     auto *tuple = new StateTuple;
-    tuple->subset.push_front(element);
+    tuple->subset.emplace_front(s, Weight::One());
     tuple->filter_state = filter_->Start();
     return FindState(tuple);
   }
@@ -638,7 +635,7 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
     LabelMap label_map;
     GetLabelMap(s, &label_map);
     for (auto it = label_map.begin(); it != label_map.end(); ++it) {
-      AddArc(s, it->second);
+      AddArc(s, std::move(it->second));
     }
     SetArcs(s);
   }
@@ -657,9 +654,10 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
       for (ArcIterator<Fst<Arc>> aiter(GetFst(), src_element.state_id);
            !aiter.Done(); aiter.Next()) {
         const auto &arc = aiter.Value();
-        const Element dest_element(arc.nextstate,
-                                   Times(src_element.weight, arc.weight));
-        filter_->FilterArc(arc, src_element, dest_element, label_map);
+        Element dest_element(arc.nextstate,
+                             Times(src_element.weight, arc.weight));
+        filter_->FilterArc(arc, src_element, std::move(dest_element),
+                           label_map);
       }
     }
     for (auto it = label_map->begin(); it != label_map->end(); ++it) {
@@ -703,22 +701,25 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
 
   // Adds an arc from state S to the destination state associated with state
   // tuple in det_arc as created by GetLabelMap.
-  void AddArc(StateId s, const DetArc &det_arc) {
-    const Arc arc(det_arc.label, det_arc.label, det_arc.weight,
-                  FindState(det_arc.dest_tuple));
-    CacheImpl<Arc>::PushArc(s, arc);
+  void AddArc(StateId s, DetArc &&det_arc) {
+    CacheImpl<Arc>::EmplaceArc(s, det_arc.label, det_arc.label,
+                               std::move(det_arc.weight),
+                               FindState(det_arc.dest_tuple));
   }
 
   float delta_;                         // Quantization delta for weights.
   const std::vector<Weight> *in_dist_;  // Distance to final NFA states.
   std::vector<Weight> *out_dist_;       // Distance to final DFA states.
 
-  // FIXME(kbg): Ought to be static const?
-  CommonDivisor common_divisor_;
+  static const CommonDivisor common_divisor_;
   std::unique_ptr<Filter> filter_;
   std::unique_ptr<StateTable> state_table_;
 };
 
+template <class Arc, class CommonDivisor, class Filter, class StateTable>
+const CommonDivisor DeterminizeFsaImpl<Arc, CommonDivisor, Filter,
+                                       StateTable>::common_divisor_{};
+
 // Implementation of delayed determinization for transducers. Transducer
 // determinization is implemented by mapping the input to the Gallic semiring as
 // an acceptor whose weights contain the output strings and using acceptor
@@ -765,8 +766,7 @@ class DeterminizeFstImpl : public DeterminizeFstImplBase<Arc> {
     Init(GetFst(), opts.filter);
   }
 
-  DeterminizeFstImpl(
-      const DeterminizeFstImpl<Arc, G, CommonDivisor, Filter, StateTable> &impl)
+  DeterminizeFstImpl(const DeterminizeFstImpl &impl)
       : DeterminizeFstImplBase<Arc>(impl),
         delta_(impl.delta_),
         subsequential_label_(impl.subsequential_label_),
@@ -774,10 +774,8 @@ class DeterminizeFstImpl : public DeterminizeFstImplBase<Arc> {
     Init(GetFst(), nullptr);
   }
 
-  DeterminizeFstImpl<Arc, G, CommonDivisor, Filter, StateTable> *Copy()
-      const override {
-    return new DeterminizeFstImpl<Arc, G, CommonDivisor, Filter, StateTable>(
-        *this);
+  DeterminizeFstImpl *Copy() const override {
+    return new DeterminizeFstImpl(*this);
   }
 
   uint64 Properties() const override { return Properties(kFstProperties); }
@@ -863,20 +861,26 @@ class DeterminizeFst : public ImplToFst<internal::DeterminizeFstImplBase<A>> {
       : ImplToFst<Impl>(CreateImpl(fst)) {}
 
   template <class CommonDivisor, class Filter, class StateTable>
-  DeterminizeFst(
+  explicit DeterminizeFst(
       const Fst<Arc> &fst,
-      const DeterminizeFstOptions<Arc, CommonDivisor, Filter, StateTable> &opts)
+      const DeterminizeFstOptions<Arc, CommonDivisor, Filter, StateTable>
+          &opts =
+              DeterminizeFstOptions<Arc, CommonDivisor, Filter, StateTable>())
       : ImplToFst<Impl>(CreateImpl(fst, opts)) {}
 
-  // This acceptor-only version additionally computes the distance to
-  // final states in the output if provided with those distances for the
-  // input. Useful for e.g., computing the k-shortest unique paths.
-  template <class D, class F, class T>
-  DeterminizeFst(const Fst<A> &fst, const std::vector<Weight> *in_dist,
-                 std::vector<Weight> *out_dist,
-                 const DeterminizeFstOptions<A, D, F, T> &opts)
+  // This acceptor-only version additionally computes the distance to final
+  // states in the output if provided with those distances for the input; this
+  // is useful for e.g., computing the k-shortest unique paths.
+  template <class CommonDivisor, class Filter, class StateTable>
+  DeterminizeFst(
+      const Fst<Arc> &fst, const std::vector<Weight> *in_dist,
+      std::vector<Weight> *out_dist,
+      const DeterminizeFstOptions<Arc, CommonDivisor, Filter, StateTable>
+          &opts =
+              DeterminizeFstOptions<Arc, CommonDivisor, Filter, StateTable>())
       : ImplToFst<Impl>(
-            std::make_shared<internal::DeterminizeFsaImpl<A, D, F, T>>(
+            std::make_shared<internal::DeterminizeFsaImpl<Arc, CommonDivisor,
+                                                          Filter, StateTable>>(
                 fst, in_dist, out_dist, opts)) {
     if (!fst.Properties(kAcceptor, true)) {
       FSTERROR() << "DeterminizeFst: "
@@ -886,13 +890,13 @@ class DeterminizeFst : public ImplToFst<internal::DeterminizeFstImplBase<A>> {
   }
 
   // See Fst<>::Copy() for doc.
-  DeterminizeFst(const DeterminizeFst<Arc> &fst, bool safe = false)
+  DeterminizeFst(const DeterminizeFst &fst, bool safe = false)
       : ImplToFst<Impl>(safe ? std::shared_ptr<Impl>(fst.GetImpl()->Copy())
                              : fst.GetSharedImpl()) {}
 
   // Get a copy of this DeterminizeFst. See Fst<>::Copy() for further doc.
-  DeterminizeFst<Arc> *Copy(bool safe = false) const override {
-    return new DeterminizeFst<Arc>(*this, safe);
+  DeterminizeFst *Copy(bool safe = false) const override {
+    return new DeterminizeFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -955,22 +959,23 @@ namespace internal {
 template <class A, GallicType G, class D, class F, class T>
 void DeterminizeFstImpl<A, G, D, F, T>::Init(const Fst<A> &fst, F *filter) {
   // Mapper to an acceptor.
-  ToFst to_fst(fst, ToMapper());
+  const ToFst to_fst(fst, ToMapper());
   auto *to_filter = filter ? new ToFilter(to_fst, filter) : nullptr;
   // This recursive call terminates since it is to a (non-recursive)
   // different constructor.
-  CacheOptions copts(GetCacheGc(), GetCacheLimit());
-  DeterminizeFstOptions<ToArc, ToCommonDivisor, ToFilter, ToStateTable> dopts(
-      copts, delta_, 0, DETERMINIZE_FUNCTIONAL, false, to_filter);
+  const CacheOptions copts(GetCacheGc(), GetCacheLimit());
+  const DeterminizeFstOptions<ToArc, ToCommonDivisor, ToFilter, ToStateTable>
+      dopts(copts, delta_, 0, DETERMINIZE_FUNCTIONAL, false, to_filter);
   // Uses acceptor-only constructor to avoid template recursion.
-  DeterminizeFst<ToArc> det_fsa(to_fst, nullptr, nullptr, dopts);
+  const DeterminizeFst<ToArc> det_fsa(to_fst, nullptr, nullptr, dopts);
   // Mapper back to transducer.
-  FactorWeightOptions<ToArc> fopts(
+  const FactorWeightOptions<ToArc> fopts(
       CacheOptions(true, 0), delta_, kFactorFinalWeights, subsequential_label_,
       subsequential_label_, increment_subsequential_label_,
       increment_subsequential_label_);
-  FactorWeightFst<ToArc, FactorIterator> factored_fst(det_fsa, fopts);
-  from_fst_.reset(new FromFst(factored_fst, FromMapper(subsequential_label_)));
+  const FactorWeightFst<ToArc, FactorIterator> factored_fst(det_fsa, fopts);
+  from_fst_ = fst::make_unique<FromFst>(factored_fst,
+                                         FromMapper(subsequential_label_));
 }
 
 }  // namespace internal
@@ -1038,7 +1043,7 @@ struct DeterminizeOptions {
 // Determinizes a weighted transducer. This version writes the
 // determinized Fst to an output MutableFst. The result will be an
 // equivalent FST that has the property that no state has two
-// transitions with the same input label.  For this algorithm, epsilon
+// transitions with the same input label. For this algorithm, epsilon
 // transitions are treated as regular symbols (cf. RmEpsilon).
 //
 // The transducer must be functional. The weights must be (weakly)
@@ -1083,4 +1088,4 @@ void Determinize(
 
 }  // namespace fst
 
-#endif  // FST_LIB_DETERMINIZE_H_
+#endif  // FST_DETERMINIZE_H_
