@@ -3,8 +3,8 @@
 //
 // Functions and classes to disambiguate an FST.
 
-#ifndef FST_LIB_DISAMBIGUATE_H_
-#define FST_LIB_DISAMBIGUATE_H_
+#ifndef FST_DISAMBIGUATE_H_
+#define FST_DISAMBIGUATE_H_
 
 #include <list>
 #include <map>
@@ -12,6 +12,8 @@
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/arcsort.h>
 #include <fst/compose.h>
 #include <fst/connect.h>
@@ -24,7 +26,6 @@
 #include <fst/union-find.h>
 #include <fst/verify.h>
 
-
 namespace fst {
 
 template <class Arc>
@@ -174,7 +175,7 @@ void RelationDeterminizeFilter<Arc, Relation>::InitLabelMap(
     if (arc.ilabel == label && arc.nextstate == nextstate) continue;
     DeterminizeArc<StateTuple> det_arc(arc);
     det_arc.dest_tuple->filter_state = FilterState(arc.nextstate);
-    label_map->insert(std::make_pair(arc.ilabel, det_arc));
+    label_map->emplace(arc.ilabel, det_arc);
     label = arc.ilabel;
     nextstate = arc.nextstate;
   }
@@ -267,7 +268,7 @@ class Disambiguator {
       // Ensures composition is between acceptors.
       const bool trans = ifst.Properties(kNotAcceptor, true);
       const auto *fsa =
-          trans ? new ProjectFst<Arc>(ifst, PROJECT_INPUT) : &ifst;
+          trans ? new ProjectFst<Arc>(ifst, ProjectType::INPUT) : &ifst;
       opts.state_table = new StateTable(*fsa, *fsa);
       const ComposeFst<Arc> cfst(*fsa, *fsa, opts);
       std::vector<bool> coaccess;
@@ -393,7 +394,8 @@ void Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
                                                    AnyArcFilter<Arc>(),
                                                    &odistance);
       Prune(dfst, ofst, popts);
-      } else */ {
+      } else */
+    {
       *ofst = DeterminizeFst<Arc>(ifst, nopts);
       Prune(ofst, opts.weight_threshold, opts.state_threshold);
     }
@@ -406,7 +408,7 @@ void Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
 template <class Arc>
 void Disambiguator<Arc>::FindAmbiguities(const ExpandedFst<Arc> &fst) {
   if (fst.Start() == kNoStateId) return;
-  candidates_.reset(new ArcIdMap(ArcIdCompare(head_)));
+  candidates_ = fst::make_unique<ArcIdMap>(ArcIdCompare(head_));
   const auto start_pr = std::make_pair(fst.Start(), fst.Start());
   coreachable_.insert(start_pr);
   queue_.push_back(start_pr);
@@ -447,7 +449,8 @@ void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
           if (spr.first != spr.second &&
               head_[spr.first] == head_[spr.second]) {
             if (!merge_) {
-              merge_.reset(new UnionFind<StateId>(fst.NumStates(), kNoStateId));
+              merge_ = fst::make_unique<UnionFind<StateId>>(fst.NumStates(),
+                                                             kNoStateId);
               merge_->MakeAllSet(fst.NumStates());
             }
             merge_->Union(spr.first, spr.second);
@@ -561,4 +564,4 @@ void Disambiguate(
 
 }  // namespace fst
 
-#endif  // FST_LIB_DISAMBIGUATE_H_
+#endif  // FST_DISAMBIGUATE_H_
