@@ -3,14 +3,15 @@
 //
 // Functions and classes to determine the equivalence of two FSTs.
 
-#ifndef FST_LIB_EQUIVALENT_H_
-#define FST_LIB_EQUIVALENT_H_
+#ifndef FST_EQUIVALENT_H_
+#define FST_EQUIVALENT_H_
 
 #include <algorithm>
 #include <deque>
 #include <unordered_map>
-#include <utility>
 #include <vector>
+
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/encode.h>
@@ -112,7 +113,7 @@ constexpr
 //        by 4 by all practical purposes.
 template <class Arc>
 bool Equivalent(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
-                double delta = kDelta, bool *error = nullptr) {
+                float delta = kDelta, bool *error = nullptr) {
   using Weight = typename Arc::Weight;
   if (error) *error = false;
   // Check that the symbol table are compatible.
@@ -174,7 +175,7 @@ bool Equivalent(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
   // Main loop: explores the two acceptors in a breadth-first manner, updating
   // the equivalence relation on the statesets. Loop invariant: each block of
   // the states contains either final states only or non-final states only.
-  for (q.push_back(std::make_pair(s1, s2)); ret && !q.empty(); q.pop_front()) {
+  for (q.emplace_back(s1, s2); ret && !q.empty(); q.pop_front()) {
     s1 = q.front().first;
     s2 = q.front().second;
     // Representatives of the equivalence classes of s1/s2.
@@ -227,4 +228,4 @@ bool Equivalent(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
 
 }  // namespace fst
 
-#endif  // FST_LIB_EQUIVALENT_H_
+#endif  // FST_EQUIVALENT_H_
