@@ -15,13 +15,13 @@ namespace fst {
 template <class Var>
 class Elias {
  public:
-  // Gamma encoding is a subroutine for Delta encoding
+  // A subroutine for Delta encoding.
   static void GammaEncode(const Var &input, std::vector<bool> *code);
 
-  // Elias Delta encoding for a single integer
+  // Elias Delta encoding for a single integer.
   static void DeltaEncode(const Var &input, std::vector<bool> *code);
 
-  // Batch decoding of a set of integers
+  // Batch decoding of a set of integers.
   static void BatchDecode(const std::vector<bool> &input,
                           std::vector<Var> *output);
 };
@@ -34,13 +34,13 @@ void Elias<Var>::GammaEncode(const Var &input, std::vector<bool> *code) {
     reverse_code.push(input_copy % 2);
     input_copy = input_copy / 2;
   }
-  for (Var auxvar = 0; auxvar < reverse_code.size() - 1; auxvar++)
-    code->push_back(0);
-  while (reverse_code.empty() != 1) {
+  code->resize(reverse_code.size() - 1, false);
+  while (!reverse_code.empty()) {
     code->push_back(reverse_code.top());
     reverse_code.pop();
   }
 }
+
 template <class Var>
 void Elias<Var>::DeltaEncode(const Var &input, std::vector<bool> *code) {
   Var input_copy = input + 1;
@@ -53,7 +53,7 @@ void Elias<Var>::DeltaEncode(const Var &input, std::vector<bool> *code) {
   }
   GammaEncode(auxvar, code);
   reverse_remainder.pop();
-  while (reverse_remainder.empty() != 1) {
+  while (!reverse_remainder.empty()) {
     code->push_back(reverse_remainder.top());
     reverse_remainder.pop();
   }
@@ -66,27 +66,26 @@ void Elias<Var>::BatchDecode(const std::vector<bool> &input,
   Var remainder_bits = 0;
   Var current_word = 1;
   Var value = 1;
-  std::vector<bool>::const_iterator it = input.begin();
-  while (it != input.end()) {
+  for (auto it = input.cbegin(); it != input.cend();) {
     lead_zeros = 0;
     remainder_bits = 0;
     current_word = 1;
     value = 1;
-    while (*it != 1) {
-      it++;
-      lead_zeros++;
+    while (!*it) {
+      ++it;
+      ++lead_zeros;
     }
-    it++;
+    ++it;
     while (lead_zeros > 0) {
-      lead_zeros--;
+      --lead_zeros;
       current_word = 2 * current_word + *it;
-      it++;
+      ++it;
     }
-    current_word--;
+    --current_word;
     while (current_word > 0) {
       value = 2 * value + *it;
-      current_word--;
-      it++;
+      --current_word;
+      ++it;
     }
     output->push_back(value - 1);
   }
