@@ -7,6 +7,7 @@
 #define FST_EXTENSIONS_FAR_CREATE_H_
 
 #include <libgen.h>
+
 #include <sstream>
 #include <string>
 #include <vector>
@@ -16,16 +17,17 @@
 namespace fst {
 
 template <class Arc>
-void FarCreate(const std::vector<string> &in_fnames, const string &out_fname,
-               const int32 generate_keys, const FarType &far_type,
-               const string &key_prefix, const string &key_suffix) {
+void FarCreate(const std::vector<std::string> &in_sources,
+               const std::string &out_source, const int32 generate_keys,
+               const FarType &far_type, const std::string &key_prefix,
+               const std::string &key_suffix) {
   std::unique_ptr<FarWriter<Arc>> far_writer(
-      FarWriter<Arc>::Create(out_fname, far_type));
+      FarWriter<Arc>::Create(out_source, far_type));
   if (!far_writer) return;
-  for (size_t i = 0; i < in_fnames.size(); ++i) {
-    std::unique_ptr<Fst<Arc>> ifst(Fst<Arc>::Read(in_fnames[i]));
+  for (size_t i = 0; i < in_sources.size(); ++i) {
+    std::unique_ptr<Fst<Arc>> ifst(Fst<Arc>::Read(in_sources[i]));
     if (!ifst) return;
-    string key;
+    std::string key;
     if (generate_keys > 0) {
       std::ostringstream keybuf;
       keybuf.width(generate_keys);
@@ -33,10 +35,10 @@ void FarCreate(const std::vector<string> &in_fnames, const string &out_fname,
       keybuf << i + 1;
       key = keybuf.str();
     } else {
-      auto *filename = new char[in_fnames[i].size() + 1];
-      strcpy(filename, in_fnames[i].c_str());
-      key = basename(filename);
-      delete[] filename;
+      auto *source = new char[in_sources[i].size() + 1];
+      strcpy(source, in_sources[i].c_str());  // NOLINT
+      key = basename(source);
+      delete[] source;
     }
     far_writer->Add(key_prefix + key + key_suffix, *ifst);
   }
