@@ -31,28 +31,28 @@ static constexpr int32 kSTTableFileVersion = 1;
 template <class T, class Writer>
 class STTableWriter {
  public:
-  explicit STTableWriter(const string &filename)
-      : stream_(filename, std::ios_base::out | std::ios_base::binary),
+  explicit STTableWriter(const std::string &source)
+      : stream_(source, std::ios_base::out | std::ios_base::binary),
         error_(false) {
     WriteType(stream_, kSTTableMagicNumber);
     WriteType(stream_, kSTTableFileVersion);
     if (stream_.fail()) {
       FSTERROR() << "STTableWriter::STTableWriter: Error writing to file: "
-                 << filename;
+                 << source;
       error_ = true;
     }
   }
 
-  static STTableWriter<T, Writer> *Create(const string &filename) {
-    if (filename.empty()) {
+  static STTableWriter<T, Writer> *Create(const std::string &source) {
+    if (source.empty()) {
       LOG(ERROR) << "STTableWriter: Writing to standard out unsupported.";
       return nullptr;
     }
-    return new STTableWriter<T, Writer>(filename);
+    return new STTableWriter<T, Writer>(source);
   }
 
-  void Add(const string &key, const T &t) {
-    if (key == "") {
+  void Add(const std::string &key, const T &t) {
+    if (key.empty()) {
       FSTERROR() << "STTableWriter::Add: Key empty: " << key;
       error_ = true;
     } else if (key < last_key_) {
@@ -77,7 +77,7 @@ class STTableWriter {
   Writer entry_writer_;
   std::ofstream stream_;
   std::vector<int64> positions_;  // Position in file of each key-entry pair.
-  string last_key_;               // Last key.
+  std::string last_key_;          // Last key.
   bool error_;
 
   STTableWriter(const STTableWriter &) = delete;
@@ -94,28 +94,34 @@ class STTableWriter {
 template <class T, class Reader>
 class STTableReader {
  public:
-  explicit STTableReader(const std::vector<string> &filenames)
-      : sources_(filenames), error_(false) {
+  explicit STTableReader(const std::vector<std::string> &sources)
+      : sources_(sources), error_(false) {
     compare_.reset(new Compare(&keys_));
-    keys_.resize(filenames.size());
-    streams_.resize(filenames.size(), 0);
-    positions_.resize(filenames.size());
-    for (size_t i = 0; i < filenames.size(); ++i) {
+    keys_.resize(sources.size());
+    streams_.resize(sources.size(), nullptr);
+    positions_.resize(sources.size());
+    for (size_t i = 0; i < sources.size(); ++i) {
       streams_[i] = new std::ifstream(
-          filenames[i], std::ios_base::in | std::ios_base::binary);
+          sources[i], std::ios_base::in | std::ios_base::binary);
+      if (streams_[i]->fail()) {
+        FSTERROR() << "STTableReader::STTableReader: Error reading file: "
+                   << sources[i];
+        error_ = true;
+        return;
+      }
       int32 magic_number = 0;
       ReadType(*streams_[i], &magic_number);
       int32 file_version = 0;
       ReadType(*streams_[i], &file_version);
       if (magic_number != kSTTableMagicNumber) {
         FSTERROR() << "STTableReader::STTableReader: Wrong file type: "
-                   << filenames[i];
+                   << sources[i];
         error_ = true;
         return;
       }
       if (file_version != kSTTableFileVersion) {
         FSTERROR() << "STTableReader::STTableReader: Wrong file version: "
-                   << filenames[i];
+                   << sources[i];
         error_ = true;
         return;
       }
@@ -132,7 +138,7 @@ class STTableReader {
         streams_[i]->seekg(positions_[i][0]);
         if (streams_[i]->fail()) {
           FSTERROR() << "STTableReader::STTableReader: Error reading file: "
-                     << filenames[i];
+                     << sources[i];
           error_ = true;
           return;
         }
@@ -145,18 +151,19 @@ class STTableReader {
     for (auto &stream : streams_) delete stream;
   }
 
-  static STTableReader<T, Reader> *Open(const string &filename) {
-    if (filename.empty()) {
+  static STTableReader<T, Reader> *Open(const std::string &source) {
+    if (source.empty()) {
       LOG(ERROR) << "STTableReader: Operation not supported on standard input";
       return nullptr;
     }
-    std::vector<string> filenames;
-    filenames.push_back(filename);
-    return new STTableReader<T, Reader>(filenames);
+    std::vector<std::string> sources;
+    sources.push_back(source);
+    return new STTableReader<T, Reader>(sources);
   }
 
-  static STTableReader<T, Reader> *Open(const std::vector<string> &filenames) {
-    return new STTableReader<T, Reader>(filenames);
+  static STTableReader<T, Reader> *Open(
+      const std::vector<std::string> &sources) {
+    return new STTableReader<T, Reader>(sources);
   }
 
   void Reset() {
@@ -166,7 +173,7 @@ class STTableReader {
     MakeHeap();
   }
 
-  bool Find(const string &key) {
+  bool Find(const std::string &key) {
     if (error_) return false;
     for (size_t i = 0; i < streams_.size(); ++i) LowerBound(i, key);
     MakeHeap();
@@ -193,7 +200,7 @@ class STTableReader {
     if (!heap_.empty()) PopHeap();
   }
 
-  const string &GetKey() const { return keys_[current_]; }
+  const std::string &GetKey() const { return keys_[current_]; }
 
   const T *GetEntry() const { return entry_.get(); }
 
@@ -202,19 +209,19 @@ class STTableReader {
  private:
   // Comparison functor used to compare stream IDs in the heap.
   struct Compare {
-    explicit Compare(const std::vector<string> *keys) : keys(keys) {}
+    explicit Compare(const std::vector<std::string> *keys) : keys(keys) {}
 
     bool operator()(size_t i, size_t j) const {
       return (*keys)[i] > (*keys)[j];
-    };
+    }
 
    private:
-    const std::vector<string> *keys;
+    const std::vector<std::string> *keys;
   };
 
   // Positions the stream at the position corresponding to the lower bound for
   // the specified key.
-  void LowerBound(size_t id, const string &find_key) {
+  void LowerBound(size_t id, const std::string &find_key) {
     auto *strm = streams_[id];
     const auto &positions = positions_[id];
     if (positions.empty()) return;
@@ -223,7 +230,7 @@ class STTableReader {
     while (low < high) {
       size_t mid = (low + high) / 2;
       strm->seekg(positions[mid]);
-      string key;
+      std::string key;
       ReadType(*strm, &key);
       if (key > find_key) {
         high = mid;
@@ -280,11 +287,11 @@ class STTableReader {
 
   Reader entry_reader_;
   std::vector<std::istream *> streams_;        // Input streams.
-  std::vector<string> sources_;                // Corresponding file names.
+  std::vector<std::string> sources_;           // Corresponding file names.
   std::vector<std::vector<int64>> positions_;  // Index of positions.
-  std::vector<string> keys_;  // Lowest unread key for each stream.
-  std::vector<int64> heap_;   // Heap containing ID of streams with unread keys.
-  int64 current_;             // ID of current stream to be read.
+  std::vector<std::string> keys_;  // Lowest unread key for each stream.
+  std::vector<int64> heap_;  // Heap containing ID of streams with unread keys.
+  int64 current_;            // ID of current stream to be read.
   std::unique_ptr<Compare> compare_;  // Functor comparing stream IDs.
   mutable std::unique_ptr<T> entry_;  // The currently read entry.
   bool error_;
@@ -294,17 +301,17 @@ class STTableReader {
 // The Header type must provide at least the following interface:
 //
 //   struct Header {
-//     void Read(std::istream &istrm, const string &filename);
+//     void Read(std::istream &istrm, const string &source);
 //   };
 template <class Header>
-bool ReadSTTableHeader(const string &filename, Header *header) {
-  if (filename.empty()) {
+bool ReadSTTableHeader(const std::string &source, Header *header) {
+  if (source.empty()) {
     LOG(ERROR) << "ReadSTTable: Can't read header from standard input";
     return false;
   }
-  std::ifstream strm(filename, std::ios_base::in | std::ios_base::binary);
+  std::ifstream strm(source, std::ios_base::in | std::ios_base::binary);
   if (!strm) {
-    LOG(ERROR) << "ReadSTTableHeader: Could not open file: " << filename;
+    LOG(ERROR) << "ReadSTTableHeader: Could not open file: " << source;
     return false;
   }
   int32 magic_number = 0;
@@ -312,35 +319,35 @@ bool ReadSTTableHeader(const string &filename, Header *header) {
   int32 file_version = 0;
   ReadType(strm, &file_version);
   if (magic_number != kSTTableMagicNumber) {
-    LOG(ERROR) << "ReadSTTableHeader: Wrong file type: " << filename;
+    LOG(ERROR) << "ReadSTTableHeader: Wrong file type: " << source;
     return false;
   }
   if (file_version != kSTTableFileVersion) {
-    LOG(ERROR) << "ReadSTTableHeader: Wrong file version: " << filename;
+    LOG(ERROR) << "ReadSTTableHeader: Wrong file version: " << source;
     return false;
   }
   int64 i = -1;
   strm.seekg(-static_cast<int>(sizeof(int64)), std::ios_base::end);
   ReadType(strm, &i);  // Reads number of entries
   if (strm.fail()) {
-    LOG(ERROR) << "ReadSTTableHeader: Error reading file: " << filename;
+    LOG(ERROR) << "ReadSTTableHeader: Error reading file: " << source;
     return false;
   }
   if (i == 0) return true;  // No entry header to read.
   strm.seekg(-2 * static_cast<int>(sizeof(int64)), std::ios_base::end);
   ReadType(strm, &i);  // Reads position for last entry in file.
   strm.seekg(i);
-  string key;
+  std::string key;
   ReadType(strm, &key);
-  header->Read(strm, filename + ":" + key);
+  header->Read(strm, source + ":" + key);
   if (strm.fail()) {
-    LOG(ERROR) << "ReadSTTableHeader: Error reading file: " << filename;
+    LOG(ERROR) << "ReadSTTableHeader: Error reading file: " << source;
     return false;
   }
   return true;
 }
 
-bool IsSTTable(const string &filename);
+bool IsSTTable(const std::string &source);
 
 }  // namespace fst
 
