@@ -5,8 +5,8 @@
 #define FST_EXTENSIONS_LINEAR_LOGLINEAR_APPLY_H_
 
 #include <fst/compat.h>
-#include <fst/arc.h>
 #include <fst/arc-map.h>
+#include <fst/arc.h>
 #include <fst/compose.h>
 #include <fst/determinize.h>
 #include <fst/float-weight.h>
@@ -49,7 +49,7 @@ void LogLinearApply(const Fst<A> &ifst, const Fst<A> &lfst, MutableFst<A> *ofst,
     Compose(ifst, lfst, &unnormalized_ofst);
     {
       VectorFst<A> tropical_ifsa(unnormalized_ofst);
-      Project(&tropical_ifsa, PROJECT_INPUT);
+      Project(&tropical_ifsa, ProjectType::INPUT);
       {
         VectorFst<B> minimal_log_ifsa;
         {
