@@ -9,8 +9,10 @@
 #include <unordered_map>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/extensions/mpdt/mpdt.h>
 #include <fst/fst.h>
+#include <unordered_set>
 
 namespace fst {
 
@@ -26,9 +28,9 @@ class MPdtInfo {
            const std::vector<std::pair<Label, Label>> &parens,
            const std::vector<Label> &assignments);
 
-  const string &FstType() const { return fst_type_; }
+  const std::string &FstType() const { return fst_type_; }
 
-  const string &ArcType() const { return Arc::Type(); }
+  const std::string &ArcType() const { return Arc::Type(); }
 
   int64 NumStates() const { return nstates_; }
 
@@ -58,7 +60,7 @@ class MPdtInfo {
   void Print();
 
  private:
-  string fst_type_;
+  std::string fst_type_;
   int64 nstates_;
   int64 narcs_;
   int64 nopen_parens_[nlevels];
@@ -123,23 +125,19 @@ MPdtInfo<Arc, nlevels>::MPdtInfo(
         const auto level = paren_levels[arc.ilabel];
         if (arc.ilabel == open_paren) {
           ++nopen_parens_[level];
-          if (!paren_set.count(open_paren)) {
+          if (paren_set.insert(open_paren).second) {
             ++nuniq_open_parens_[level];
-            paren_set.insert(open_paren);
           }
-          if (!open_paren_state_set.count(arc.nextstate)) {
+          if (open_paren_state_set.insert(arc.nextstate).second) {
             ++nopen_paren_states_[level];
-            open_paren_state_set.insert(arc.nextstate);
           }
         } else {
           ++nclose_parens_[level];
-          if (!paren_set.count(close_paren)) {
+          if (paren_set.insert(close_paren).second) {
             ++nuniq_close_parens_[level];
-            paren_set.insert(close_paren);
           }
-          if (!close_paren_state_set.count(s)) {
+          if (close_paren_state_set.insert(s).second) {
             ++nclose_paren_states_[level];
-            close_paren_state_set.insert(s);
           }
         }
       }
