@@ -32,6 +32,7 @@ class ParenMatcher {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  // This makes a copy of the FST.
   ParenMatcher(const FST &fst, MatchType match_type,
                uint32 flags = (kParenLoop | kParenList))
       : matcher_(fst, match_type), match_type_(match_type), flags_(flags) {
@@ -46,6 +47,22 @@ class ParenMatcher {
     loop_.nextstate = kNoStateId;
   }
 
+  // This doesn't copy the FST.
+  ParenMatcher(const FST *fst, MatchType match_type,
+               uint32 flags = (kParenLoop | kParenList))
+      : matcher_(fst, match_type), match_type_(match_type), flags_(flags) {
+    if (match_type == MATCH_INPUT) {
+      loop_.ilabel = kNoLabel;
+      loop_.olabel = 0;
+    } else {
+      loop_.ilabel = 0;
+      loop_.olabel = kNoLabel;
+    }
+    loop_.weight = Weight::One();
+    loop_.nextstate = kNoStateId;
+  }
+
+  // This makes a copy of the FST.
   ParenMatcher(const ParenMatcher<FST> &matcher, bool safe = false)
       : matcher_(matcher.matcher_, safe),
         match_type_(matcher.match_type_),
@@ -437,11 +454,12 @@ struct PdtComposeOptions {
 // interpreted as a PDT, the parens must balance on a path (see PdtExpand()).
 // The open-close parenthesis label pairs are passed using the parens argument.
 template <class Arc>
-void Compose(const Fst<Arc> &ifst1,
-             const std::vector<
-                 std::pair<typename Arc::Label, typename Arc::Label>> &parens,
-             const Fst<Arc> &ifst2, MutableFst<Arc> *ofst,
-             const PdtComposeOptions &opts = PdtComposeOptions()) {
+void Compose(
+    const Fst<Arc> &ifst1,
+    const std::vector<std::pair<typename Arc::Label, typename Arc::Label>>
+        &parens,
+    const Fst<Arc> &ifst2, MutableFst<Arc> *ofst,
+    const PdtComposeOptions &opts = PdtComposeOptions()) {
   bool expand = opts.filter_type != PAREN_FILTER;
   bool keep_parens = opts.filter_type != EXPAND_FILTER;
   PdtComposeFstOptions<Arc, true> copts(ifst1, parens, ifst2, expand,
@@ -457,11 +475,12 @@ void Compose(const Fst<Arc> &ifst1,
 // interpreted as a PDT, the parens must balance on a path (see ExpandFst()).
 // The open-close parenthesis label pairs are passed using the parens argument.
 template <class Arc>
-void Compose(const Fst<Arc> &ifst1, const Fst<Arc> &ifst2,
-             const std::vector<
-                 std::pair<typename Arc::Label, typename Arc::Label>> &parens,
-             MutableFst<Arc> *ofst,
-             const PdtComposeOptions &opts = PdtComposeOptions()) {
+void Compose(
+    const Fst<Arc> &ifst1, const Fst<Arc> &ifst2,
+    const std::vector<std::pair<typename Arc::Label, typename Arc::Label>>
+        &parens,
+    MutableFst<Arc> *ofst,
+    const PdtComposeOptions &opts = PdtComposeOptions()) {
   bool expand = opts.filter_type != PAREN_FILTER;
   bool keep_parens = opts.filter_type != EXPAND_FILTER;
   PdtComposeFstOptions<Arc, false> copts(ifst1, ifst2, parens, expand,
