@@ -9,8 +9,8 @@
 #include <forward_list>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
-
 #include <fst/extensions/pdt/paren.h>
 #include <fst/extensions/pdt/pdt.h>
 #include <fst/extensions/pdt/reverse.h>
@@ -20,6 +20,7 @@
 #include <fst/queue.h>
 #include <fst/state-table.h>
 #include <fst/test-properties.h>
+#include <unordered_map>
 
 namespace fst {
 
@@ -75,8 +76,8 @@ class PdtExpandFstImpl : public CacheImpl<Arc> {
         stack_(opts.stack ? opts.stack : new PdtStack<StateId, Label>(parens)),
         state_table_(opts.state_table ? opts.state_table
                                       : new PdtStateTable<StateId, StackId>()),
-        own_stack_(opts.stack == 0),
-        own_state_table_(opts.state_table == 0),
+        own_stack_(opts.stack == nullptr),
+        own_state_table_(opts.state_table == nullptr),
         keep_parentheses_(opts.keep_parentheses) {
     SetType("expand");
     const auto props = fst.Properties(kFstProperties, false);
@@ -882,8 +883,8 @@ struct PdtExpandOptions {
   bool keep_parentheses;
   Weight weight_threshold;
 
-  PdtExpandOptions(bool connect = true, bool keep_parentheses = false,
-                   Weight weight_threshold = Weight::Zero())
+  explicit PdtExpandOptions(bool connect = true, bool keep_parentheses = false,
+                            Weight weight_threshold = Weight::Zero())
       : connect(connect),
         keep_parentheses(keep_parentheses),
         weight_threshold(std::move(weight_threshold)) {}
@@ -920,10 +921,11 @@ void Expand(
 // pairs are passed using the parents argument. Expansion enforces the
 // parenthesis constraints. The PDT must be expandable as an FST.
 template <class Arc>
-void Expand(const Fst<Arc> &ifst,
+void Expand(
+    const Fst<Arc> &ifst,
     const std::vector<std::pair<typename Arc::Label, typename Arc::Label>>
-    &parens, MutableFst<Arc> *ofst, bool connect = true,
-    bool keep_parentheses = false) {
+        &parens,
+    MutableFst<Arc> *ofst, bool connect = true, bool keep_parentheses = false) {
   const PdtExpandOptions<Arc> opts(connect, keep_parentheses);
   Expand(ifst, parens, ofst, opts);
 }
