@@ -13,9 +13,10 @@
 #include <utility>
 #include <vector>
 
-#include <fst/replace.h>
 #include <fst/replace-util.h>
+#include <fst/replace.h>
 #include <fst/symbol-table-ops.h>
+#include <unordered_map>
 
 namespace fst {
 namespace internal {
@@ -65,20 +66,21 @@ template <class Arc>
 struct PdtReplaceOptions {
   using Label = typename Arc::Label;
 
-  explicit PdtReplaceOptions(Label root,
-                             PdtParserType type = PDT_LEFT_PARSER,
+  explicit PdtReplaceOptions(Label root, PdtParserType type = PDT_LEFT_PARSER,
                              Label start_paren_labels = kNoLabel,
-                             string left_paren_prefix = "(_",
-                             string right_paren_prefix = ")_") :
-      root(root), type(type), start_paren_labels(start_paren_labels),
-      left_paren_prefix(std::move(left_paren_prefix)),
-      right_paren_prefix(std::move(right_paren_prefix)) {}
+                             std::string left_paren_prefix = "(_",
+                             std::string right_paren_prefix = ")_")
+      : root(root),
+        type(type),
+        start_paren_labels(start_paren_labels),
+        left_paren_prefix(std::move(left_paren_prefix)),
+        right_paren_prefix(std::move(right_paren_prefix)) {}
 
   Label root;
   PdtParserType type;
   Label start_paren_labels;
-  const string left_paren_prefix;
-  const string right_paren_prefix;
+  const std::string left_paren_prefix;
+  const std::string right_paren_prefix;
 };
 
 // PdtParser: Base PDT parser class common to specific parsers.
@@ -98,11 +100,12 @@ class PdtParser {
       std::unordered_map<ParenKey, size_t, internal::ReplaceParenHash<StateId>>;
 
   PdtParser(const std::vector<LabelFstPair> &fst_array,
-            const PdtReplaceOptions<Arc> &opts) :
-      root_(opts.root), start_paren_labels_(opts.start_paren_labels),
-      left_paren_prefix_(std::move(opts.left_paren_prefix)),
-      right_paren_prefix_(std::move(opts.right_paren_prefix)),
-      error_(false) {
+            const PdtReplaceOptions<Arc> &opts)
+      : root_(opts.root),
+        start_paren_labels_(opts.start_paren_labels),
+        left_paren_prefix_(std::move(opts.left_paren_prefix)),
+        right_paren_prefix_(std::move(opts.right_paren_prefix)),
+        error_(false) {
     for (size_t i = 0; i < fst_array.size(); ++i) {
       if (!CompatSymbols(fst_array[0].second->InputSymbols(),
                          fst_array[i].second->InputSymbols())) {
@@ -113,7 +116,7 @@ class PdtParser {
       if (!CompatSymbols(fst_array[0].second->OutputSymbols(),
                          fst_array[i].second->OutputSymbols())) {
         FSTERROR() << "PdtParser: Output symbol table of input FST " << i
-                   << " does not match input symbol table of 0th input FST";
+                   << " does not match output symbol table of 0th input FST";
         error_ = true;
       }
       fst_array_.emplace_back(fst_array[i].first, fst_array[i].second->Copy());
@@ -198,12 +201,10 @@ class PdtParser {
   // the parenthesis that would carry the non-terminal arc weight and the other
   // parenthesis is omitted (appropriate for the strongly-regular case).
   void AddParensToFst(
-      const std::vector<LabelPair> &parens,
-      const ParenMap &paren_map,
+      const std::vector<LabelPair> &parens, const ParenMap &paren_map,
       const std::vector<StateId> &open_dest,
       const std::vector<std::vector<StateWeightPair>> &close_src,
-      const std::vector<bool> &close_non_term_weight,
-      MutableFst<Arc> *ofst);
+      const std::vector<bool> &close_non_term_weight, MutableFst<Arc> *ofst);
 
   // Ensures that parentheses arcs are added to the symbol table.
   void AddParensToSymbolTables(const std::vector<LabelPair> &parens,
@@ -214,8 +215,8 @@ class PdtParser {
   Label root_;
   // Index to use for the first parenthesis.
   Label start_paren_labels_;
-  const string left_paren_prefix_;
-  const string right_paren_prefix_;
+  const std::string left_paren_prefix_;
+  const std::string right_paren_prefix_;
   // Maps from non-terminal label to FST ID.
   std::unordered_map<Label, StateId> label2id_;
   // Given an output state, specifies the input FST (label, state) pair.
@@ -280,18 +281,15 @@ void PdtParser<Arc>::CreateFst(
       }
     }
   }
-  if (start_paren_labels_ == kNoLabel)
-    start_paren_labels_ = max_label + 1;
+  if (start_paren_labels_ == kNoLabel) start_paren_labels_ = max_label + 1;
 }
 
 template <class Arc>
 void PdtParser<Arc>::AddParensToFst(
-    const std::vector<LabelPair> &parens,
-    const ParenMap &paren_map,
+    const std::vector<LabelPair> &parens, const ParenMap &paren_map,
     const std::vector<StateId> &open_dest,
     const std::vector<std::vector<StateWeightPair>> &close_src,
-    const std::vector<bool> &close_non_term_weight,
-    MutableFst<Arc> *ofst) {
+    const std::vector<bool> &close_non_term_weight, MutableFst<Arc> *ofst) {
   StateId dead_state = kNoStateId;
   using MIter = MutableArcIterator<MutableFst<Arc>>;
   for (StateIterator<Fst<Arc>> siter(*ofst); !siter.Done(); siter.Next()) {
@@ -403,8 +401,8 @@ class PdtLeftParser final : public PdtParser<Arc> {
   using PdtParser<Arc>::Root;
 
   PdtLeftParser(const std::vector<LabelFstPair> &fst_array,
-                const PdtReplaceOptions<Arc> &opts) :
-      PdtParser<Arc>(fst_array, opts) { }
+                const PdtReplaceOptions<Arc> &opts)
+      : PdtParser<Arc>(fst_array, opts) {}
 
   void GetParser(MutableFst<Arc> *ofst,
                  std::vector<LabelPair> *parens) override;
@@ -417,9 +415,8 @@ class PdtLeftParser final : public PdtParser<Arc> {
 };
 
 template <class Arc>
-void PdtLeftParser<Arc>::GetParser(
-    MutableFst<Arc> *ofst,
-    std::vector<LabelPair> *parens) {
+void PdtLeftParser<Arc>::GetParser(MutableFst<Arc> *ofst,
+                                   std::vector<LabelPair> *parens) {
   ofst->DeleteStates();
   parens->clear();
   const auto &fst_array = FstArray();
@@ -450,9 +447,8 @@ void PdtLeftParser<Arc>::GetParser(
 }
 
 template <class Arc>
-size_t PdtLeftParser<Arc>::AssignParenIds(
-    const Fst<Arc> &ofst,
-    ParenMap *paren_map) const {
+size_t PdtLeftParser<Arc>::AssignParenIds(const Fst<Arc> &ofst,
+                                          ParenMap *paren_map) const {
   // Number of distinct parenthesis pairs per FST.
   std::vector<size_t> nparens(FstArray().size(), 0);
   // Number of distinct parenthesis pairs overall.
@@ -507,9 +503,9 @@ class PdtLeftSRParser final : public PdtParser<Arc> {
   using PdtParser<Arc>::Root;
 
   PdtLeftSRParser(const std::vector<LabelFstPair> &fst_array,
-                  const PdtReplaceOptions<Arc> &opts) :
-      PdtParser<Arc>(fst_array, opts),
-      replace_util_(fst_array, ReplaceUtilOptions(opts.root)) { }
+                  const PdtReplaceOptions<Arc> &opts)
+      : PdtParser<Arc>(fst_array, opts),
+        replace_util_(fst_array, ReplaceUtilOptions(opts.root)) {}
 
   void GetParser(MutableFst<Arc> *ofst,
                  std::vector<LabelPair> *parens) override;
@@ -568,8 +564,7 @@ class PdtLeftSRParser final : public PdtParser<Arc> {
  private:
   // Merges initial (final) states of in a left- (right-) linear dependency SCC
   // after dealing with the non-terminal arc and final weights.
-  void ProcSCCs(MutableFst<Arc> *ofst,
-                std::vector<StateId> *open_dest,
+  void ProcSCCs(MutableFst<Arc> *ofst, std::vector<StateId> *open_dest,
                 std::vector<std::vector<StateWeightPair>> *close_src,
                 std::vector<bool> *close_non_term_weight) const;
 
@@ -604,9 +599,8 @@ class PdtLeftSRParser final : public PdtParser<Arc> {
 };
 
 template <class Arc>
-void PdtLeftSRParser<Arc>::GetParser(
-    MutableFst<Arc> *ofst,
-    std::vector<LabelPair> *parens) {
+void PdtLeftSRParser<Arc>::GetParser(MutableFst<Arc> *ofst,
+                                     std::vector<LabelPair> *parens) {
   ofst->DeleteStates();
   parens->clear();
   const auto &fst_array = FstArray();
@@ -639,8 +633,7 @@ void PdtLeftSRParser<Arc>::GetParser(
 
 template <class Arc>
 void PdtLeftSRParser<Arc>::ProcSCCs(
-    MutableFst<Arc> *ofst,
-    std::vector<StateId> *open_dest,
+    MutableFst<Arc> *ofst, std::vector<StateId> *open_dest,
     std::vector<std::vector<StateWeightPair>> *close_src,
     std::vector<bool> *close_non_term_weight) const {
   const auto &fst_array = FstArray();
@@ -663,8 +656,7 @@ void PdtLeftSRParser<Arc>::ProcSCCs(
           ofst->AddArc(rs, arc);
         }
         ofst->DeleteArcs(os);
-        if (os == ofst->Start())
-          ofst->SetStart(rs);
+        if (os == ofst->Start()) ofst->SetStart(rs);
         (*open_dest)[fst_id] = rs;
       }
     }
@@ -727,9 +719,8 @@ void PdtLeftSRParser<Arc>::GetNonTermDests() const {
 }
 
 template <class Arc>
-size_t PdtLeftSRParser<Arc>::AssignParenIds(
-    const Fst<Arc> &ofst,
-    ParenMap *paren_map) const {
+size_t PdtLeftSRParser<Arc>::AssignParenIds(const Fst<Arc> &ofst,
+                                            ParenMap *paren_map) const {
   const auto &fst_array = FstArray();
   // Number of distinct parenthesis pairs per FST.
   std::vector<size_t> nparens(fst_array.size(), 0);
@@ -789,18 +780,16 @@ void Replace(
     std::vector<std::pair<typename Arc::Label, typename Arc::Label>> *parens,
     const PdtReplaceOptions<Arc> &opts) {
   switch (opts.type) {
-    case PDT_LEFT_PARSER:
-      {
-        PdtLeftParser<Arc> pr(ifst_array, opts);
-        pr.GetParser(ofst, parens);
-        return;
-      }
-    case PDT_LEFT_SR_PARSER:
-      {
-        PdtLeftSRParser<Arc> pr(ifst_array, opts);
-        pr.GetParser(ofst, parens);
-        return;
-      }
+    case PDT_LEFT_PARSER: {
+      PdtLeftParser<Arc> pr(ifst_array, opts);
+      pr.GetParser(ofst, parens);
+      return;
+    }
+    case PDT_LEFT_SR_PARSER: {
+      PdtLeftSRParser<Arc> pr(ifst_array, opts);
+      pr.GetParser(ofst, parens);
+      return;
+    }
     default:
       FSTERROR() << "Replace: Unknown PDT parser type: " << opts.type;
       ofst->DeleteStates();
