@@ -3,13 +3,15 @@
 //
 // This file contains declarations of classes in the Fst template library.
 
-#ifndef FST_LIB_FST_DECL_H_
-#define FST_LIB_FST_DECL_H_
+#ifndef FST_FST_DECL_H_
+#define FST_FST_DECL_H_
 
 #include <sys/types.h>
+
 #include <memory>  // for allocator<>
 
 #include <fst/types.h>
+#include <fst/windows_defs.inc>
 
 namespace fst {
 
@@ -53,18 +55,30 @@ using LogArc = ArcTpl<LogWeight>;
 // Stores.
 
 template <class Element, class U>
-class DefaultCompactStore;
+class CompactArcStore;
 
 template <class Arc>
 class DefaultCacheStore;
 
+// Compactors.
+
+template <class AC, class U, class S = CompactArcStore<typename AC::Element, U>>
+class CompactArcCompactor;
+
 // FST templates.
 
-template <class Arc, class Compactor, class U = uint32,
-    class CompactStore = DefaultCompactStore<typename Compactor::Element, U>,
-    class CacheStore = DefaultCacheStore<Arc>>
+template <class Arc, class Compactor, class CacheStore = DefaultCacheStore<Arc>>
 class CompactFst;
 
+// The Unsigned type is used to represent indices into the compact arc array.
+template <class Arc, class ArcCompactor, class Unsigned = uint32,
+          class CompactStore =
+              CompactArcStore<typename ArcCompactor::Element, Unsigned>,
+          class CacheStore = DefaultCacheStore<Arc>>
+using CompactArcFst =
+    CompactFst<Arc, CompactArcCompactor<ArcCompactor, Unsigned, CompactStore>,
+               CacheStore>;
+
 template <class Arc, class U = uint32>
 class ConstFst;
 
@@ -142,7 +156,7 @@ class UnionFst;
 template <class T, class Compare>
 class Heap;
 
-// Compactors.
+// ArcCompactors.
 
 template <class Arc>
 class AcceptorCompactor;
@@ -159,24 +173,24 @@ class UnweightedCompactor;
 template <class Arc>
 class WeightedStringCompactor;
 
-// Compact FSTs.
+// Compact Arc FSTs.
 
 template <class Arc, class U = uint32>
-using CompactStringFst = CompactFst<Arc, StringCompactor<Arc>, U>;
+using CompactStringFst = CompactArcFst<Arc, StringCompactor<Arc>, U>;
 
 template <class Arc, class U = uint32>
 using CompactWeightedStringFst =
-    CompactFst<Arc, WeightedStringCompactor<Arc>, U>;
+    CompactArcFst<Arc, WeightedStringCompactor<Arc>, U>;
 
 template <class Arc, class U = uint32>
-using CompactAcceptorFst = CompactFst<Arc, AcceptorCompactor<Arc>, U>;
+using CompactAcceptorFst = CompactArcFst<Arc, AcceptorCompactor<Arc>, U>;
 
 template <class Arc, class U = uint32>
-using CompactUnweightedFst = CompactFst<Arc, UnweightedCompactor<Arc>, U>;
+using CompactUnweightedFst = CompactArcFst<Arc, UnweightedCompactor<Arc>, U>;
 
 template <class Arc, class U = uint32>
 using CompactUnweightedAcceptorFst =
-    CompactFst<Arc, UnweightedAcceptorCompactor<Arc>, U>;
+    CompactArcFst<Arc, UnweightedAcceptorCompactor<Arc>, U>;
 
 // StdArc aliases for FSTs.
 
@@ -246,6 +260,9 @@ class AltSequenceComposeFilter;
 template <class Matcher1, class Matcher2 = Matcher1>
 class MatchComposeFilter;
 
+template <class Matcher1, class Matcher2 = Matcher1>
+class NoMatchComposeFilter;
+
 }  // namespace fst
 
-#endif  // FST_LIB_FST_DECL_H_
+#endif  // FST_FST_DECL_H_
