@@ -1,17 +1,16 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#ifndef FST_LIB_GENERIC_REGISTER_H_
-#define FST_LIB_GENERIC_REGISTER_H_
+#ifndef FST_GENERIC_REGISTER_H_
+#define FST_GENERIC_REGISTER_H_
 
+#include <fst/compat.h>
 #ifndef FST_NO_DYNAMIC_LINKING
 #include <dlfcn.h>
-#include <fst/compat.h>
 #endif
 #include <map>
 #include <string>
 
-#include <fst/types.h>
 #include <fst/log.h>
 
 // Generic class representing a globally-stored correspondence between
@@ -44,7 +43,7 @@ class GenericRegister {
 
   void SetEntry(const KeyType &key, const EntryType &entry) {
     MutexLock l(&register_lock_);
-    register_table_.insert(std::make_pair(key, entry));
+    register_table_.emplace(key, entry);
   }
 
   EntryType GetEntry(const KeyType &key) const {
@@ -88,7 +87,7 @@ class GenericRegister {
   }
 
   // Override this to define how to turn a key into an SO filename.
-  virtual string ConvertKeyToSoFilename(const KeyType &key) const = 0;
+  virtual std::string ConvertKeyToSoFilename(const KeyType &key) const = 0;
 
   virtual const EntryType *LookupEntry(const KeyType &key) const {
     MutexLock l(&register_lock_);
@@ -123,4 +122,4 @@ class GenericRegisterer {
 
 }  // namespace fst
 
-#endif  // FST_LIB_GENERIC_REGISTER_H_
+#endif  // FST_GENERIC_REGISTER_H_
