@@ -5,8 +5,8 @@
 // using a key. The key can be used to do an in-place update of values in the
 // heap.
 
-#ifndef FST_LIB_HEAP_H_
-#define FST_LIB_HEAP_H_
+#ifndef FST_HEAP_H_
+#define FST_HEAP_H_
 
 #include <utility>
 #include <vector>
@@ -68,7 +68,7 @@ class Heap {
   // Returns the least value.
   Value Pop() {
     Value top = values_.front();
-    Swap(0, size_-1);
+    Swap(0, size_ - 1);
     size_--;
     Heapify(0);
     return top;
@@ -94,6 +94,8 @@ class Heap {
     key_.reserve(size);
   }
 
+  const Compare &GetCompare() const { return comp_; }
+
  private:
   // The following private routines are used in a supportive role
   // for managing the heap and keeping the heap properties.
@@ -163,4 +165,4 @@ constexpr int Heap<T, Compare>::kNoKey;
 
 }  // namespace fst
 
-#endif  // FST_LIB_HEAP_H_
+#endif  // FST_HEAP_H_
