@@ -3,10 +3,11 @@
 //
 // Class to represent and operate on sets of intervals.
 
-#ifndef FST_LIB_INTERVAL_SET_H_
-#define FST_LIB_INTERVAL_SET_H_
+#ifndef FST_INTERVAL_SET_H_
+#define FST_INTERVAL_SET_H_
 
 #include <algorithm>
+#include <initializer_list>
 #include <iostream>
 #include <vector>
 
@@ -65,6 +66,8 @@ class VectorIntervalStore {
   using Iterator = typename std::vector<Interval>::const_iterator;
 
   VectorIntervalStore() : count_(-1) {}
+  VectorIntervalStore(std::initializer_list<Interval> intervals_init)
+      : intervals_(intervals_init), count_(-1) {}
 
   std::vector<Interval> *MutableIntervals() { return &intervals_; }
 
@@ -107,6 +110,9 @@ class IntervalSet {
  public:
   using Interval = IntInterval<T>;
 
+  IntervalSet(std::initializer_list<Interval> intervals_init)
+      : intervals_(intervals_init) {}
+
   template <class... A>
   explicit IntervalSet(A... args) : intervals_(args...) {}
 
@@ -395,4 +401,4 @@ std::ostream &operator<<(std::ostream &strm, const IntervalSet<T, Store> &s) {
 
 }  // namespace fst
 
-#endif  // FST_LIB_INTERVAL_SET_H_
+#endif  // FST_INTERVAL_SET_H_
