@@ -10,13 +10,13 @@
 // The + operation on two weights a and b is the lexicographically
 // prior of a and b.
 
-#ifndef FST_LIB_LEXICOGRAPHIC_WEIGHT_H_
-#define FST_LIB_LEXICOGRAPHIC_WEIGHT_H_
-
-#include <cstdlib>
+#ifndef FST_LEXICOGRAPHIC_WEIGHT_H_
+#define FST_LEXICOGRAPHIC_WEIGHT_H_
 
+#include <random>
 #include <string>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/pair-weight.h>
@@ -74,9 +74,9 @@ class LexicographicWeight : public PairWeight<W1, W2> {
     return no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type =
-        new string(W1::Type() + "_LT_" + W2::Type());
+  static const std::string &Type() {
+    static const std::string *const type =
+        new std::string(W1::Type() + "_LT_" + W2::Type());
     return *type;
   }
 
@@ -145,29 +145,32 @@ class WeightGenerate<LexicographicWeight<W1, W2>> {
   using Generate1 = WeightGenerate<W1>;
   using Generate2 = WeightGenerate<W2>;
 
-  explicit WeightGenerate(bool allow_zero = true,
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true,
                           size_t num_random_weights = kNumRandomWeights)
-      : generator1_(false, num_random_weights),
-        generator2_(false, num_random_weights), allow_zero_(allow_zero),
-        num_random_weights_(num_random_weights) {}
+      : rand_(seed),
+        allow_zero_(allow_zero),
+        num_random_weights_(num_random_weights),
+        generator1_(seed, false, num_random_weights),
+        generator2_(seed, false, num_random_weights) {}
 
   Weight operator()() const {
     if (allow_zero_) {
-      const int n = rand() % (num_random_weights_ + 1);  // NOLINT
-      if (n == num_random_weights_) return Weight(W1::Zero(), W2::Zero());
+      const int sample =
+          std::uniform_int_distribution<>(0, num_random_weights_)(rand_);
+      if (sample == num_random_weights_) return Weight(W1::Zero(), W2::Zero());
     }
     return Weight(generator1_(), generator2_());
   }
 
  private:
+  mutable std::mt19937_64 rand_;
+  const bool allow_zero_;
+  const size_t num_random_weights_;
   const Generate1 generator1_;
   const Generate2 generator2_;
-  // Permits Zero() and zero divisors.
-  const bool allow_zero_;
-  // The number of alternative random weights.
-  const size_t num_random_weights_;
 };
 
 }  // namespace fst
 
-#endif  // FST_LIB_LEXICOGRAPHIC_WEIGHT_H_
+#endif  // FST_LEXICOGRAPHIC_WEIGHT_H_
