@@ -18,11 +18,23 @@
 #ifndef FST_LIB_LOCK_H_
 #define FST_LIB_LOCK_H_
 
-#include <mutex>
+#ifdef OPENFST_HAS_ABSL
+
+#include "absl/synchronization/mutex.h"
 
 namespace fst {
 
-using namespace std;
+using absl::Mutex;
+using absl::MutexLock;
+using absl::ReaderMutexLock;
+
+}  // namespace fst
+
+#else  // OPENFST_HAS_ABSL
+
+#include <mutex>
+
+namespace fst {
 
 class Mutex {
  public:
@@ -59,4 +71,6 @@ using ReaderMutexLock = MutexLock;
 
 }  // namespace fst
 
+#endif  // OPENFST_HAS_ABSL
+
 #endif  // FST_LIB_LOCK_H_
