@@ -4,11 +4,12 @@
 // Composition filters to support lookahead matchers, useful for improving
 // composition efficiency with certain inputs.
 
-#ifndef FST_LIB_LOOKAHEAD_FILTER_H_
-#define FST_LIB_LOOKAHEAD_FILTER_H_
+#ifndef FST_LOOKAHEAD_FILTER_H_
+#define FST_LOOKAHEAD_FILTER_H_
 
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/filter-state.h>
@@ -308,7 +309,7 @@ class PushWeightsComposeFilter {
     if (!(LookAheadFlags() & kLookAheadWeight)) {
       return FilterState(fs1, FilterState2(Weight::One()));
     }
-    const auto &lweight = filter_.LookAheadArc()
+    const auto &lweight = LookAheadArc()
                               ? Selector().GetMatcher()->LookAheadWeight()
                               : Weight::One();
     const auto &fs2 = fs_.GetState2();
@@ -438,8 +439,7 @@ class PushLabelsComposeFilter {
     }
     const auto &fs1 = filter_.FilterArc(arc1, arc2);
     if (fs1 == FilterState1::NoState()) return FilterState::NoState();
-    if (!filter_.LookAheadArc())
-      return FilterState(fs1, FilterState2(kNoLabel));
+    if (!LookAheadArc()) return FilterState(fs1, FilterState2(kNoLabel));
     return LookAheadOutput() ? PushLabelFilterArc(arc1, arc2, fs1)
                              : PushLabelFilterArc(arc2, arc1, fs1);
   }
@@ -620,4 +620,4 @@ class DefaultLookAhead<LogArc, MATCH_OUTPUT> {
 
 }  // namespace fst
 
-#endif  // FST_LIB_LOOKAHEAD_FILTER_H_
+#endif  // FST_LOOKAHEAD_FILTER_H_
