@@ -4,13 +4,15 @@
 // Classes to add lookahead to FST matchers, useful for improving composition
 // efficiency with certain inputs.
 
-#ifndef FST_LIB_LOOKAHEAD_MATCHER_H_
-#define FST_LIB_LOOKAHEAD_MATCHER_H_
+#ifndef FST_LOOKAHEAD_MATCHER_H_
+#define FST_LOOKAHEAD_MATCHER_H_
 
 #include <memory>
 #include <utility>
 #include <vector>
 
+#include <fst/flags.h>
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/add-on.h>
@@ -19,7 +21,6 @@
 #include <fst/label-reachable.h>
 #include <fst/matcher.h>
 
-
 DECLARE_string(save_relabel_ipairs);
 DECLARE_string(save_relabel_opairs);
 
@@ -37,11 +38,18 @@ namespace fst {
 //   using Weight = typename Arc::Weight;
 //
 //  // Required constructors.
+//  // This makes a copy of the FST.
 //  LookAheadMatcher(const FST &fst, MatchType match_type);
-//   // If safe=true, the copy is thread-safe (except the lookahead FST is
-//   // preserved). See Fst<>::Cop() for further doc.
+//  // This doesn't copy the FST.
+//  LookAheadMatcher(const FST *fst, MatchType match_type);
+//  // This makes a copy of the FST.
+//  // See Copy() below.
 //  LookAheadMatcher(const LookAheadMatcher &matcher, bool safe = false);
 //
+//   // If safe = true, the copy is thread-safe (except the lookahead FST is
+//   // preserved). See Fst<>::Copy() for further doc.
+//   LookaheadMatcher *Copy(bool safe = false) const override;
+
 //  // Below are methods for looking ahead for a match to a label and more
 //  // generally, to a rational set. Each returns false if there is definitely
 //  // not a match and returns true if there possibly is a match.
@@ -163,15 +171,21 @@ class TrivialLookAheadMatcher
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  // This makes a copy of the FST.
   TrivialLookAheadMatcher(const FST &fst, MatchType match_type)
       : matcher_(fst, match_type) {}
 
-  TrivialLookAheadMatcher(const TrivialLookAheadMatcher<M> &lmatcher,
+  // This doesn't copy the FST.
+  TrivialLookAheadMatcher(const FST *fst, MatchType match_type)
+      : matcher_(fst, match_type) {}
+
+  // This makes a copy of the FST.
+  TrivialLookAheadMatcher(const TrivialLookAheadMatcher &lmatcher,
                           bool safe = false)
       : matcher_(lmatcher.matcher_, safe) {}
 
-  TrivialLookAheadMatcher<M> *Copy(bool safe = false) const override {
-    return new TrivialLookAheadMatcher<M>(*this, safe);
+  TrivialLookAheadMatcher *Copy(bool safe = false) const override {
+    return new TrivialLookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_.Type(test); }
@@ -218,9 +232,8 @@ class TrivialLookAheadMatcher
 
 // Look-ahead of one transition. Template argument flags accepts flags to
 // control behavior.
-template <class M,
-          uint32 flags = kLookAheadNonEpsilons | kLookAheadEpsilons |
-                         kLookAheadWeight | kLookAheadPrefix>
+template <class M, uint32 flags = kLookAheadNonEpsilons | kLookAheadEpsilons |
+                                  kLookAheadWeight | kLookAheadPrefix>
 class ArcLookAheadMatcher : public LookAheadMatcherBase<typename M::FST::Arc> {
  public:
   using FST = typename M::FST;
@@ -239,24 +252,32 @@ class ArcLookAheadMatcher : public LookAheadMatcherBase<typename M::FST::Arc> {
 
   enum : uint32 { kFlags = flags };
 
-  ArcLookAheadMatcher(
-      const FST &fst, MatchType match_type,
-      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>())
+  // This makes a copy of the FST.
+  ArcLookAheadMatcher(const FST &fst, MatchType match_type,
+                      std::shared_ptr<MatcherData> data = nullptr)
       : matcher_(fst, match_type),
         fst_(matcher_.GetFst()),
         lfst_(nullptr),
         state_(kNoStateId) {}
 
-  ArcLookAheadMatcher(const ArcLookAheadMatcher<M, flags> &lmatcher,
-                      bool safe = false)
+  // This doesn't copy the FST.
+  ArcLookAheadMatcher(const FST *fst, MatchType match_type,
+                      std::shared_ptr<MatcherData> data = nullptr)
+      : matcher_(fst, match_type),
+        fst_(matcher_.GetFst()),
+        lfst_(nullptr),
+        state_(kNoStateId) {}
+
+  // This makes a copy of the FST.
+  ArcLookAheadMatcher(const ArcLookAheadMatcher &lmatcher, bool safe = false)
       : matcher_(lmatcher.matcher_, safe),
         fst_(matcher_.GetFst()),
         lfst_(lmatcher.lfst_),
         state_(kNoStateId) {}
 
   // General matcher methods.
-  ArcLookAheadMatcher<M, flags> *Copy(bool safe = false) const override {
-    return new ArcLookAheadMatcher<M, flags>(*this, safe);
+  ArcLookAheadMatcher *Copy(bool safe = false) const override {
+    return new ArcLookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_.Type(test); }
@@ -291,9 +312,7 @@ class ArcLookAheadMatcher : public LookAheadMatcherBase<typename M::FST::Arc> {
 
   const MatcherData *GetData() const { return nullptr; }
 
-  std::shared_ptr<MatcherData> GetSharedData() const {
-    return std::shared_ptr<MatcherData>();
-  }
+  std::shared_ptr<MatcherData> GetSharedData() const { return nullptr; }
 
   // Look-ahead methods.
 
@@ -393,11 +412,15 @@ template <class M,
           uint32 flags = kLookAheadEpsilons | kLookAheadWeight |
                          kLookAheadPrefix | kLookAheadNonEpsilonPrefix |
                          kLookAheadKeepRelabelData,
-          class Accumulator = DefaultAccumulator<typename M::Arc>,
-          class Reachable = LabelReachable<typename M::Arc, Accumulator>>
+          class Accum = DefaultAccumulator<typename M::Arc>,
+          class R = LabelReachable<typename M::Arc, Accum>>
 class LabelLookAheadMatcher
     : public LookAheadMatcherBase<typename M::FST::Arc> {
  public:
+  using Matcher = M;
+  using Accumulator = Accum;
+  using Reachable = R;
+
   using FST = typename M::FST;
   using Arc = typename FST::Arc;
   using Label = typename Arc::Label;
@@ -412,35 +435,37 @@ class LabelLookAheadMatcher
   using LookAheadMatcherBase<Arc>::LookAheadPrefix;
   using LookAheadMatcherBase<Arc>::SetLookAheadPrefix;
 
+  static_assert(!(flags & kInputLookAheadMatcher) !=
+                    !(flags & kOutputLookAheadMatcher),
+                "Must include precisely one of kInputLookAheadMatcher and "
+                "kOutputLookAheadMatcher");
   enum : uint32 { kFlags = flags };
 
-  LabelLookAheadMatcher(
-      const FST &fst, MatchType match_type,
-      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>(),
-      Accumulator *accumulator = nullptr)
+  // This makes a copy of the FST.
+  LabelLookAheadMatcher(const FST &fst, MatchType match_type,
+                        std::shared_ptr<MatcherData> data = nullptr,
+                        Accumulator *accumulator = nullptr)
       : matcher_(fst, match_type),
         lfst_(nullptr),
         state_(kNoStateId),
         error_(false) {
-    if (!(kFlags & (kInputLookAheadMatcher | kOutputLookAheadMatcher))) {
-      FSTERROR() << "LabelLookaheadMatcher: Bad matcher flags: " << kFlags;
-      error_ = true;
-    }
-    const bool reach_input = match_type == MATCH_INPUT;
-    if (data) {
-      if (reach_input == data->ReachInput()) {
-        label_reachable_.reset(new Reachable(data, accumulator));
-      }
-    } else if ((reach_input && (kFlags & kInputLookAheadMatcher)) ||
-               (!reach_input && (kFlags & kOutputLookAheadMatcher))) {
-      label_reachable_.reset(new Reachable(fst, reach_input, accumulator,
-                                           kFlags & kLookAheadKeepRelabelData));
-    }
+    Init(fst, match_type, data, accumulator);
   }
 
-  LabelLookAheadMatcher(
-      const LabelLookAheadMatcher<M, flags, Accumulator, Reachable> &lmatcher,
-      bool safe = false)
+  // This doesn't copy the FST.
+  LabelLookAheadMatcher(const FST *fst, MatchType match_type,
+                        std::shared_ptr<MatcherData> data = nullptr,
+                        Accumulator *accumulator = nullptr)
+      : matcher_(fst, match_type),
+        lfst_(nullptr),
+        state_(kNoStateId),
+        error_(false) {
+    Init(*fst, match_type, data, accumulator);
+  }
+
+  // This makes a copy of the FST.
+  LabelLookAheadMatcher(const LabelLookAheadMatcher &lmatcher,
+                        bool safe = false)
       : matcher_(lmatcher.matcher_, safe),
         lfst_(lmatcher.lfst_),
         label_reachable_(lmatcher.label_reachable_
@@ -449,10 +474,8 @@ class LabelLookAheadMatcher
         state_(kNoStateId),
         error_(lmatcher.error_) {}
 
-  LabelLookAheadMatcher<M, flags, Accumulator, Reachable> *Copy(
-      bool safe = false) const override {
-    return new LabelLookAheadMatcher<M, flags, Accumulator, Reachable>(*this,
-                                                                       safe);
+  LabelLookAheadMatcher *Copy(bool safe = false) const override {
+    return new LabelLookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_.Type(test); }
@@ -504,11 +527,10 @@ class LabelLookAheadMatcher
 
   const MatcherData *GetData() const {
     return label_reachable_ ? label_reachable_->GetData() : nullptr;
-  };
+  }
 
   std::shared_ptr<MatcherData> GetSharedData() const {
-    return label_reachable_ ? label_reachable_->GetSharedData()
-                            : std::shared_ptr<MatcherData>();
+    return label_reachable_ ? label_reachable_->GetSharedData() : nullptr;
   }
   // Checks if there is a matching (possibly super-final) transition at
   // (state_, s).
@@ -530,7 +552,7 @@ class LabelLookAheadMatcher
 
   template <class LFST>
   void InitLookAheadFst(const LFST &fst, bool copy = false) {
-    lfst_ = static_cast<const Fst<Arc> *>(&fst);
+    lfst_ = &fst;
     if (label_reachable_) {
       const bool reach_input = Type(false) == MATCH_OUTPUT;
       label_reachable_->ReachInit(fst, reach_input, copy);
@@ -551,6 +573,20 @@ class LabelLookAheadMatcher
   }
 
  private:
+  void Init(const FST &fst, MatchType match_type,
+            std::shared_ptr<MatcherData> data, Accumulator *accumulator) {
+    const bool reach_input = match_type == MATCH_INPUT;
+    if (data) {
+      if (reach_input == data->ReachInput()) {
+        label_reachable_ = fst::make_unique<Reachable>(data, accumulator);
+      }
+    } else if ((reach_input && (kFlags & kInputLookAheadMatcher)) ||
+               (!reach_input && (kFlags & kOutputLookAheadMatcher))) {
+      label_reachable_ = fst::make_unique<Reachable>(
+          fst, reach_input, accumulator, kFlags & kLookAheadKeepRelabelData);
+    }
+  }
+
   mutable M matcher_;
   const Fst<Arc> *lfst_;                        // Look-ahead FST.
   std::unique_ptr<Reachable> label_reachable_;  // Label reachability info.
@@ -565,7 +601,7 @@ template <class LFST>
 inline bool LabelLookAheadMatcher<M, flags, Accumulator,
                                   Reachable>::LookAheadFst(const LFST &fst,
                                                            StateId s) {
-  if (static_cast<const Fst<Arc> *>(&fst) != lfst_) InitLookAheadFst(fst);
+  if (&fst != lfst_) InitLookAheadFst(fst);
   ClearLookAheadWeight();
   ClearLookAheadPrefix();
   if (!label_reachable_) return true;
@@ -597,6 +633,34 @@ inline bool LabelLookAheadMatcher<M, flags, Accumulator,
   return reach_arc || reach_final;
 }
 
+// Relabels the fst with Reachable::Reachable.  Relabels input
+// if data.First() is non-null, otherwise relabels output.
+// Optionally saves the input/output label pairs to a file
+// if save_relabel_ipairs/opairs is non-empty.
+template <class Reachable, class FST, class Data>
+void RelabelForReachable(FST *fst, const Data &data,
+                         const std::string &save_relabel_ipairs,
+                         const std::string &save_relabel_opairs) {
+  using Label = typename FST::Arc::Label;
+  if (data.First() != nullptr) {  // reach_input.
+    Reachable reachable(data.SharedFirst());
+    reachable.Relabel(fst, /*relabel_input=*/true);
+    if (!save_relabel_ipairs.empty()) {
+      std::vector<std::pair<Label, Label>> pairs;
+      reachable.RelabelPairs(&pairs, /*avoid_collisions=*/true);
+      WriteLabelPairs(save_relabel_ipairs, pairs);
+    }
+  } else {
+    Reachable reachable(data.SharedSecond());
+    reachable.Relabel(fst, /*relabel_input=*/false);
+    if (!save_relabel_opairs.empty()) {
+      std::vector<std::pair<Label, Label>> pairs;
+      reachable.RelabelPairs(&pairs, /*avoid_collisions=*/true);
+      WriteLabelPairs(save_relabel_opairs, pairs);
+    }
+  }
+}
+
 // Label-lookahead relabeling class.
 template <class Arc, class Data = LabelReachableData<typename Arc::Label>>
 class LabelLookAheadRelabeler {
@@ -642,28 +706,21 @@ inline LabelLookAheadRelabeler<Arc, Data>::LabelLookAheadRelabeler(
   const bool is_mutable = fst.Properties(kMutable, false);
   std::unique_ptr<MutableFst<Arc>> mfst;
   if (is_mutable) {
+    // Borrow pointer from fst without increasing ref count; it will
+    // be released below. We do not want to call Copy() since that would
+    // do a deep copy when the Fst is modified.
     mfst.reset(static_cast<MutableFst<Arc> *>(&fst));
   } else {
-    mfst.reset(new VectorFst<Arc>(fst));
+    mfst = fst::make_unique<VectorFst<Arc>>(fst);
   }
-  if (data->First()) {  // reach_input.
-    Reachable reachable(data->SharedFirst());
-    reachable.Relabel(mfst.get(), true);
-    if (!FLAGS_save_relabel_ipairs.empty()) {
-      std::vector<std::pair<Label, Label>> pairs;
-      reachable.RelabelPairs(&pairs, true);
-      WriteLabelPairs(FLAGS_save_relabel_ipairs, pairs);
-    }
+
+  RelabelForReachable<Reachable>(mfst.get(), *data, FLAGS_save_relabel_ipairs,
+                                 FLAGS_save_relabel_opairs);
+
+  if (is_mutable) {
+    // Pointer was just borrowed, don't delete it.
+    mfst.release();
   } else {
-    Reachable reachable(data->SharedSecond());
-    reachable.Relabel(mfst.get(), false);
-    if (!FLAGS_save_relabel_opairs.empty()) {
-      std::vector<std::pair<Label, Label>> pairs;
-      reachable.RelabelPairs(&pairs, true);
-      WriteLabelPairs(FLAGS_save_relabel_opairs, pairs);
-    }
-  }
-  if (!is_mutable) {
     *impl = std::make_shared<Impl>(*mfst, name);
     (*impl)->SetAddOn(data);
   }
@@ -681,23 +738,32 @@ class LookAheadMatcher {
   using Weight = typename Arc::Weight;
   using LBase = LookAheadMatcherBase<Arc>;
 
+  // This makes a copy of the FST.
   LookAheadMatcher(const FST &fst, MatchType match_type)
-      : base_(fst.InitMatcher(match_type)) {
-    if (!base_) base_.reset(new SortedMatcher<FST>(fst, match_type));
-    lookahead_ = false;
+      : owned_fst_(fst.Copy()),
+        base_(owned_fst_->InitMatcher(match_type)),
+        lookahead_(false) {
+    if (!base_)
+      base_ =
+          fst::make_unique<SortedMatcher<FST>>(owned_fst_.get(), match_type);
   }
 
+  // This doesn't copy the FST.
+  LookAheadMatcher(const FST *fst, MatchType match_type)
+      : base_(fst->InitMatcher(match_type)), lookahead_(false) {
+    if (!base_) base_ = fst::make_unique<SortedMatcher<FST>>(fst, match_type);
+  }
+
+  // This makes a copy of the FST.
+  LookAheadMatcher(const LookAheadMatcher &matcher, bool safe = false)
+      : base_(matcher.base_->Copy(safe)), lookahead_(matcher.lookahead_) {}
+
   // Takes ownership of base.
   explicit LookAheadMatcher(MatcherBase<Arc> *base)
       : base_(base), lookahead_(false) {}
 
-  LookAheadMatcher(const LookAheadMatcher<FST> &matcher, bool safe = false)
-      : base_(matcher.base_->Copy(safe)) {
-    lookahead_ = matcher.lookahead_;
-  }
-
-  LookAheadMatcher<FST> *Copy(bool safe = false) const {
-    return new LookAheadMatcher<FST>(*this, safe);
+  LookAheadMatcher *Copy(bool safe = false) const {
+    return new LookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const { return base_->Type(test); }
@@ -716,9 +782,7 @@ class LookAheadMatcher {
 
   ssize_t Priority(StateId s) { return base_->Priority(s); }
 
-  const FST &GetFst() const {
-    return static_cast<const FST &>(base_->GetFst());
-  }
+  const FST &GetFst() const { return static_cast<const FST &>(base_->GetFst()); }
 
   uint64 Properties(uint64 props) const { return base_->Properties(props); }
 
@@ -774,6 +838,7 @@ class LookAheadMatcher {
     return lookahead_;
   }
 
+  std::unique_ptr<const FST> owned_fst_;
   std::unique_ptr<MatcherBase<Arc>> base_;
   mutable bool lookahead_;
 
@@ -782,4 +847,4 @@ class LookAheadMatcher {
 
 }  // namespace fst
 
-#endif  // FST_LIB_LOOKAHEAD_MATCHER_H_
+#endif  // FST_LOOKAHEAD_MATCHER_H_
