@@ -3,18 +3,19 @@
 //
 // Classes to allow matching labels leaving FST states.
 
-#ifndef FST_LIB_MATCHER_H_
-#define FST_LIB_MATCHER_H_
+#ifndef FST_MATCHER_H_
+#define FST_MATCHER_H_
 
 #include <algorithm>
+#include <memory>
 #include <unordered_map>
 #include <utility>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/mutable-fst.h>  // for all internal FST accessors.
 
-
 namespace fst {
 
 // Matchers find and iterate through requested labels at FST states. In the
@@ -32,20 +33,33 @@ namespace fst {
 //   using StateId = typename Arc::StateId;
 //   using Weight = typename Arc::Weight;
 //
-//   // Required constructors.
+//   // Required constructors. Note:
+//   // -- the constructors that copy the FST arg are useful for
+//   // letting the matcher manage the FST through copies
+//   // (esp with 'safe' copies); e.g. ComposeFst depends on this.
+//   // -- the constructor that does not copy is useful when the
+//   // the FST is mutated during the lifetime of the matcher
+//   // (o.w. the matcher would have its own unmutated deep copy).
 //
+//   // This makes a copy of the FST.
 //   Matcher(const FST &fst, MatchType type);
+//   // This doesn't copy the FST.
+//   Matcher(const FST *fst, MatchType type);
+//   // This makes a copy of the FST.
+//   // See Copy() below.
 //   Matcher(const Matcher &matcher, bool safe = false);
 //
-//   // Standard copy method.
-//   Matcher<FST> *Copy(bool safe = false) const override;
+//   // If safe = true, the copy is thread-safe. See Fst<>::Copy() for
+//   // further doc.
+//   Matcher *Copy(bool safe = false) const override;
 //
 //   // Returns the match type that can be provided (depending on compatibility
-//   of the input FST). It is either the requested match type, MATCH_NONE, or
-//   MATCH_UNKNOWN. If test is false, a costly testing is avoided, but
-//   MATCH_UNKNOWN may be returned. If test is true, a definite answer is
-//   returned, but may involve more costly computation (e.g., visiting the FST).
-//   MatchType Type(bool test) const override;
+//   // of the input FST). It is either the requested match type, MATCH_NONE,
+//   // or MATCH_UNKNOWN. If test is false, a costly testing is avoided, but
+//   // MATCH_UNKNOWN may be returned. If test is true, a definite answer is
+//   // returned, but may involve more costly computation (e.g., visiting
+//   // the FST).
+//   // MatchType Type(bool test) const override;
 //
 //   // Specifies the current state.
 //   void SetState(StateId s) final;
@@ -56,7 +70,7 @@ namespace fst {
 //   bool Find(Label label) final;
 //
 //   // Iterator methods. Note that initially and after SetState() these have
-//   undefined behavior until Find() is called.
+//   // undefined behavior until Find() is called.
 //
 //   bool Done() const final;
 //
@@ -110,7 +124,7 @@ class MatcherBase {
 
   // Virtual interface.
 
-  virtual MatcherBase<Arc> *Copy(bool safe = false) const = 0;
+  virtual MatcherBase *Copy(bool safe = false) const = 0;
   virtual MatchType Type(bool) const = 0;
   virtual void SetState(StateId) = 0;
   virtual bool Find(Label) = 0;
@@ -148,8 +162,17 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
 
   // Labels >= binary_label will be searched for by binary search;
   // o.w. linear search is used.
+  // This makes a copy of the FST.
   SortedMatcher(const FST &fst, MatchType match_type, Label binary_label = 1)
-      : fst_(fst.Copy()),
+      : SortedMatcher(fst.Copy(), match_type, binary_label) {
+    owned_fst_.reset(&fst_);
+  }
+
+  // Labels >= binary_label will be searched for by binary search;
+  // o.w. linear search is used.
+  // This doesn't copy the FST.
+  SortedMatcher(const FST *fst, MatchType match_type, Label binary_label = 1)
+      : fst_(*fst),
         state_(kNoStateId),
         aiter_(nullptr),
         match_type_(match_type),
@@ -173,8 +196,10 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
     }
   }
 
-  SortedMatcher(const SortedMatcher<FST> &matcher, bool safe = false)
-      : fst_(matcher.fst_->Copy(safe)),
+  // This makes a copy of the FST.
+  SortedMatcher(const SortedMatcher &matcher, bool safe = false)
+      : owned_fst_(matcher.fst_.Copy(safe)),
+        fst_(*owned_fst_),
         state_(kNoStateId),
         aiter_(nullptr),
         match_type_(matcher.match_type_),
@@ -187,8 +212,8 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
 
   ~SortedMatcher() override { Destroy(aiter_, &aiter_pool_); }
 
-  SortedMatcher<FST> *Copy(bool safe = false) const override {
-    return new SortedMatcher<FST>(*this, safe);
+  SortedMatcher *Copy(bool safe = false) const override {
+    return new SortedMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override {
@@ -197,7 +222,7 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
         match_type_ == MATCH_INPUT ? kILabelSorted : kOLabelSorted;
     const auto false_prop =
         match_type_ == MATCH_INPUT ? kNotILabelSorted : kNotOLabelSorted;
-    const auto props = fst_->Properties(true_prop | false_prop, test);
+    const auto props = fst_.Properties(true_prop | false_prop, test);
     if (props & true_prop) {
       return match_type_;
     } else if (props & false_prop) {
@@ -215,9 +240,9 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
       error_ = true;
     }
     Destroy(aiter_, &aiter_pool_);
-    aiter_ = new (&aiter_pool_) ArcIterator<FST>(*fst_, s);
+    aiter_ = new (&aiter_pool_) ArcIterator<FST>(fst_, s);
     aiter_->SetFlags(kArcNoCache, kArcNoCache);
-    narcs_ = internal::NumArcs(*fst_, s);
+    narcs_ = internal::NumArcs(fst_, s);
     loop_.nextstate = s;
   }
 
@@ -259,9 +284,7 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
     aiter_->SetFlags(
         match_type_ == MATCH_INPUT ? kArcILabelValue : kArcOLabelValue,
         kArcValueFlags);
-    const auto label = match_type_ == MATCH_INPUT ? aiter_->Value().ilabel
-                                                  : aiter_->Value().olabel;
-    return label != match_label_;
+    return GetLabel() != match_label_;
   }
 
   const Arc &Value() const final {
@@ -278,15 +301,11 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
     }
   }
 
-  Weight Final(StateId s) const final {
-    return MatcherBase<Arc>::Final(s);
-  }
+  Weight Final(StateId s) const final { return MatcherBase<Arc>::Final(s); }
 
-  ssize_t Priority(StateId s) final {
-    return MatcherBase<Arc>::Priority(s);
-  }
+  ssize_t Priority(StateId s) final { return MatcherBase<Arc>::Priority(s); }
 
-  const FST &GetFst() const override { return *fst_; }
+  const FST &GetFst() const override { return fst_; }
 
   uint64 Properties(uint64 inprops) const override {
     return inprops | (error_ ? kError : 0);
@@ -295,22 +314,74 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
   size_t Position() const { return aiter_ ? aiter_->Position() : 0; }
 
  private:
+  Label GetLabel() const {
+    const auto &arc = aiter_->Value();
+    return match_type_ == MATCH_INPUT ? arc.ilabel : arc.olabel;
+  }
+
+  bool BinarySearch();
+  bool LinearSearch();
   bool Search();
 
-  std::unique_ptr<const FST> fst_;
-  StateId state_;            // Matcher state.
-  ArcIterator<FST> *aiter_;  // Iterator for current state.
-  MatchType match_type_;     // Type of match to perform.
-  Label binary_label_;       // Least label for binary search.
-  Label match_label_;        // Current label to be matched.
-  size_t narcs_;             // Current state arc count.
-  Arc loop_;                 // For non-consuming symbols.
-  bool current_loop_;        // Current arc is the implicit loop.
-  bool exact_match_;         // Exact match or lower bound?
-  bool error_;               // Error encountered?
+  std::unique_ptr<const FST> owned_fst_;  // FST ptr if owned.
+  const FST &fst_;                        // FST for matching.
+  StateId state_;                         // Matcher state.
+  ArcIterator<FST> *aiter_;               // Iterator for current state.
+  MatchType match_type_;                  // Type of match to perform.
+  Label binary_label_;                    // Least label for binary search.
+  Label match_label_;                     // Current label to be matched.
+  size_t narcs_;                          // Current state arc count.
+  Arc loop_;                              // For non-consuming symbols.
+  bool current_loop_;                     // Current arc is the implicit loop.
+  bool exact_match_;                      // Exact match or lower bound?
+  bool error_;                            // Error encountered?
   MemoryPool<ArcIterator<FST>> aiter_pool_;  // Pool of arc iterators.
 };
 
+// Returns true iff match to match_label_. The arc iterator is positioned at the
+// lower bound, that is, the first element greater than or equal to
+// match_label_, or the end if all elements are less than match_label_.
+// If multiple elements are equal to the `match_label_`, returns the rightmost
+// one.
+template <class FST>
+inline bool SortedMatcher<FST>::BinarySearch() {
+  size_t size = narcs_;
+  if (size == 0) {
+    return false;
+  }
+  size_t high = size - 1;
+  while (size > 1) {
+    const size_t half = size / 2;
+    const size_t mid = high - half;
+    aiter_->Seek(mid);
+    if (GetLabel() >= match_label_) {
+      high = mid;
+    }
+    size -= half;
+  }
+  aiter_->Seek(high);
+  const auto label = GetLabel();
+  if (label == match_label_) {
+    return true;
+  }
+  if (label < match_label_) {
+    aiter_->Next();
+  }
+  return false;
+}
+
+// Returns true iff match to match_label_, positioning arc iterator at lower
+// bound.
+template <class FST>
+inline bool SortedMatcher<FST>::LinearSearch() {
+  for (aiter_->Reset(); !aiter_->Done(); aiter_->Next()) {
+    const auto label = GetLabel();
+    if (label == match_label_) return true;
+    if (label > match_label_) break;
+  }
+  return false;
+}
+
 // Returns true iff match to match_label_, positioning arc iterator at lower
 // bound.
 template <class FST>
@@ -319,43 +390,9 @@ inline bool SortedMatcher<FST>::Search() {
       match_type_ == MATCH_INPUT ? kArcILabelValue : kArcOLabelValue,
       kArcValueFlags);
   if (match_label_ >= binary_label_) {
-    // Binary search for match.
-    size_t low = 0;
-    size_t high = narcs_;
-    while (low < high) {
-      const size_t mid = (low + high) / 2;
-      aiter_->Seek(mid);
-      auto label = match_type_ == MATCH_INPUT ? aiter_->Value().ilabel
-                                              : aiter_->Value().olabel;
-      if (label > match_label_) {
-        high = mid;
-      } else if (label < match_label_) {
-        low = mid + 1;
-      } else {
-        // Finds first matching label (when non-deterministic).
-        for (size_t i = mid; i > low; --i) {
-          aiter_->Seek(i - 1);
-          label = match_type_ == MATCH_INPUT ? aiter_->Value().ilabel
-                                             : aiter_->Value().olabel;
-          if (label != match_label_) {
-            aiter_->Seek(i);
-            return true;
-          }
-        }
-        return true;
-      }
-    }
-    aiter_->Seek(low);
-    return false;
+    return BinarySearch();
   } else {
-    // Linear search for match.
-    for (aiter_->Reset(); !aiter_->Done(); aiter_->Next()) {
-      const auto label = match_type_ == MATCH_INPUT ? aiter_->Value().ilabel
-                                                    : aiter_->Value().olabel;
-      if (label == match_label_) return true;
-      if (label > match_label_) break;
-    }
-    return false;
+    return LinearSearch();
   }
 }
 
@@ -375,11 +412,20 @@ class HashMatcher : public MatcherBase<typename F::Arc> {
   using MatcherBase<Arc>::Final;
   using MatcherBase<Arc>::Priority;
 
+  // This makes a copy of the FST.
   HashMatcher(const FST &fst, MatchType match_type)
-      : fst_(fst.Copy()),
+      : HashMatcher(fst.Copy(), match_type) {
+    owned_fst_.reset(&fst_);
+  }
+
+  // This doesn't copy the FST.
+  HashMatcher(const FST *fst, MatchType match_type)
+      : fst_(*fst),
         state_(kNoStateId),
         match_type_(match_type),
-        loop_(kNoLabel, 0, Weight::One(), kNoStateId) {
+        loop_(kNoLabel, 0, Weight::One(), kNoStateId),
+        error_(false),
+        state_table_(std::make_shared<StateTable>()) {
     switch (match_type_) {
       case MATCH_INPUT:
       case MATCH_NONE:
@@ -388,21 +434,25 @@ class HashMatcher : public MatcherBase<typename F::Arc> {
         std::swap(loop_.ilabel, loop_.olabel);
         break;
       default:
-        FSTERROR() << "SortedMatcher: Bad match type";
+        FSTERROR() << "HashMatcher: Bad match type";
         match_type_ = MATCH_NONE;
         error_ = true;
     }
   }
 
-  HashMatcher(const HashMatcher<FST> &matcher, bool safe = false)
-      : fst_(matcher.fst_->Copy(safe)),
+  // This makes a copy of the FST.
+  HashMatcher(const HashMatcher &matcher, bool safe = false)
+      : owned_fst_(matcher.fst_.Copy(safe)),
+        fst_(*owned_fst_),
         state_(kNoStateId),
         match_type_(matcher.match_type_),
         loop_(matcher.loop_),
-        error_(matcher.error_) {}
+        error_(matcher.error_),
+        state_table_(safe ? std::make_shared<StateTable>()
+                          : matcher.state_table_) {}
 
-  HashMatcher<FST> *Copy(bool safe = false) const override {
-    return new HashMatcher<FST>(*this, safe);
+  HashMatcher *Copy(bool safe = false) const override {
+    return new HashMatcher(*this, safe);
   }
 
   // The argument is ignored as there are no relevant properties to test.
@@ -439,26 +489,32 @@ class HashMatcher : public MatcherBase<typename F::Arc> {
     }
   }
 
-  const FST &GetFst() const override { return *fst_; }
+  const FST &GetFst() const override { return fst_; }
 
   uint64 Properties(uint64 inprops) const override {
     return inprops | (error_ ? kError : 0);
   }
 
  private:
+  Label GetLabel() const {
+    const auto &arc = aiter_->Value();
+    return match_type_ == MATCH_INPUT ? arc.ilabel : arc.olabel;
+  }
+
   bool Search(Label match_label);
 
   using LabelTable = std::unordered_multimap<Label, size_t>;
-  using StateTable = std::unordered_map<StateId, LabelTable>;
+  using StateTable = std::unordered_map<StateId, std::unique_ptr<LabelTable>>;
 
-  std::unique_ptr<const FST> fst_;
-  StateId state_;  // Matcher state.
+  std::unique_ptr<const FST> owned_fst_;  // ptr to FST if owned.
+  const FST &fst_;                        // FST for matching.
+  StateId state_;                         // Matcher state.
   MatchType match_type_;
   Arc loop_;           // The implicit loop itself.
-  bool current_loop_;  // Is the current arc is the implicit loop?
+  bool current_loop_;  // Is the current arc the implicit loop?
   bool error_;         // Error encountered?
   std::unique_ptr<ArcIterator<FST>> aiter_;
-  StateTable state_table_;   // Table from states to label table.
+  std::shared_ptr<StateTable> state_table_;  // Table from state to label table.
   LabelTable *label_table_;  // Pointer to current state's label table.
   typename LabelTable::iterator label_it_;   // Position for label.
   typename LabelTable::iterator label_end_;  // Position for last label + 1.
@@ -470,28 +526,27 @@ void HashMatcher<FST>::SetState(typename FST::Arc::StateId s) {
   // Resets everything for the state.
   state_ = s;
   loop_.nextstate = state_;
-  aiter_.reset(new ArcIterator<FST>(*fst_, state_));
+  aiter_ = fst::make_unique<ArcIterator<FST>>(fst_, state_);
   if (match_type_ == MATCH_NONE) {
     FSTERROR() << "HashMatcher: Bad match type";
-    error_ = false;
+    error_ = true;
   }
-  // Attempts to insert a new label table; if it already exists,
-  // no additional work is done and we simply return.
-  auto it_and_success = state_table_.emplace(state_, LabelTable());
+  // Attempts to insert a new label table.
+  auto it_and_success = state_table_->emplace(
+      state_, std::unique_ptr<LabelTable>(new LabelTable()));
+  // Sets instance's pointer to the label table for this state.
+  label_table_ = it_and_success.first->second.get();
+  // If it already exists, no additional work is done and we simply return.
   if (!it_and_success.second) return;
   // Otherwise, populate this new table.
-  // Sets instance's pointer to the label table for this state.
-  label_table_ = &(it_and_success.first->second);
   // Populates the label table.
-  label_table_->reserve(internal::NumArcs(*fst_, state_));
+  label_table_->reserve(internal::NumArcs(fst_, state_));
   const auto aiter_flags =
       (match_type_ == MATCH_INPUT ? kArcILabelValue : kArcOLabelValue) |
       kArcNoCache;
   aiter_->SetFlags(aiter_flags, kArcFlags);
   for (; !aiter_->Done(); aiter_->Next()) {
-    const auto label = (match_type_ == MATCH_INPUT) ? aiter_->Value().ilabel
-                                                    : aiter_->Value().olabel;
-    label_table_->emplace(label, aiter_->Position());
+    label_table_->emplace(GetLabel(), aiter_->Position());
   }
   aiter_->SetFlags(kArcValueFlags, kArcValueFlags);
 }
@@ -499,9 +554,9 @@ void HashMatcher<FST>::SetState(typename FST::Arc::StateId s) {
 template <class FST>
 inline bool HashMatcher<FST>::Search(typename FST::Arc::Label match_label) {
   auto range = label_table_->equal_range(match_label);
-  if (range.first == range.second) return false;
   label_it_ = range.first;
   label_end_ = range.second;
+  if (label_it_ == label_end_) return false;
   aiter_->Seek(label_it_->second);
   return true;
 }
@@ -537,6 +592,7 @@ class PhiMatcher : public MatcherBase<typename M::Arc> {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  // This makes a copy of the FST (w/o 'matcher' arg).
   PhiMatcher(const FST &fst, MatchType match_type, Label phi_label = kNoLabel,
              bool phi_loop = true,
              MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
@@ -561,7 +617,16 @@ class PhiMatcher : public MatcherBase<typename M::Arc> {
     }
   }
 
-  PhiMatcher(const PhiMatcher<M> &matcher, bool safe = false)
+  // This doesn't copy the FST.
+  PhiMatcher(const FST *fst, MatchType match_type, Label phi_label = kNoLabel,
+             bool phi_loop = true,
+             MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
+             M *matcher = nullptr)
+      : PhiMatcher(*fst, match_type, phi_label, phi_loop, rewrite_mode,
+                   matcher ? matcher : new M(fst, match_type)) {}
+
+  // This makes a copy of the FST.
+  PhiMatcher(const PhiMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         phi_label_(matcher.phi_label_),
@@ -570,8 +635,8 @@ class PhiMatcher : public MatcherBase<typename M::Arc> {
         phi_loop_(matcher.phi_loop_),
         error_(matcher.error_) {}
 
-  PhiMatcher<M> *Copy(bool safe = false) const override {
-    return new PhiMatcher<M>(*this, safe);
+  PhiMatcher *Copy(bool safe = false) const override {
+    return new PhiMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -731,7 +796,7 @@ inline uint64 PhiMatcher<M>::Properties(uint64 inprops) const {
     return outprops;
   } else if (match_type_ == MATCH_INPUT) {
     if (phi_label_ == 0) {
-      outprops &= ~kEpsilons | ~kIEpsilons | ~kOEpsilons;
+      outprops &= ~(kEpsilons | kIEpsilons | kOEpsilons);
       outprops |= kNoEpsilons | kNoIEpsilons;
     }
     if (rewrite_both_) {
@@ -745,7 +810,7 @@ inline uint64 PhiMatcher<M>::Properties(uint64 inprops) const {
     }
   } else if (match_type_ == MATCH_OUTPUT) {
     if (phi_label_ == 0) {
-      outprops &= ~kEpsilons | ~kIEpsilons | ~kOEpsilons;
+      outprops &= ~(kEpsilons | kIEpsilons | kOEpsilons);
       outprops |= kNoEpsilons | kNoOEpsilons;
     }
     if (rewrite_both_) {
@@ -786,6 +851,7 @@ class RhoMatcher : public MatcherBase<typename M::Arc> {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  // This makes a copy of the FST (w/o 'matcher' arg).
   RhoMatcher(const FST &fst, MatchType match_type, Label rho_label = kNoLabel,
              MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
              M *matcher = nullptr)
@@ -793,7 +859,8 @@ class RhoMatcher : public MatcherBase<typename M::Arc> {
         match_type_(match_type),
         rho_label_(rho_label),
         error_(false),
-        state_(kNoStateId) {
+        state_(kNoStateId),
+        has_rho_(false) {
     if (match_type == MATCH_BOTH) {
       FSTERROR() << "RhoMatcher: Bad match type";
       match_type_ = MATCH_NONE;
@@ -813,16 +880,25 @@ class RhoMatcher : public MatcherBase<typename M::Arc> {
     }
   }
 
-  RhoMatcher(const RhoMatcher<M> &matcher, bool safe = false)
+  // This doesn't copy the FST.
+  RhoMatcher(const FST *fst, MatchType match_type, Label rho_label = kNoLabel,
+             MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
+             M *matcher = nullptr)
+      : RhoMatcher(*fst, match_type, rho_label, rewrite_mode,
+                   matcher ? matcher : new M(fst, match_type)) {}
+
+  // This makes a copy of the FST.
+  RhoMatcher(const RhoMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         rho_label_(matcher.rho_label_),
         rewrite_both_(matcher.rewrite_both_),
         error_(matcher.error_),
-        state_(kNoStateId) {}
+        state_(kNoStateId),
+        has_rho_(false) {}
 
-  RhoMatcher<M> *Copy(bool safe = false) const override {
-    return new RhoMatcher<M>(*this, safe);
+  RhoMatcher *Copy(bool safe = false) const override {
+    return new RhoMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -904,11 +980,11 @@ class RhoMatcher : public MatcherBase<typename M::Arc> {
   MatchType match_type_;  // Type of match requested.
   Label rho_label_;       // Label that represents the rho transition
   bool rewrite_both_;     // Rewrite both sides when both are rho_label_?
-  bool has_rho_;          // Are there possibly rhos at the current state?
   Label rho_match_;       // Current label that matches rho transition.
   mutable Arc rho_arc_;   // Arc to return when rho match.
   bool error_;            // Error encountered?
   StateId state_;         // Matcher state.
+  bool has_rho_;          // Are there possibly rhos at the current state?
 };
 
 template <class M>
@@ -923,9 +999,8 @@ inline uint64 RhoMatcher<M>::Properties(uint64 inprops) const {
              ~(kODeterministic | kNonODeterministic | kString | kILabelSorted |
                kNotILabelSorted | kOLabelSorted | kNotOLabelSorted);
     } else {
-      return outprops &
-             ~(kODeterministic | kAcceptor | kString | kILabelSorted |
-               kNotILabelSorted);
+      return outprops & ~(kODeterministic | kAcceptor | kString |
+                          kILabelSorted | kNotILabelSorted);
     }
   } else if (match_type_ == MATCH_OUTPUT) {
     if (rewrite_both_) {
@@ -933,9 +1008,8 @@ inline uint64 RhoMatcher<M>::Properties(uint64 inprops) const {
              ~(kIDeterministic | kNonIDeterministic | kString | kILabelSorted |
                kNotILabelSorted | kOLabelSorted | kNotOLabelSorted);
     } else {
-      return outprops &
-             ~(kIDeterministic | kAcceptor | kString | kOLabelSorted |
-               kNotOLabelSorted);
+      return outprops & ~(kIDeterministic | kAcceptor | kString |
+                          kOLabelSorted | kNotOLabelSorted);
     }
   } else {
     // Shouldn't ever get here.
@@ -966,6 +1040,7 @@ class SigmaMatcher : public MatcherBase<typename M::Arc> {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  // This makes a copy of the FST (w/o 'matcher' arg).
   SigmaMatcher(const FST &fst, MatchType match_type,
                Label sigma_label = kNoLabel,
                MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
@@ -994,7 +1069,16 @@ class SigmaMatcher : public MatcherBase<typename M::Arc> {
     }
   }
 
-  SigmaMatcher(const SigmaMatcher<M> &matcher, bool safe = false)
+  // This doesn't copy the FST.
+  SigmaMatcher(const FST *fst, MatchType match_type,
+               Label sigma_label = kNoLabel,
+               MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
+               M *matcher = nullptr)
+      : SigmaMatcher(*fst, match_type, sigma_label, rewrite_mode,
+                     matcher ? matcher : new M(fst, match_type)) {}
+
+  // This makes a copy of the FST.
+  SigmaMatcher(const SigmaMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         sigma_label_(matcher.sigma_label_),
@@ -1002,8 +1086,8 @@ class SigmaMatcher : public MatcherBase<typename M::Arc> {
         error_(matcher.error_),
         state_(kNoStateId) {}
 
-  SigmaMatcher<M> *Copy(bool safe = false) const override {
-    return new SigmaMatcher<M>(*this, safe);
+  SigmaMatcher *Copy(bool safe = false) const override {
+    return new SigmaMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -1107,20 +1191,17 @@ inline uint64 SigmaMatcher<M>::Properties(uint64 inprops) const {
   if (match_type_ == MATCH_NONE) {
     return outprops;
   } else if (rewrite_both_) {
-    return outprops &
-           ~(kIDeterministic | kNonIDeterministic | kODeterministic |
-             kNonODeterministic | kILabelSorted | kNotILabelSorted |
-             kOLabelSorted | kNotOLabelSorted | kString);
+    return outprops & ~(kIDeterministic | kNonIDeterministic | kODeterministic |
+                        kNonODeterministic | kILabelSorted | kNotILabelSorted |
+                        kOLabelSorted | kNotOLabelSorted | kString);
   } else if (match_type_ == MATCH_INPUT) {
-    return outprops &
-           ~(kIDeterministic | kNonIDeterministic | kODeterministic |
-             kNonODeterministic | kILabelSorted | kNotILabelSorted | kString |
-             kAcceptor);
+    return outprops & ~(kIDeterministic | kNonIDeterministic | kODeterministic |
+                        kNonODeterministic | kILabelSorted | kNotILabelSorted |
+                        kString | kAcceptor);
   } else if (match_type_ == MATCH_OUTPUT) {
-    return outprops &
-           ~(kIDeterministic | kNonIDeterministic | kODeterministic |
-             kNonODeterministic | kOLabelSorted | kNotOLabelSorted | kString |
-             kAcceptor);
+    return outprops & ~(kIDeterministic | kNonIDeterministic | kODeterministic |
+                        kNonODeterministic | kOLabelSorted | kNotOLabelSorted |
+                        kString | kAcceptor);
   } else {
     // Shouldn't ever get here.
     FSTERROR() << "SigmaMatcher: Bad match type: " << match_type_;
@@ -1152,24 +1233,28 @@ class MultiEpsMatcher {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  // This makes a copy of the FST (w/o 'matcher' arg).
   MultiEpsMatcher(const FST &fst, MatchType match_type,
                   uint32 flags = (kMultiEpsLoop | kMultiEpsList),
                   M *matcher = nullptr, bool own_matcher = true)
       : matcher_(matcher ? matcher : new M(fst, match_type)),
         flags_(flags),
         own_matcher_(matcher ? own_matcher : true) {
-    if (match_type == MATCH_INPUT) {
-      loop_.ilabel = kNoLabel;
-      loop_.olabel = 0;
-    } else {
-      loop_.ilabel = 0;
-      loop_.olabel = kNoLabel;
-    }
-    loop_.weight = Weight::One();
-    loop_.nextstate = kNoStateId;
+    Init(match_type);
   }
 
-  MultiEpsMatcher(const MultiEpsMatcher<M> &matcher, bool safe = false)
+  // This doesn't copy the FST.
+  MultiEpsMatcher(const FST *fst, MatchType match_type,
+                  uint32 flags = (kMultiEpsLoop | kMultiEpsList),
+                  M *matcher = nullptr, bool own_matcher = true)
+      : matcher_(matcher ? matcher : new M(fst, match_type)),
+        flags_(flags),
+        own_matcher_(matcher ? own_matcher : true) {
+    Init(match_type);
+  }
+
+  // This makes a copy of the FST.
+  MultiEpsMatcher(const MultiEpsMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         flags_(matcher.flags_),
         own_matcher_(true),
@@ -1182,8 +1267,8 @@ class MultiEpsMatcher {
     if (own_matcher_) delete matcher_;
   }
 
-  MultiEpsMatcher<M> *Copy(bool safe = false) const {
-    return new MultiEpsMatcher<M>(*this, safe);
+  MultiEpsMatcher *Copy(bool safe = false) const {
+    return new MultiEpsMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const { return matcher_->Type(test); }
@@ -1251,6 +1336,18 @@ class MultiEpsMatcher {
   void ClearMultiEpsLabels() { multi_eps_labels_.Clear(); }
 
  private:
+  void Init(MatchType match_type) {
+    if (match_type == MATCH_INPUT) {
+      loop_.ilabel = kNoLabel;
+      loop_.olabel = 0;
+    } else {
+      loop_.ilabel = 0;
+      loop_.olabel = kNoLabel;
+    }
+    loop_.weight = Weight::One();
+    loop_.nextstate = kNoStateId;
+  }
+
   M *matcher_;
   uint32 flags_;
   bool own_matcher_;  // Does this class delete the matcher?
@@ -1318,18 +1415,26 @@ class ExplicitMatcher : public MatcherBase<typename M::Arc> {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  // This makes a copy of the FST.
   ExplicitMatcher(const FST &fst, MatchType match_type, M *matcher = nullptr)
       : matcher_(matcher ? matcher : new M(fst, match_type)),
         match_type_(match_type),
         error_(false) {}
 
-  ExplicitMatcher(const ExplicitMatcher<M> &matcher, bool safe = false)
+  // This doesn't copy the FST.
+  ExplicitMatcher(const FST *fst, MatchType match_type, M *matcher = nullptr)
+      : matcher_(matcher ? matcher : new M(fst, match_type)),
+        match_type_(match_type),
+        error_(false) {}
+
+  // This makes a copy of the FST.
+  ExplicitMatcher(const ExplicitMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         error_(matcher.error_) {}
 
-  ExplicitMatcher<M> *Copy(bool safe = false) const override {
-    return new ExplicitMatcher<M>(*this, safe);
+  ExplicitMatcher *Copy(bool safe = false) const override {
+    return new ExplicitMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -1401,23 +1506,28 @@ class Matcher {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
-  Matcher(const FST &fst, MatchType match_type) {
-    base_.reset(fst.InitMatcher(match_type));
-    if (!base_) base_.reset(new SortedMatcher<FST>(fst, match_type));
+  // This makes a copy of the FST.
+  Matcher(const FST &fst, MatchType match_type)
+      : owned_fst_(fst.Copy()), base_(owned_fst_->InitMatcher(match_type)) {
+    if (!base_)
+      base_ =
+          fst::make_unique<SortedMatcher<FST>>(owned_fst_.get(), match_type);
   }
 
-  Matcher(const Matcher<FST> &matcher, bool safe = false) {
-    base_.reset(matcher.base_->Copy(safe));
+  // This doesn't copy the FST.
+  Matcher(const FST *fst, MatchType match_type)
+      : base_(fst->InitMatcher(match_type)) {
+    if (!base_) base_ = fst::make_unique<SortedMatcher<FST>>(fst, match_type);
   }
 
+  // This makes a copy of the FST.
+  Matcher(const Matcher &matcher, bool safe = false)
+      : base_(matcher.base_->Copy(safe)) {}
+
   // Takes ownership of the provided matcher.
-  explicit Matcher(MatcherBase<Arc> *base_matcher) {
-    base_.reset(base_matcher);
-  }
+  explicit Matcher(MatcherBase<Arc> *base_matcher) : base_(base_matcher) {}
 
-  Matcher<FST> *Copy(bool safe = false) const {
-    return new Matcher<FST>(*this, safe);
-  }
+  Matcher *Copy(bool safe = false) const { return new Matcher(*this, safe); }
 
   MatchType Type(bool test) const { return base_->Type(test); }
 
@@ -1431,9 +1541,7 @@ class Matcher {
 
   void Next() { base_->Next(); }
 
-  const FST &GetFst() const {
-    return static_cast<const FST &>(base_->GetFst());
-  }
+  const FST &GetFst() const { return static_cast<const FST &>(base_->GetFst()); }
 
   uint64 Properties(uint64 props) const { return base_->Properties(props); }
 
@@ -1444,9 +1552,10 @@ class Matcher {
   ssize_t Priority(StateId s) { return base_->Priority(s); }
 
  private:
+  std::unique_ptr<const FST> owned_fst_;
   std::unique_ptr<MatcherBase<Arc>> base_;
 };
 
 }  // namespace fst
 
-#endif  // FST_LIB_MATCHER_H_
+#endif  // FST_MATCHER_H_
