@@ -4,17 +4,18 @@
 // Expanded FST augmented with mutators; interface class definition and
 // mutable arc iterator interface.
 
-#ifndef FST_LIB_MUTABLE_FST_H_
-#define FST_LIB_MUTABLE_FST_H_
+#ifndef FST_MUTABLE_FST_H_
+#define FST_MUTABLE_FST_H_
 
-#include <stddef.h>
 #include <sys/types.h>
 
+#include <cstddef>
 #include <istream>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 #include <fstream>
 
@@ -37,7 +38,7 @@ class MutableFst : public ExpandedFst<A> {
 
   virtual MutableFst<Arc> &operator=(const Fst<Arc> &fst) = 0;
 
-  MutableFst<Arc> &operator=(const MutableFst<Arc> &fst) {
+  MutableFst &operator=(const MutableFst &fst) {
     return operator=(static_cast<const Fst<Arc> &>(fst));
   }
 
@@ -45,7 +46,7 @@ class MutableFst : public ExpandedFst<A> {
   virtual void SetStart(StateId) = 0;
 
   // Sets a state's final weight.
-  virtual void SetFinal(StateId, Weight) = 0;
+  virtual void SetFinal(StateId s, Weight weight = Weight::One()) = 0;
 
   // Sets property bits w.r.t. mask.
   virtual void SetProperties(uint64 props, uint64 mask) = 0;
@@ -53,8 +54,15 @@ class MutableFst : public ExpandedFst<A> {
   // Adds a state and returns its ID.
   virtual StateId AddState() = 0;
 
+  // Adds multiple states.
+  virtual void AddStates(size_t) = 0;
+
   // Adds an arc to state.
-  virtual void AddArc(StateId, const Arc &arc) = 0;
+  virtual void AddArc(StateId, const Arc &) = 0;
+
+  // Adds an arc (passed by rvalue reference) to state. Allows subclasses
+  // to optionally implement move semantics. Defaults to lvalue overload.
+  virtual void AddArc(StateId state, Arc &&arc) { AddArc(state, arc); }
 
   // Deletes some states, preserving original StateId ordering.
   virtual void DeleteStates(const std::vector<StateId> &) = 0;
@@ -63,16 +71,16 @@ class MutableFst : public ExpandedFst<A> {
   virtual void DeleteStates() = 0;
 
   // Delete some arcs at a given state.
-  virtual void DeleteArcs(StateId, size_t n) = 0;
+  virtual void DeleteArcs(StateId, size_t) = 0;
 
   // Delete all arcs at a given state.
   virtual void DeleteArcs(StateId) = 0;
 
   // Optional, best effort only.
-  virtual void ReserveStates(StateId n) {}
+  virtual void ReserveStates(size_t) {}
 
   // Optional, best effort only.
-  virtual void ReserveArcs(StateId s, size_t n) {}
+  virtual void ReserveArcs(StateId, size_t) {}
 
   // Returns input label symbol table or nullptr if not specified.
   const SymbolTable *InputSymbols() const override = 0;
@@ -93,10 +101,10 @@ class MutableFst : public ExpandedFst<A> {
   virtual void SetOutputSymbols(const SymbolTable *osyms) = 0;
 
   // Gets a copy of this MutableFst. See Fst<>::Copy() for further doc.
-  MutableFst<A> *Copy(bool safe = false) const override = 0;
+  MutableFst *Copy(bool safe = false) const override = 0;
 
   // Reads a MutableFst from an input stream, returning nullptr on error.
-  static MutableFst<Arc> *Read(std::istream &strm, const FstReadOptions &opts) {
+  static MutableFst *Read(std::istream &strm, const FstReadOptions &opts) {
     FstReadOptions ropts(opts);
     FstHeader hdr;
     if (ropts.header) {
@@ -118,32 +126,32 @@ class MutableFst : public ExpandedFst<A> {
     }
     auto *fst = reader(strm, ropts);
     if (!fst) return nullptr;
-    return static_cast<MutableFst<Arc> *>(fst);
+    return static_cast<MutableFst *>(fst);
   }
 
   // Reads a MutableFst from a file; returns nullptr on error. An empty
-  // filename results in reading from standard input. If convert is true,
+  // source results in reading from standard input. If convert is true,
   // convert to a mutable FST subclass (given by convert_type) in the case
   // that the input FST is non-mutable.
-  static MutableFst<Arc> *Read(const string &filename, bool convert = false,
-                               const string &convert_type = "vector") {
+  static MutableFst *Read(const std::string &source, bool convert = false,
+                          const std::string &convert_type = "vector") {
     if (convert == false) {
-      if (!filename.empty()) {
-        std::ifstream strm(filename,
+      if (!source.empty()) {
+        std::ifstream strm(source,
                                 std::ios_base::in | std::ios_base::binary);
         if (!strm) {
-          LOG(ERROR) << "MutableFst::Read: Can't open file: " << filename;
+          LOG(ERROR) << "MutableFst::Read: Can't open file: " << source;
           return nullptr;
         }
-        return Read(strm, FstReadOptions(filename));
+        return Read(strm, FstReadOptions(source));
       } else {
         return Read(std::cin, FstReadOptions("standard input"));
       }
     } else {  // Converts to 'convert_type' if not mutable.
-      std::unique_ptr<Fst<Arc>> ifst(Fst<Arc>::Read(filename));
+      std::unique_ptr<Fst<Arc>> ifst(Fst<Arc>::Read(source));
       if (!ifst) return nullptr;
       if (ifst->Properties(kMutable, false)) {
-        return static_cast<MutableFst<Arc> *>(ifst.release());
+        return static_cast<MutableFst *>(ifst.release());
       } else {
         std::unique_ptr<Fst<Arc>> ofst(Convert(*ifst, convert_type));
         ifst.reset();
@@ -151,7 +159,7 @@ class MutableFst : public ExpandedFst<A> {
         if (!ofst->Properties(kMutable, false)) {
           LOG(ERROR) << "MutableFst: Bad convert type: " << convert_type;
         }
-        return static_cast<MutableFst<Arc> *>(ofst.release());
+        return static_cast<MutableFst *>(ofst.release());
       }
     }
   }
@@ -218,9 +226,9 @@ class MutableArcIterator {
 
   void SetValue(const Arc &arc) { data_.base->SetValue(arc); }
 
-  uint32 Flags() const { return data_.base->Flags(); }
+  uint8 Flags() const { return data_.base->Flags(); }
 
-  void SetFlags(uint32 flags, uint32 mask) {
+  void SetFlags(uint8 flags, uint8 mask) {
     return data_.base->SetFlags(flags, mask);
   }
 
@@ -278,7 +286,7 @@ class ImplToMutableFst : public ImplToExpandedFst<Impl, FST> {
     GetMutableImpl()->SetStart(s);
   }
 
-  void SetFinal(StateId s, Weight weight) override {
+  void SetFinal(StateId s, Weight weight = Weight::One()) override {
     MutateCheck();
     GetMutableImpl()->SetFinal(s, std::move(weight));
   }
@@ -296,11 +304,21 @@ class ImplToMutableFst : public ImplToExpandedFst<Impl, FST> {
     return GetMutableImpl()->AddState();
   }
 
+  void AddStates(size_t n) override {
+    MutateCheck();
+    return GetMutableImpl()->AddStates(n);
+  }
+
   void AddArc(StateId s, const Arc &arc) override {
     MutateCheck();
     GetMutableImpl()->AddArc(s, arc);
   }
 
+  void AddArc(StateId s, Arc &&arc) override {
+    MutateCheck();
+    GetMutableImpl()->AddArc(s, std::forward<Arc>(arc));
+  }
+
   void DeleteStates(const std::vector<StateId> &dstates) override {
     MutateCheck();
     GetMutableImpl()->DeleteStates(dstates);
@@ -328,9 +346,9 @@ class ImplToMutableFst : public ImplToExpandedFst<Impl, FST> {
     GetMutableImpl()->DeleteArcs(s);
   }
 
-  void ReserveStates(StateId s) override {
+  void ReserveStates(size_t n) override {
     MutateCheck();
-    GetMutableImpl()->ReserveStates(s);
+    GetMutableImpl()->ReserveStates(n);
   }
 
   void ReserveArcs(StateId s, size_t n) override {
@@ -376,7 +394,7 @@ class ImplToMutableFst : public ImplToExpandedFst<Impl, FST> {
   explicit ImplToMutableFst(std::shared_ptr<Impl> impl)
       : ImplToExpandedFst<Impl, FST>(impl) {}
 
-  ImplToMutableFst(const ImplToMutableFst<Impl, FST> &fst, bool safe)
+  ImplToMutableFst(const ImplToMutableFst &fst, bool safe)
       : ImplToExpandedFst<Impl, FST>(fst, safe) {}
 
   void MutateCheck() {
@@ -386,4 +404,4 @@ class ImplToMutableFst : public ImplToExpandedFst<Impl, FST> {
 
 }  // namespace fst
 
-#endif  // FST_LIB_MUTABLE_FST_H_
+#endif  // FST_MUTABLE_FST_H_
