@@ -3,11 +3,14 @@
 //
 // Cartesian power weight semiring operation definitions.
 
-#ifndef FST_LIB_POWER_WEIGHT_H_
-#define FST_LIB_POWER_WEIGHT_H_
+#ifndef FST_POWER_WEIGHT_H_
+#define FST_POWER_WEIGHT_H_
 
+#include <random>
 #include <string>
 
+#include <fst/types.h>
+
 #include <fst/tuple-weight.h>
 #include <fst/weight.h>
 
@@ -36,6 +39,12 @@ class PowerWeight : public TupleWeight<W, n> {
   template <class Iterator>
   PowerWeight(Iterator begin, Iterator end) : TupleWeight<W, n>(begin, end) {}
 
+  // Initialize component `index` to `weight`; initialize all other components
+  // to `default_weight`
+  PowerWeight(size_t index, const W &weight,
+              const W &default_weight = W::Zero())
+      : TupleWeight<W, n>(index, weight, default_weight) {}
+
   static const PowerWeight &Zero() {
     static const PowerWeight zero(TupleWeight<W, n>::Zero());
     return zero;
@@ -51,12 +60,10 @@ class PowerWeight : public TupleWeight<W, n> {
     return no_weight;
   }
 
-  static const string &Type() {
-    static string type;
-    if (type.empty()) {
-      type = W::Type() + "_^" + std::to_string(n);
-    }
-    return type;
+  static const std::string &Type() {
+    static const std::string *const type =
+        new std::string(W::Type() + "_^" + std::to_string(n));
+    return *type;
   }
 
   static constexpr uint64 Properties() {
@@ -147,7 +154,9 @@ class WeightGenerate<PowerWeight<W, n>> {
   using Weight = PowerWeight<W, n>;
   using Generate = WeightGenerate<W>;
 
-  explicit WeightGenerate(bool allow_zero = true) : generate_(allow_zero) {}
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true)
+      : generate_(seed, allow_zero) {}
 
   Weight operator()() const {
     Weight result;
@@ -156,9 +165,9 @@ class WeightGenerate<PowerWeight<W, n>> {
   }
 
  private:
-  Generate generate_;
+  const Generate generate_;
 };
 
 }  // namespace fst
 
-#endif  // FST_LIB_POWER_WEIGHT_H_
+#endif  // FST_POWER_WEIGHT_H_
