@@ -3,17 +3,22 @@
 //
 // Functions and classes to project an FST on to its domain or range.
 
-#ifndef FST_LIB_PROJECT_H_
-#define FST_LIB_PROJECT_H_
+#ifndef FST_PROJECT_H_
+#define FST_PROJECT_H_
+
+#include <fst/types.h>
 
 #include <fst/arc-map.h>
 #include <fst/mutable-fst.h>
 
-
 namespace fst {
 
 // This specifies whether to project on input or output.
-enum ProjectType { PROJECT_INPUT = 1, PROJECT_OUTPUT = 2 };
+enum class ProjectType { INPUT = 1, OUTPUT = 2 };
+OPENFST_DEPRECATED("Use `ProjectType::INPUT` instead.")
+static constexpr ProjectType PROJECT_INPUT = ProjectType::INPUT;
+OPENFST_DEPRECATED("Use `ProjectType::OUTPUT` instead.")
+static constexpr ProjectType PROJECT_OUTPUT = ProjectType::OUTPUT;
 
 // Mapper to implement projection per arc.
 template <class A>
@@ -22,36 +27,37 @@ class ProjectMapper {
   using FromArc = A;
   using ToArc = A;
 
-  explicit ProjectMapper(ProjectType project_type)
+  constexpr explicit ProjectMapper(ProjectType project_type)
       : project_type_(project_type) {}
 
-  ToArc operator()(const FromArc &arc) {
-    const auto label = project_type_ == PROJECT_INPUT ? arc.ilabel : arc.olabel;
+  ToArc operator()(const FromArc &arc) const {
+    const auto label =
+        project_type_ == ProjectType::INPUT ? arc.ilabel : arc.olabel;
     return ToArc(label, label, arc.weight, arc.nextstate);
   }
 
-  MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
+  constexpr MapFinalAction FinalAction() const { return MAP_NO_SUPERFINAL; }
 
-  MapSymbolsAction InputSymbolsAction() const {
-    return project_type_ == PROJECT_INPUT ? MAP_COPY_SYMBOLS
-                                          : MAP_CLEAR_SYMBOLS;
+  constexpr MapSymbolsAction InputSymbolsAction() const {
+    return project_type_ == ProjectType::INPUT ? MAP_COPY_SYMBOLS
+                                               : MAP_CLEAR_SYMBOLS;
   }
 
-  MapSymbolsAction OutputSymbolsAction() const {
-    return project_type_ == PROJECT_OUTPUT ? MAP_COPY_SYMBOLS
-                                           : MAP_CLEAR_SYMBOLS;
+  constexpr MapSymbolsAction OutputSymbolsAction() const {
+    return project_type_ == ProjectType::OUTPUT ? MAP_COPY_SYMBOLS
+                                                : MAP_CLEAR_SYMBOLS;
   }
 
-  uint64 Properties(uint64 props) {
-    return ProjectProperties(props, project_type_ == PROJECT_INPUT);
+  constexpr uint64 Properties(uint64 props) const {
+    return ProjectProperties(props, project_type_ == ProjectType::INPUT);
   }
 
  private:
-  ProjectType project_type_;
+  const ProjectType project_type_;
 };
 
 // Projects an FST onto its domain or range by either copying each arcs' input
-// label to the output label or vice versa. This version modifies its input.
+// label to the output label or vice versa.
 //
 // Complexity:
 //
@@ -60,11 +66,30 @@ class ProjectMapper {
 //
 // where V is the number of states and E is the number of arcs.
 template <class Arc>
+inline void Project(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
+                    ProjectType project_type) {
+  ArcMap(ifst, ofst, ProjectMapper<Arc>(project_type));
+  switch (project_type) {
+    case ProjectType::INPUT:
+      ofst->SetOutputSymbols(ifst.InputSymbols());
+      return;
+    case ProjectType::OUTPUT:
+      ofst->SetInputSymbols(ifst.OutputSymbols());
+      return;
+  }
+}
+
+// Destructive variant of the above.
+template <class Arc>
 inline void Project(MutableFst<Arc> *fst, ProjectType project_type) {
   ArcMap(fst, ProjectMapper<Arc>(project_type));
-  if (project_type == PROJECT_INPUT) fst->SetOutputSymbols(fst->InputSymbols());
-  if (project_type == PROJECT_OUTPUT) {
-    fst->SetInputSymbols(fst->OutputSymbols());
+  switch (project_type) {
+    case ProjectType::INPUT:
+      fst->SetOutputSymbols(fst->InputSymbols());
+      return;
+    case ProjectType::OUTPUT:
+      fst->SetInputSymbols(fst->OutputSymbols());
+      return;
   }
 }
 
@@ -89,20 +114,20 @@ class ProjectFst : public ArcMapFst<A, A, ProjectMapper<A>> {
 
   ProjectFst(const Fst<A> &fst, ProjectType project_type)
       : ArcMapFst<A, A, ProjectMapper<A>>(fst, ProjectMapper<A>(project_type)) {
-    if (project_type == PROJECT_INPUT) {
+    if (project_type == ProjectType::INPUT) {
       GetMutableImpl()->SetOutputSymbols(fst.InputSymbols());
     }
-    if (project_type == PROJECT_OUTPUT) {
+    if (project_type == ProjectType::OUTPUT) {
       GetMutableImpl()->SetInputSymbols(fst.OutputSymbols());
     }
   }
 
   // See Fst<>::Copy() for doc.
-  ProjectFst(const ProjectFst<A> &fst, bool safe = false)
+  ProjectFst(const ProjectFst &fst, bool safe = false)
       : ArcMapFst<A, A, ProjectMapper<A>>(fst, safe) {}
 
   // Gets a copy of this ProjectFst. See Fst<>::Copy() for further doc.
-  ProjectFst<A> *Copy(bool safe = false) const override {
+  ProjectFst *Copy(bool safe = false) const override {
     return new ProjectFst(*this, safe);
   }
 
@@ -135,4 +160,4 @@ using StdProjectFst = ProjectFst<StdArc>;
 
 }  // namespace fst
 
-#endif  // FST_LIB_PROJECT_H_
+#endif  // FST_PROJECT_H_
