@@ -3,21 +3,24 @@
 //
 // Functions and classes for various FST state queues with a unified interface.
 
-#ifndef FST_LIB_QUEUE_H_
-#define FST_LIB_QUEUE_H_
+#ifndef FST_QUEUE_H_
+#define FST_QUEUE_H_
 
-#include <deque>
 #include <memory>
+#include <queue>
+#include <stack>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/arcfilter.h>
 #include <fst/connect.h>
 #include <fst/heap.h>
 #include <fst/topsort.h>
-
+#include <fst/weight.h>
 
 namespace fst {
 
@@ -100,23 +103,25 @@ class QueueBase {
 // can be used for strongly connected components with only one state and no
 // self-loops.
 template <class S>
-class TrivialQueue final : public QueueBase<S> {
+class TrivialQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
   TrivialQueue() : QueueBase<StateId>(TRIVIAL_QUEUE), front_(kNoStateId) {}
 
-  StateId Head() const override { return front_; }
+  ~TrivialQueue() override = default;
 
-  void Enqueue(StateId s) override { front_ = s; }
+  StateId Head() const final { return front_; }
 
-  void Dequeue() override { front_ = kNoStateId; }
+  void Enqueue(StateId s) final { front_ = s; }
 
-  void Update(StateId) override {}
+  void Dequeue() final { front_ = kNoStateId; }
 
-  bool Empty() const override { return front_ == kNoStateId; }
+  void Update(StateId) final {}
 
-  void Clear() override { front_ = kNoStateId; }
+  bool Empty() const final { return front_ == kNoStateId; }
+
+  void Clear() final { front_ = kNoStateId; }
 
  private:
   StateId front_;
@@ -126,56 +131,54 @@ class TrivialQueue final : public QueueBase<S> {
 //
 // This is not a final class.
 template <class S>
-class FifoQueue : public QueueBase<S>, public std::deque<S> {
+class FifoQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
-  using std::deque<S>::back;
-  using std::deque<S>::push_front;
-  using std::deque<S>::pop_back;
-  using std::deque<S>::empty;
-  using std::deque<S>::clear;
-
   FifoQueue() : QueueBase<StateId>(FIFO_QUEUE) {}
 
-  StateId Head() const override { return back(); }
+  ~FifoQueue() override = default;
 
-  void Enqueue(StateId s) override { push_front(s); }
+  StateId Head() const override { return queue_.front(); }
 
-  void Dequeue() override { pop_back(); }
+  void Enqueue(StateId s) override { queue_.push(s); }
+
+  void Dequeue() override { queue_.pop(); }
 
   void Update(StateId) override {}
 
-  bool Empty() const override { return empty(); }
+  bool Empty() const override { return queue_.empty(); }
 
-  void Clear() override { clear(); }
+  void Clear() override { queue_ = std::queue<StateId>(); }
+
+ private:
+  std::queue<StateId> queue_;
 };
 
 // Last-in, first-out queue discipline.
 template <class S>
-class LifoQueue final : public QueueBase<S>, public std::deque<S> {
+class LifoQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
-  using std::deque<S>::front;
-  using std::deque<S>::push_front;
-  using std::deque<S>::pop_front;
-  using std::deque<S>::empty;
-  using std::deque<S>::clear;
-
   LifoQueue() : QueueBase<StateId>(LIFO_QUEUE) {}
 
-  StateId Head() const override { return front(); }
+  ~LifoQueue() override = default;
 
-  void Enqueue(StateId s) override { push_front(s); }
+  StateId Head() const final { return stack_.top(); }
 
-  void Dequeue() override { pop_front(); }
+  void Enqueue(StateId s) final { stack_.push(s); }
 
-  void Update(StateId) override {}
+  void Dequeue() final { stack_.pop(); }
 
-  bool Empty() const override { return empty(); }
+  void Update(StateId) final {}
 
-  void Clear() override { clear(); }
+  bool Empty() const final { return stack_.empty(); }
+
+  void Clear() final { stack_ = std::stack<StateId>(); }
+
+ private:
+  std::stack<StateId> stack_;
 };
 
 // Shortest-first queue discipline, templated on the StateId and as well as a
@@ -189,16 +192,16 @@ class ShortestFirstQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
-  static constexpr StateId kNoKey = -1;
-
   explicit ShortestFirstQueue(Compare comp)
       : QueueBase<StateId>(SHORTEST_FIRST_QUEUE), heap_(comp) {}
 
+  ~ShortestFirstQueue() override = default;
+
   StateId Head() const override { return heap_.Top(); }
 
   void Enqueue(StateId s) override {
     if (update) {
-      for (StateId i = key_.size(); i <= s; ++i) key_.push_back(kNoKey);
+      for (StateId i = key_.size(); i <= s; ++i) key_.push_back(kNoStateId);
       key_[s] = heap_.Insert(s);
     } else {
       heap_.Insert(s);
@@ -207,7 +210,7 @@ class ShortestFirstQueue : public QueueBase<S> {
 
   void Dequeue() override {
     if (update) {
-      key_[heap_.Pop()] = kNoKey;
+      key_[heap_.Pop()] = kNoStateId;
     } else {
       heap_.Pop();
     }
@@ -215,7 +218,7 @@ class ShortestFirstQueue : public QueueBase<S> {
 
   void Update(StateId s) override {
     if (!update) return;
-    if (s >= key_.size() || key_[s] == kNoKey) {
+    if (s >= key_.size() || key_[s] == kNoStateId) {
       Enqueue(s);
     } else {
       heap_.Update(key_[s], s);
@@ -229,14 +232,15 @@ class ShortestFirstQueue : public QueueBase<S> {
     if (update) key_.clear();
   }
 
+  ssize_t Size() const { return heap_.Size(); }
+
+  const Compare &GetCompare() const { return heap_.GetCompare(); }
+
  private:
   Heap<StateId, Compare> heap_;
   std::vector<ssize_t> key_;
 };
 
-template <typename StateId, typename Compare, bool update>
-constexpr StateId ShortestFirstQueue<StateId, Compare, update>::kNoKey;
-
 namespace internal {
 
 // Given a vector that maps from states to weights, and a comparison functor
@@ -264,7 +268,7 @@ class StateWeightCompare {
 // Shortest-first queue discipline, templated on the StateId and Weight, is
 // specialized to use the weight's natural order for the comparison function.
 template <typename S, typename Weight>
-class NaturalShortestFirstQueue final
+class NaturalShortestFirstQueue
     : public ShortestFirstQueue<
           S, internal::StateWeightCompare<S, NaturalLess<Weight>>> {
  public:
@@ -274,16 +278,116 @@ class NaturalShortestFirstQueue final
   explicit NaturalShortestFirstQueue(const std::vector<Weight> &distance)
       : ShortestFirstQueue<StateId, Compare>(Compare(distance, less_)) {}
 
+  ~NaturalShortestFirstQueue() override = default;
+
  private:
   // This is non-static because the constructor for non-idempotent weights will
-  // result in a an error.
+  // result in an error.
   const NaturalLess<Weight> less_{};
 };
 
+// In a shortest path computation on a lattice-like FST, we may keep many old
+// nonviable paths as a part of the search. Since the search process always
+// expands the lowest cost path next, that lowest cost path may be a very old
+// nonviable path instead of one we expect to lead to a shortest path.
+//
+// For instance, suppose that the current best path in an alignment has
+// traversed 500 arcs with a cost of 10. We may also have a bad path in
+// the queue that has traversed only 40 arcs but also has a cost of 10.
+// This path is very unlikely to lead to a reasonable alignment, so this queue
+// can prune it from the search space.
+//
+// This queue relies on the caller using a shortest-first exploration order
+// like this:
+//   while (true) {
+//     StateId head = queue.Head();
+//     queue.Dequeue();
+//     for (const auto& arc : GetArcs(fst, head)) {
+//       queue.Enqueue(arc.nextstate);
+//     }
+//   }
+// We use this assumption to guess that there is an arc between Head and the
+// Enqueued state; this is how the number of path steps is measured.
+template <typename S, typename Weight>
+class PruneNaturalShortestFirstQueue
+    : public NaturalShortestFirstQueue<S, Weight> {
+ public:
+  using StateId = S;
+  using Base = NaturalShortestFirstQueue<StateId, Weight>;
+
+  PruneNaturalShortestFirstQueue(const std::vector<Weight> &distance,
+                                 ssize_t arc_threshold, ssize_t state_limit = 0)
+      : Base(distance),
+        arc_threshold_(arc_threshold),
+        state_limit_(state_limit),
+        head_steps_(0),
+        max_head_steps_(0) {}
+
+  ~PruneNaturalShortestFirstQueue() override = default;
+
+  StateId Head() const override {
+    const auto head = Base::Head();
+    // Stores the number of steps from the start of the graph to this state
+    // along the shortest-weight path.
+    if (head < steps_.size()) {
+      max_head_steps_ = std::max(steps_[head], max_head_steps_);
+      head_steps_ = steps_[head];
+    }
+    return head;
+  }
+
+  void Enqueue(StateId s) override {
+    // We assume that there is an arc between the Head() state and this
+    // Enqueued state.
+    const ssize_t state_steps = head_steps_ + 1;
+    if (s >= steps_.size()) {
+      steps_.resize(s + 1, state_steps);
+    }
+    // This is the number of arcs in the minimum cost path from Start to s.
+    steps_[s] = state_steps;
+
+    // Adjust the threshold in cases where path step thresholding wasn't
+    // enough to keep the queue small.
+    ssize_t adjusted_threshold = arc_threshold_;
+    if (Base::Size() > state_limit_ && state_limit_ > 0) {
+      adjusted_threshold = std::max<ssize_t>(
+          0, arc_threshold_ - (Base::Size() / state_limit_) - 1);
+    }
+
+    if (state_steps > (max_head_steps_ - adjusted_threshold) ||
+        arc_threshold_ < 0) {
+      if (adjusted_threshold == 0 && state_limit_ > 0) {
+        // If the queue is continuing to grow without bound, we follow any
+        // path that makes progress and clear the rest.
+        Base::Clear();
+      }
+      Base::Enqueue(s);
+    }
+  }
+
+ private:
+  // A dense map from StateId to the number of arcs in the minimum weight
+  // path from Start to this state.
+  std::vector<ssize_t> steps_;
+  // We only keep paths that are within this number of arcs (not weight!)
+  // of the longest path.
+  const ssize_t arc_threshold_;
+  // If the size of the queue climbs above this number, we increase the
+  // threshold to reduce the amount of work we have to do.
+  const ssize_t state_limit_;
+
+  // The following are mutable because Head() is const.
+  // The number of arcs traversed in the minimum cost path from the start
+  // state to the current Head() state.
+  mutable ssize_t head_steps_;
+  // The maximum number of arcs traversed by any low-cost path so far.
+  mutable ssize_t max_head_steps_;
+};
+
 // Topological-order queue discipline, templated on the StateId. States are
 // ordered in the queue topologically. The FST must be acyclic.
 template <class S>
-class TopOrderQueue final : public QueueBase<S> {
+class TopOrderQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
@@ -315,9 +419,11 @@ class TopOrderQueue final : public QueueBase<S> {
         order_(order),
         state_(order.size(), kNoStateId) {}
 
-  StateId Head() const override { return state_[front_]; }
+  ~TopOrderQueue() override = default;
 
-  void Enqueue(StateId s) override {
+  StateId Head() const final { return state_[front_]; }
+
+  void Enqueue(StateId s) final {
     if (front_ > back_) {
       front_ = back_ = order_[s];
     } else if (order_[s] > back_) {
@@ -328,16 +434,16 @@ class TopOrderQueue final : public QueueBase<S> {
     state_[order_[s]] = s;
   }
 
-  void Dequeue() override {
+  void Dequeue() final {
     state_[front_] = kNoStateId;
     while ((front_ <= back_) && (state_[front_] == kNoStateId)) ++front_;
   }
 
-  void Update(StateId) override {}
+  void Update(StateId) final {}
 
-  bool Empty() const override { return front_ > back_; }
+  bool Empty() const final { return front_ > back_; }
 
-  void Clear() override {
+  void Clear() final {
     for (StateId s = front_; s <= back_; ++s) state_[s] = kNoStateId;
     back_ = kNoStateId;
     front_ = 0;
@@ -353,16 +459,18 @@ class TopOrderQueue final : public QueueBase<S> {
 // State order queue discipline, templated on the StateId. States are ordered in
 // the queue by state ID.
 template <class S>
-class StateOrderQueue final : public QueueBase<S> {
+class StateOrderQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
   StateOrderQueue()
       : QueueBase<StateId>(STATE_ORDER_QUEUE), front_(0), back_(kNoStateId) {}
 
-  StateId Head() const override { return front_; }
+  ~StateOrderQueue() override = default;
 
-  void Enqueue(StateId s) override {
+  StateId Head() const final { return front_; }
+
+  void Enqueue(StateId s) final {
     if (front_ > back_) {
       front_ = back_ = s;
     } else if (s > back_) {
@@ -374,16 +482,16 @@ class StateOrderQueue final : public QueueBase<S> {
     enqueued_[s] = true;
   }
 
-  void Dequeue() override {
+  void Dequeue() final {
     enqueued_[front_] = false;
     while ((front_ <= back_) && (enqueued_[front_] == false)) ++front_;
   }
 
-  void Update(StateId) override {}
+  void Update(StateId) final {}
 
-  bool Empty() const override { return front_ > back_; }
+  bool Empty() const final { return front_ > back_; }
 
-  void Clear() override {
+  void Clear() final {
     for (StateId i = front_; i <= back_; ++i) enqueued_[i] = false;
     front_ = 0;
     back_ = kNoStateId;
@@ -399,7 +507,7 @@ class StateOrderQueue final : public QueueBase<S> {
 // queue used inside each SCC. It visits the SCCs of an FST in topological
 // order. Its constructor is passed the queues to to use within an SCC.
 template <class S, class Queue>
-class SccQueue final : public QueueBase<S> {
+class SccQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
@@ -413,7 +521,9 @@ class SccQueue final : public QueueBase<S> {
         front_(0),
         back_(kNoStateId) {}
 
-  StateId Head() const override {
+  ~SccQueue() override = default;
+
+  StateId Head() const final {
     while ((front_ <= back_) &&
            (((*queue_)[front_] && (*queue_)[front_]->Empty()) ||
             (((*queue_)[front_] == nullptr) &&
@@ -428,7 +538,7 @@ class SccQueue final : public QueueBase<S> {
     }
   }
 
-  void Enqueue(StateId s) override {
+  void Enqueue(StateId s) final {
     if (front_ > back_) {
       front_ = back_ = scc_[s];
     } else if (scc_[s] > back_) {
@@ -446,7 +556,7 @@ class SccQueue final : public QueueBase<S> {
     }
   }
 
-  void Dequeue() override {
+  void Dequeue() final {
     if ((*queue_)[front_]) {
       (*queue_)[front_]->Dequeue();
     } else if (front_ < trivial_queue_.size()) {
@@ -454,12 +564,12 @@ class SccQueue final : public QueueBase<S> {
     }
   }
 
-  void Update(StateId s) override {
+  void Update(StateId s) final {
     if ((*queue_)[scc_[s]]) (*queue_)[scc_[s]]->Update(s);
   }
 
-  bool Empty() const override {
-    // Queue SCC number back_ is not empty unless back_ == front_.
+  bool Empty() const final {
+    // Queues SCC number back_ is not empty unless back_ == front_.
     if (front_ < back_) {
       return false;
     } else if (front_ > back_) {
@@ -472,7 +582,7 @@ class SccQueue final : public QueueBase<S> {
     }
   }
 
-  void Clear() override {
+  void Clear() final {
     for (StateId i = front_; i <= back_; ++i) {
       if ((*queue_)[i]) {
         (*queue_)[i]->Clear();
@@ -495,7 +605,7 @@ class SccQueue final : public QueueBase<S> {
 // Automatic queue discipline. It selects a queue discipline for a given FST
 // based on its properties.
 template <class S>
-class AutoQueue final : public QueueBase<S> {
+class AutoQueue : public QueueBase<S> {
  public:
   using StateId = S;
 
@@ -507,18 +617,19 @@ class AutoQueue final : public QueueBase<S> {
             const std::vector<typename Arc::Weight> *distance, ArcFilter filter)
       : QueueBase<StateId>(AUTO_QUEUE) {
     using Weight = typename Arc::Weight;
-    using Compare = internal::StateWeightCompare<StateId, NaturalLess<Weight>>;
+    using Less = NaturalLess<Weight>;
+    using Compare = internal::StateWeightCompare<StateId, Less>;
     // First checks if the FST is known to have these properties.
     const auto props =
         fst.Properties(kAcyclic | kCyclic | kTopSorted | kUnweighted, false);
     if ((props & kTopSorted) || fst.Start() == kNoStateId) {
-      queue_.reset(new StateOrderQueue<StateId>());
+      queue_ = fst::make_unique<StateOrderQueue<StateId>>();
       VLOG(2) << "AutoQueue: using state-order discipline";
     } else if (props & kAcyclic) {
-      queue_.reset(new TopOrderQueue<StateId>(fst, filter));
+      queue_ = fst::make_unique<TopOrderQueue<StateId>>(fst, filter);
       VLOG(2) << "AutoQueue: using top-order discipline";
     } else if ((props & kUnweighted) && (Weight::Properties() & kIdempotent)) {
-      queue_.reset(new LifoQueue<StateId>());
+      queue_ = fst::make_unique<LifoQueue<StateId>>();
       VLOG(2) << "AutoQueue: using LIFO discipline";
     } else {
       uint64 properties;
@@ -527,11 +638,11 @@ class AutoQueue final : public QueueBase<S> {
       DfsVisit(fst, &scc_visitor, filter);
       auto nscc = *std::max_element(scc_.begin(), scc_.end()) + 1;
       std::vector<QueueType> queue_types(nscc);
-      std::unique_ptr<NaturalLess<Weight>> less;
+      std::unique_ptr<Less> less;
       std::unique_ptr<Compare> comp;
-      if (distance && (Weight::Properties() & kPath)) {
-        less.reset(new NaturalLess<Weight>());
-        comp.reset(new Compare(*distance, *less));
+      if (distance && (Weight::Properties() & kPath) == kPath) {
+        less = fst::make_unique<Less>();
+        comp = fst::make_unique<Compare>(*distance, *less);
       }
       // Finds the queue type to use per SCC.
       bool unweighted;
@@ -540,14 +651,14 @@ class AutoQueue final : public QueueBase<S> {
                    &unweighted);
       // If unweighted and semiring is idempotent, uses LIFO queue.
       if (unweighted) {
-        queue_.reset(new LifoQueue<StateId>());
+        queue_ = fst::make_unique<LifoQueue<StateId>>();
         VLOG(2) << "AutoQueue: using LIFO discipline";
         return;
       }
       // If all the SCC are trivial, the FST is acyclic and the scc number gives
       // the topological order.
       if (all_trivial) {
-        queue_.reset(new TopOrderQueue<StateId>(scc_));
+        queue_ = fst::make_unique<TopOrderQueue<StateId>>(scc_);
         VLOG(2) << "AutoQueue: using top-order discipline";
         return;
       }
@@ -566,31 +677,34 @@ class AutoQueue final : public QueueBase<S> {
                     << ": using shortest-first discipline";
             break;
           case LIFO_QUEUE:
-            queues_[i].reset(new LifoQueue<StateId>());
+            queues_[i] = fst::make_unique<LifoQueue<StateId>>();
             VLOG(3) << "AutoQueue: SCC #" << i << ": using LIFO discipline";
             break;
           case FIFO_QUEUE:
           default:
-            queues_[i].reset(new FifoQueue<StateId>());
+            queues_[i] = fst::make_unique<FifoQueue<StateId>>();
             VLOG(3) << "AutoQueue: SCC #" << i << ": using FIFO discipine";
             break;
         }
       }
-      queue_.reset(new SccQueue<StateId, QueueBase<StateId>>(scc_, &queues_));
+      queue_ = fst::make_unique<SccQueue<StateId, QueueBase<StateId>>>(
+          scc_, &queues_);
     }
   }
 
-  StateId Head() const override { return queue_->Head(); }
+  ~AutoQueue() override = default;
 
-  void Enqueue(StateId s) override { queue_->Enqueue(s); }
+  StateId Head() const final { return queue_->Head(); }
 
-  void Dequeue() override { queue_->Dequeue(); }
+  void Enqueue(StateId s) final { queue_->Enqueue(s); }
 
-  void Update(StateId s) override { queue_->Update(s); }
+  void Dequeue() final { queue_->Dequeue(); }
 
-  bool Empty() const override { return queue_->Empty(); }
+  void Update(StateId s) final { queue_->Update(s); }
 
-  void Clear() override { queue_->Clear(); }
+  bool Empty() const final { return queue_->Empty(); }
+
+  void Clear() final { queue_->Clear(); }
 
  private:
   template <class Arc, class ArcFilter, class Less>
@@ -652,15 +766,35 @@ void AutoQueue<StateId>::SccQueueType(const Fst<Arc> &fst,
   }
 }
 
-// An A* estimate is a function object that maps from a state ID to a an
+// An A* estimate is a function object that maps from a state ID to an
 // estimate of the shortest distance to the final states.
 
-// The trivial A* estimate is always One().
+// A trivial A* estimate, yielding a queue which behaves the same in Dijkstra's
+// algorithm.
 template <typename StateId, typename Weight>
 struct TrivialAStarEstimate {
-  Weight operator()(StateId) const { return Weight::One(); }
+  constexpr Weight operator()(StateId) const { return Weight::One(); }
 };
 
+// A non-trivial A* estimate using a vector of the estimated future costs.
+template <typename StateId, typename Weight>
+class NaturalAStarEstimate {
+ public:
+  NaturalAStarEstimate(const std::vector<Weight> &beta) : beta_(beta) {}
+
+  const Weight &operator()(StateId s) const {
+    return (s < beta_.size()) ? beta_[s] : kZero;
+  }
+
+ private:
+  static constexpr Weight kZero = Weight::Zero();
+
+  const std::vector<Weight> &beta_;
+};
+
+template <typename Arc, typename Weight>
+constexpr Weight NaturalAStarEstimate<Arc, Weight>::kZero;
+
 // Given a vector that maps from states to weights representing the shortest
 // distance from the initial state, a comparison function object between
 // weights, and an estimate of the shortest distance to the final states, this
@@ -675,14 +809,15 @@ class AStarWeightCompare {
                      const Estimate &estimate)
       : weights_(weights), less_(less), estimate_(estimate) {}
 
-  bool operator()(const StateId s1, const StateId s2) const {
+  bool operator()(StateId s1, StateId s2) const {
     const auto w1 = Times(weights_[s1], estimate_(s1));
     const auto w2 = Times(weights_[s2], estimate_(s2));
     return less_(w1, w2);
   }
 
+  const Estimate &GetEstimate() const { return estimate_; }
+
  private:
-  // Borrowed references.
   const std::vector<Weight> &weights_;
   const Less &less_;
   const Estimate &estimate_;
@@ -690,7 +825,7 @@ class AStarWeightCompare {
 
 // A* queue discipline templated on StateId, Weight, and Estimate.
 template <typename S, typename Weight, typename Estimate>
-class NaturalAStarQueue final
+class NaturalAStarQueue
     : public ShortestFirstQueue<
           S, AStarWeightCompare<S, NaturalLess<Weight>, Estimate>> {
  public:
@@ -702,9 +837,11 @@ class NaturalAStarQueue final
       : ShortestFirstQueue<StateId, Compare>(
             Compare(distance, less_, estimate)) {}
 
+  ~NaturalAStarQueue() override = default;
+
  private:
   // This is non-static because the constructor for non-idempotent weights will
-  // result in a an error.
+  // result in an error.
   const NaturalLess<Weight> less_{};
 };
 
@@ -739,6 +876,8 @@ class PruneQueue : public QueueBase<typename Queue::StateId> {
         class_fnc_(class_fnc),
         threshold_(std::move(threshold)) {}
 
+  ~PruneQueue() override = default;
+
   StateId Head() const override { return queue_->Head(); }
 
   void Enqueue(StateId s) override {
@@ -790,6 +929,8 @@ class NaturalPruneQueue final
                     const ClassFnc &class_fnc, Weight threshold)
       : PruneQueue<Queue, NaturalLess<Weight>, ClassFnc>(
             distance, queue, NaturalLess<Weight>(), class_fnc, threshold) {}
+
+  ~NaturalPruneQueue() override = default;
 };
 
 // Filter-based pruning queue discipline: enqueues a state only if allowed by
@@ -797,27 +938,29 @@ class NaturalPruneQueue final
 // queue discipline is specified by the queue argument. The ownership of the
 // queue is given to this class.
 template <typename Queue, typename Filter>
-class FilterQueue final : public QueueBase<typename Queue::StateId> {
+class FilterQueue : public QueueBase<typename Queue::StateId> {
  public:
   using StateId = typename Queue::StateId;
 
   FilterQueue(Queue *queue, const Filter &filter)
       : QueueBase<StateId>(OTHER_QUEUE), queue_(queue), filter_(filter) {}
 
-  StateId Head() const override { return queue_->Head(); }
+  ~FilterQueue() override = default;
+
+  StateId Head() const final { return queue_->Head(); }
 
   // Enqueues only if allowed by state filter.
-  void Enqueue(StateId s) override {
+  void Enqueue(StateId s) final {
     if (filter_(s)) queue_->Enqueue(s);
   }
 
-  void Dequeue() override { queue_->Dequeue(); }
+  void Dequeue() final { queue_->Dequeue(); }
 
-  void Update(StateId s) override {}
+  void Update(StateId s) final {}
 
-  bool Empty() const override { return queue_->Empty(); }
+  bool Empty() const final { return queue_->Empty(); }
 
-  void Clear() override { queue_->Clear(); }
+  void Clear() final { queue_->Clear(); }
 
  private:
   std::unique_ptr<Queue> queue_;
@@ -826,4 +969,4 @@ class FilterQueue final : public QueueBase<typename Queue::StateId> {
 
 }  // namespace fst
 
-#endif  // FST_LIB_QUEUE_H_
+#endif  // FST_QUEUE_H_
