@@ -3,10 +3,11 @@
 //
 // Classes for registering derived FST for generic reading.
 
-#ifndef FST_LIB_REGISTER_H_
-#define FST_LIB_REGISTER_H_
+#ifndef FST_REGISTER_H_
+#define FST_REGISTER_H_
 
 #include <string>
+#include <type_traits>
 
 
 #include <fst/compat.h>
@@ -14,7 +15,6 @@
 #include <fst/util.h>
 
 
-#include <fst/types.h>
 #include <fst/log.h>
 
 namespace fst {
@@ -41,23 +41,23 @@ struct FstRegisterEntry {
 // This class maintains the correspondence between a string describing
 // an FST type, and its reader and converter.
 template <class Arc>
-class FstRegister
-    : public GenericRegister<string, FstRegisterEntry<Arc>, FstRegister<Arc>> {
+class FstRegister : public GenericRegister<std::string, FstRegisterEntry<Arc>,
+                                           FstRegister<Arc>> {
  public:
   using Reader = typename FstRegisterEntry<Arc>::Reader;
   using Converter = typename FstRegisterEntry<Arc>::Converter;
 
-  const Reader GetReader(const string &type) const {
+  const Reader GetReader(const std::string &type) const {
     return this->GetEntry(type).reader;
   }
 
-  const Converter GetConverter(const string &type) const {
+  const Converter GetConverter(const std::string &type) const {
     return this->GetEntry(type).converter;
   }
 
  protected:
-  string ConvertKeyToSoFilename(const string &key) const override {
-    string legal_type(key);
+  std::string ConvertKeyToSoFilename(const std::string &key) const override {
+    std::string legal_type(key);
     ConvertToLegalCSymbol(&legal_type);
     return legal_type + "-fst.so";
   }
@@ -78,11 +78,14 @@ class FstRegisterer : public GenericRegisterer<FstRegister<typename FST::Arc>> {
                                                           BuildEntry()) {}
 
  private:
+  static Fst<Arc> *ReadGeneric(std::istream &strm, const FstReadOptions &opts) {
+    static_assert(std::is_base_of<Fst<Arc>, FST>::value,
+                  "FST class does not inherit from Fst<Arc>");
+    return FST::Read(strm, opts);
+  }
+
   static Entry BuildEntry() {
-    FST *(*reader)(std::istream & strm, const FstReadOptions &opts) =
-        &FST::Read;
-    return Entry(reinterpret_cast<Reader>(reader),
-                 &FstRegisterer<FST>::Convert);
+    return Entry(&ReadGeneric, &FstRegisterer<FST>::Convert);
   }
 
   static Fst<Arc> *Convert(const Fst<Arc> &fst) { return new FST(fst); }
@@ -94,7 +97,7 @@ class FstRegisterer : public GenericRegisterer<FstRegister<typename FST::Arc>> {
 
 // Converts an FST to the specified type.
 template <class Arc>
-Fst<Arc> *Convert(const Fst<Arc> &fst, const string &fst_type) {
+Fst<Arc> *Convert(const Fst<Arc> &fst, const std::string &fst_type) {
   auto *reg = FstRegister<Arc>::GetRegister();
   const auto converter = reg->GetConverter(fst_type);
   if (!converter) {
@@ -107,4 +110,4 @@ Fst<Arc> *Convert(const Fst<Arc> &fst, const string &fst_type) {
 
 }  // namespace fst
 
-#endif  // FST_LIB_REGISTER_H_
+#endif  // FST_REGISTER_H_
