@@ -3,14 +3,15 @@
 //
 // Functions and classes to relabel an FST (either on input or output).
 
-#ifndef FST_LIB_RELABEL_H_
-#define FST_LIB_RELABEL_H_
+#ifndef FST_RELABEL_H_
+#define FST_RELABEL_H_
 
 #include <string>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/cache.h>
@@ -35,10 +36,10 @@ void Relabel(
   using Label = typename Arc::Label;
   const auto props = fst->Properties(kFstProperties, false);
   // Constructs label-to-label maps.
-  std::unordered_map<Label, Label> input_map;
-  for (auto &ipair : ipairs) input_map[ipair.first] = ipair.second;
-  std::unordered_map<Label, Label> output_map;
-  for (auto &opair : opairs) output_map[opair.first] = opair.second;
+  const std::unordered_map<Label, Label> input_map(
+      ipairs.begin(), ipairs.end());
+  const std::unordered_map<Label, Label> output_map(
+      opairs.begin(), opairs.end());
   for (StateIterator<MutableFst<Arc>> siter(*fst); !siter.Done();
        siter.Next()) {
     for (MutableArcIterator<MutableFst<Arc>> aiter(fst, siter.Value());
@@ -78,11 +79,11 @@ void Relabel(
 // FST. If the 'unknown_i(o)symbol' is non-empty, it is used to label any
 // missing symbol in new_i(o)symbols table.
 template <class Arc>
-void Relabel(MutableFst<Arc> *fst,
-             const SymbolTable *old_isymbols, const SymbolTable *new_isymbols,
-             const string &unknown_isymbol, bool attach_new_isymbols,
+void Relabel(MutableFst<Arc> *fst, const SymbolTable *old_isymbols,
+             const SymbolTable *new_isymbols,
+             const std::string &unknown_isymbol, bool attach_new_isymbols,
              const SymbolTable *old_osymbols, const SymbolTable *new_osymbols,
-             const string &unknown_osymbol, bool attach_new_osymbols) {
+             const std::string &unknown_osymbol, bool attach_new_osymbols) {
   using Label = typename Arc::Label;
   // Constructs vectors of input-side label pairs.
   std::vector<std::pair<Label, Label>> ipairs;
@@ -98,11 +99,10 @@ void Relabel(MutableFst<Arc> *fst,
       }
     }
 
-    for (SymbolTableIterator siter(*old_isymbols); !siter.Done();
-         siter.Next()) {
-      const auto old_index = siter.Value();
-      const auto symbol = siter.Symbol();
-      auto new_index = new_isymbols->Find(siter.Symbol());
+    for (const auto &sitem : *old_isymbols) {
+      const auto old_index = sitem.Label();
+      const auto symbol = sitem.Symbol();
+      auto new_index = new_isymbols->Find(symbol);
       if (new_index == kNoLabel) {
         if (unknown_ilabel != kNoLabel) {
           new_index = unknown_ilabel;
@@ -112,7 +112,7 @@ void Relabel(MutableFst<Arc> *fst,
           ++num_missing_syms;
         }
       }
-      ipairs.push_back(std::make_pair(old_index, new_index));
+      ipairs.emplace_back(old_index, new_index);
     }
     if (num_missing_syms > 0) {
       LOG(WARNING) << "Target symbol table missing: " << num_missing_syms
@@ -133,12 +133,10 @@ void Relabel(MutableFst<Arc> *fst,
         ++num_missing_syms;
       }
     }
-
-    for (SymbolTableIterator siter(*old_osymbols); !siter.Done();
-         siter.Next()) {
-      const auto old_index = siter.Value();
-      const auto symbol = siter.Symbol();
-      auto new_index = new_osymbols->Find(siter.Symbol());
+    for (const auto &sitem : *old_osymbols) {
+      const auto old_index = sitem.Label();
+      const auto symbol = sitem.Symbol();
+      auto new_index = new_osymbols->Find(symbol);
       if (new_index == kNoLabel) {
         if (unknown_olabel != kNoLabel) {
           new_index = unknown_olabel;
@@ -148,7 +146,7 @@ void Relabel(MutableFst<Arc> *fst,
           ++num_missing_syms;
         }
       }
-      opairs.push_back(std::make_pair(old_index, new_index));
+      opairs.emplace_back(old_index, new_index);
     }
     if (num_missing_syms > 0) {
       LOG(WARNING) << "Target symbol table missing: " << num_missing_syms
@@ -167,14 +165,11 @@ void Relabel(MutableFst<Arc> *fst, const SymbolTable *old_isymbols,
              const SymbolTable *new_isymbols, bool attach_new_isymbols,
              const SymbolTable *old_osymbols, const SymbolTable *new_osymbols,
              bool attach_new_osymbols) {
-  Relabel(fst,
-          old_isymbols, new_isymbols, "" /* no unknown isymbol */,
-          attach_new_isymbols,
-          old_osymbols, new_osymbols, "" /* no unknown ioymbol */,
-          attach_new_osymbols);
+  Relabel(fst, old_isymbols, new_isymbols, "" /* no unknown isymbol */,
+          attach_new_isymbols, old_osymbols, new_osymbols,
+          "" /* no unknown ioymbol */, attach_new_osymbols);
 }
 
-
 // Relabels either the input labels or output labels. The old to
 // new labels are specified using symbol tables. Any label associations not
 // specified are assumed to be identity mapping.
@@ -228,20 +223,12 @@ class RelabelFstImpl : public CacheImpl<Arc> {
                  const RelabelFstOptions &opts)
       : CacheImpl<Arc>(opts),
         fst_(fst.Copy()),
-        relabel_input_(false),
-        relabel_output_(false) {
+        input_map_(ipairs.begin(), ipairs.end()),
+        output_map_(opairs.begin(), opairs.end()),
+        relabel_input_(!ipairs.empty()),
+        relabel_output_(!opairs.empty()) {
     SetProperties(RelabelProperties(fst.Properties(kCopyProperties, false)));
     SetType("relabel");
-    // Creates input label map.
-    if (!ipairs.empty()) {
-      for (auto &ipair : ipairs) input_map_[ipair.first] = ipair.second;
-      relabel_input_ = true;
-    }
-    // Creates output label map.
-    if (!opairs.empty()) {
-      for (auto &opair : opairs) output_map_[opair.first] = opair.second;
-      relabel_output_ = true;
-    }
   }
 
   RelabelFstImpl(const Fst<Arc> &fst, const SymbolTable *old_isymbols,
@@ -258,18 +245,16 @@ class RelabelFstImpl : public CacheImpl<Arc> {
     SetOutputSymbols(old_osymbols);
     if (old_isymbols && new_isymbols &&
         old_isymbols->LabeledCheckSum() != new_isymbols->LabeledCheckSum()) {
-      for (SymbolTableIterator siter(*old_isymbols); !siter.Done();
-           siter.Next()) {
-        input_map_[siter.Value()] = new_isymbols->Find(siter.Symbol());
+      for (const auto &sitem : *old_isymbols) {
+        input_map_[sitem.Label()] = new_isymbols->Find(sitem.Symbol());
       }
       SetInputSymbols(new_isymbols);
       relabel_input_ = true;
     }
     if (old_osymbols && new_osymbols &&
         old_osymbols->LabeledCheckSum() != new_osymbols->LabeledCheckSum()) {
-      for (SymbolTableIterator siter(*old_osymbols); !siter.Done();
-           siter.Next()) {
-        output_map_[siter.Value()] = new_osymbols->Find(siter.Symbol());
+      for (const auto &sitem : *old_osymbols) {
+        output_map_[sitem.Label()] = new_osymbols->Find(sitem.Symbol());
       }
       SetOutputSymbols(new_osymbols);
       relabel_output_ = true;
@@ -342,7 +327,7 @@ class RelabelFstImpl : public CacheImpl<Arc> {
           arc.olabel = it->second;
         }
       }
-      PushArc(s, arc);
+      PushArc(s, std::move(arc));
     }
     SetArcs(s);
   }
@@ -397,12 +382,12 @@ class RelabelFst : public ImplToFst<internal::RelabelFstImpl<A>> {
                                                opts)) {}
 
   // See Fst<>::Copy() for doc.
-  RelabelFst(const RelabelFst<Arc> &fst, bool safe = false)
+  RelabelFst(const RelabelFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
   // Gets a copy of this RelabelFst. See Fst<>::Copy() for further doc.
-  RelabelFst<Arc> *Copy(bool safe = false) const override {
-    return new RelabelFst<Arc>(*this, safe);
+  RelabelFst *Copy(bool safe = false) const override {
+    return new RelabelFst(*this, safe);
   }
 
   void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -444,7 +429,7 @@ class StateIterator<RelabelFst<Arc>> : public StateIteratorBase<Arc> {
   }
 
  private:
-  const internal::RelabelFstImpl<Arc>* impl_;
+  const internal::RelabelFstImpl<Arc> *impl_;
   StateIterator<Fst<Arc>> siter_;
   StateId s_;
 
@@ -475,4 +460,4 @@ using StdRelabelFst = RelabelFst<StdArc>;
 
 }  // namespace fst
 
-#endif  // FST_LIB_RELABEL_H_
+#endif  // FST_RELABEL_H_
