@@ -3,8 +3,8 @@
 //
 // Functions and classes for the recursive replacement of FSTs.
 
-#ifndef FST_LIB_REPLACE_H_
-#define FST_LIB_REPLACE_H_
+#ifndef FST_REPLACE_H_
+#define FST_REPLACE_H_
 
 #include <set>
 #include <string>
@@ -12,6 +12,7 @@
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/cache.h>
@@ -120,9 +121,9 @@ class ReplaceFstStateFingerprint {
 template <typename S, typename P>
 class ReplaceHash {
  public:
-  size_t operator()(const ReplaceStateTuple<S, P>& t) const {
-    static constexpr auto prime0 = 7853;
-    static constexpr auto prime1 = 7867;
+  size_t operator()(const ReplaceStateTuple<S, P> &t) const {
+    static constexpr size_t prime0 = 7853;
+    static constexpr size_t prime1 = 7867;
     return t.prefix_id + t.fst_id * prime0 + t.fst_state * prime1;
   }
 };
@@ -179,7 +180,7 @@ class ReplaceStackPrefixHash {
   size_t operator()(const ReplaceStackPrefix<Label, StateId> &prefix) const {
     size_t sum = 0;
     for (const auto &pair : prefix.prefix_) {
-      static constexpr auto prime = 7863;
+      static constexpr size_t prime = 7863;
       sum += pair.fst_id + pair.nextstate * prime;
     }
     return sum;
@@ -224,9 +225,9 @@ class VectorHashReplaceStateTable {
       }
     }
     state_table_.reset(
-        new StateTable(new ReplaceRootSelector<StateId, PrefixId>,
-                       new ReplaceFstStateFingerprint<StateId, PrefixId>,
-                       new ReplaceFingerprint<StateId, PrefixId>(&size_array_),
+        new StateTable(ReplaceRootSelector<StateId, PrefixId>(),
+                       ReplaceFstStateFingerprint<StateId, PrefixId>(),
+                       ReplaceFingerprint<StateId, PrefixId>(&size_array_),
                        root_size_, root_size_ + size_array_.back()));
   }
 
@@ -236,9 +237,9 @@ class VectorHashReplaceStateTable {
         size_array_(table.size_array_),
         prefix_table_(table.prefix_table_) {
     state_table_.reset(
-        new StateTable(new ReplaceRootSelector<StateId, PrefixId>,
-                       new ReplaceFstStateFingerprint<StateId, PrefixId>,
-                       new ReplaceFingerprint<StateId, PrefixId>(&size_array_),
+        new StateTable(ReplaceRootSelector<StateId, PrefixId>(),
+                       ReplaceFstStateFingerprint<StateId, PrefixId>(),
+                       ReplaceFingerprint<StateId, PrefixId>(&size_array_),
                        root_size_, root_size_ + size_array_.back()));
   }
 
@@ -252,7 +253,7 @@ class VectorHashReplaceStateTable {
     return prefix_table_.FindId(prefix);
   }
 
-  const StackPrefix& GetStackPrefix(PrefixId id) const {
+  const StackPrefix &GetStackPrefix(PrefixId id) const {
     return prefix_table_.FindEntry(id);
   }
 
@@ -472,10 +473,10 @@ class ReplaceFstImpl
   using FstImpl<Arc>::InputSymbols;
   using FstImpl<Arc>::OutputSymbols;
 
-  using CacheImpl::PushArc;
   using CacheImpl::HasArcs;
   using CacheImpl::HasFinal;
   using CacheImpl::HasStart;
+  using CacheImpl::PushArc;
   using CacheImpl::SetArcs;
   using CacheImpl::SetFinal;
   using CacheImpl::SetStart;
@@ -735,10 +736,10 @@ class ReplaceFstImpl
     ArcIterator<Fst<Arc>> aiter(*fst_array_[tuple.fst_id], tuple.fst_state);
     Arc arc;
     // Creates a final arc when needed.
-    if (ComputeFinalArc(tuple, &arc)) PushArc(s, arc);
+    if (ComputeFinalArc(tuple, &arc)) PushArc(s, std::move(arc));
     // Expands all arcs leaving the state.
     for (; !aiter.Done(); aiter.Next()) {
-      if (ComputeArc(tuple, aiter.Value(), &arc)) PushArc(s, arc);
+      if (ComputeArc(tuple, aiter.Value(), &arc)) PushArc(s, std::move(arc));
     }
     SetArcs(s);
   }
@@ -763,7 +764,7 @@ class ReplaceFstImpl
   // If acpp is null, only returns true if a final arcp is required, but does
   // not actually compute it.
   bool ComputeFinalArc(const StateTuple &tuple, Arc *arcp,
-                       uint32 flags = kArcValueFlags) {
+                       uint8 flags = kArcValueFlags) {
     const auto fst_state = tuple.fst_state;
     if (fst_state == kNoStateId) return false;
     // If state is final, pops the stack.
@@ -794,7 +795,7 @@ class ReplaceFstImpl
   // Returns false if the underlying arc corresponds to no arc in the resulting
   // FST.
   bool ComputeArc(const StateTuple &tuple, const Arc &arc, Arc *arcp,
-                  uint32 flags = kArcValueFlags) {
+                  uint8 flags = kArcValueFlags) {
     if (!EpsilonOnInput(call_label_type_) &&
         (flags == (flags & (kArcILabelValue | kArcWeightValue)))) {
       *arcp = arc;
@@ -848,8 +849,8 @@ class ReplaceFstImpl
   }
 
   // Returns the arc iterator flags supported by this FST.
-  uint32 ArcIteratorFlags() const {
-    uint32 flags = kArcValueFlags;
+  uint8 ArcIteratorFlags() const {
+    uint8 flags = kArcValueFlags;
     if (!always_cache_) flags |= kArcNoCache;
     return flags;
   }
@@ -974,14 +975,12 @@ class ReplaceFst
       : ImplToFst<Impl>(std::make_shared<Impl>(fst_array, opts)) {}
 
   // See Fst<>::Copy() for doc.
-  ReplaceFst(const ReplaceFst<Arc, StateTable, CacheStore> &fst,
-             bool safe = false)
+  ReplaceFst(const ReplaceFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
   // Get a copy of this ReplaceFst. See Fst<>::Copy() for further doc.
-  ReplaceFst<Arc, StateTable, CacheStore> *Copy(
-      bool safe = false) const override {
-    return new ReplaceFst<Arc, StateTable, CacheStore>(*this, safe);
+  ReplaceFst *Copy(bool safe = false) const override {
+    return new ReplaceFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -994,7 +993,7 @@ class ReplaceFst
     if ((GetImpl()->ArcIteratorFlags() & kArcNoCache) &&
         ((match_type == MATCH_INPUT && Properties(kILabelSorted, false)) ||
          (match_type == MATCH_OUTPUT && Properties(kOLabelSorted, false)))) {
-      return new ReplaceFstMatcher<Arc, StateTable, CacheStore>(*this,
+      return new ReplaceFstMatcher<Arc, StateTable, CacheStore>(this,
                                                                 match_type);
     } else {
       VLOG(2) << "Not using replace matcher";
@@ -1193,9 +1192,9 @@ class ArcIterator<ReplaceFst<Arc, StateTable, CacheStore>> {
 
   void Seek(size_t pos) { pos_ = pos; }
 
-  uint32 Flags() const { return flags_; }
+  uint8 Flags() const { return flags_; }
 
-  void SetFlags(uint32 flags, uint32 mask) {
+  void SetFlags(uint8 flags, uint8 mask) {
     // Updates the flags taking into account what flags are supported
     // by the FST.
     flags_ &= ~mask;
@@ -1219,16 +1218,16 @@ class ArcIterator<ReplaceFst<Arc, StateTable, CacheStore>> {
   ssize_t pos_;             // Current position.
   mutable ssize_t offset_;  // Offset between position in iterator and in arcs_.
   ssize_t num_arcs_;        // Number of arcs at state_.
-  uint32 flags_;            // Behavorial flags for the arc iterator
+  uint8 flags_;             // Behavorial flags for the arc iterator
   mutable Arc arc_;         // Memory to temporarily store computed arcs.
 
   mutable ArcIteratorData<Arc> cache_data_;  // Arc iterator data in cache.
   mutable ArcIteratorData<Arc> local_data_;  // Arc iterator data in local FST.
 
-  mutable const Arc *arcs_;     // Array of arcs.
-  mutable uint32 data_flags_;   // Arc value flags valid for data in arcs_.
-  mutable Arc final_arc_;       // Final arc (when required).
-  mutable uint32 final_flags_;  // Arc value flags valid for final_arc_.
+  mutable const Arc *arcs_;    // Array of arcs.
+  mutable uint8 data_flags_;   // Arc value flags valid for data in arcs_.
+  mutable Arc final_arc_;      // Final arc (when required).
+  mutable uint8 final_flags_;  // Arc value flags valid for final_arc_.
 
   ArcIterator(const ArcIterator &) = delete;
   ArcIterator &operator=(const ArcIterator &) = delete;
@@ -1246,9 +1245,11 @@ class ReplaceFstMatcher : public MatcherBase<Arc> {
 
   using StateTuple = typename StateTable::StateTuple;
 
+  // This makes a copy of the FST.
   ReplaceFstMatcher(const ReplaceFst<Arc, StateTable, CacheStore> &fst,
                     MatchType match_type)
-      : fst_(fst),
+      : owned_fst_(fst.Copy()),
+        fst_(*owned_fst_),
         impl_(fst_.GetMutableImpl()),
         s_(fst::kNoStateId),
         match_type_(match_type),
@@ -1261,10 +1262,26 @@ class ReplaceFstMatcher : public MatcherBase<Arc> {
     InitMatchers();
   }
 
-  ReplaceFstMatcher(
-      const ReplaceFstMatcher<Arc, StateTable, CacheStore> &matcher,
-      bool safe = false)
-      : fst_(matcher.fst_),
+  // This doesn't copy the FST.
+  ReplaceFstMatcher(const ReplaceFst<Arc, StateTable, CacheStore> *fst,
+                    MatchType match_type)
+      : fst_(*fst),
+        impl_(fst_.GetMutableImpl()),
+        s_(fst::kNoStateId),
+        match_type_(match_type),
+        current_loop_(false),
+        final_arc_(false),
+        loop_(kNoLabel, 0, Weight::One(), kNoStateId) {
+    if (match_type_ == fst::MATCH_OUTPUT) {
+      std::swap(loop_.ilabel, loop_.olabel);
+    }
+    InitMatchers();
+  }
+
+  // This makes a copy of the FST.
+  ReplaceFstMatcher(const ReplaceFstMatcher &matcher, bool safe = false)
+      : owned_fst_(matcher.fst_.Copy(safe)),
+        fst_(*owned_fst_),
         impl_(fst_.GetMutableImpl()),
         s_(fst::kNoStateId),
         match_type_(matcher.match_type_),
@@ -1296,9 +1313,8 @@ class ReplaceFstMatcher : public MatcherBase<Arc> {
     }
   }
 
-  ReplaceFstMatcher<Arc, StateTable, CacheStore> *Copy(
-      bool safe = false) const override {
-    return new ReplaceFstMatcher<Arc, StateTable, CacheStore>(*this, safe);
+  ReplaceFstMatcher *Copy(bool safe = false) const override {
+    return new ReplaceFstMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override {
@@ -1393,6 +1409,7 @@ class ReplaceFstMatcher : public MatcherBase<Arc> {
   ssize_t Priority(StateId s) final { return fst_.NumArcs(s); }
 
  private:
+  std::unique_ptr<const ReplaceFst<Arc, StateTable, CacheStore>> owned_fst_;
   const ReplaceFst<Arc, StateTable, CacheStore> &fst_;
   internal::ReplaceFstImpl<Arc, StateTable, CacheStore> *impl_;
   LocalMatcher *current_matcher_;
@@ -1467,4 +1484,4 @@ void Replace(const std::vector<std::pair<typename Arc::Label, const Fst<Arc> *>>
 
 }  // namespace fst
 
-#endif  // FST_LIB_REPLACE_H_
+#endif  // FST_REPLACE_H_
