@@ -3,8 +3,8 @@
 //
 // Functions and classes that implemement epsilon-removal.
 
-#ifndef FST_LIB_RMEPSILON_H_
-#define FST_LIB_RMEPSILON_H_
+#ifndef FST_RMEPSILON_H_
+#define FST_RMEPSILON_H_
 
 #include <forward_list>
 #include <stack>
@@ -13,6 +13,7 @@
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/arcfilter.h>
@@ -29,9 +30,8 @@
 namespace fst {
 
 template <class Arc, class Queue>
-class RmEpsilonOptions
+struct RmEpsilonOptions
     : public ShortestDistanceOptions<Arc, Queue, EpsilonArcFilter<Arc>> {
- public:
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
@@ -39,7 +39,7 @@ class RmEpsilonOptions
   Weight weight_threshold;  // Pruning weight threshold.
   StateId state_threshold;  // Pruning state threshold.
 
-  explicit RmEpsilonOptions(Queue *queue, float delta = kDelta,
+  explicit RmEpsilonOptions(Queue *queue, float delta = kShortestDelta,
                             bool connect = true,
                             Weight weight_threshold = Weight::Zero(),
                             StateId state_threshold = kNoStateId)
@@ -71,7 +71,7 @@ class RmEpsilonState {
 
   std::vector<Arc> &Arcs() { return arcs_; }
 
-  const Weight &Final() const { return final_; }
+  const Weight &Final() const { return final_weight_; }
 
   bool Error() const { return sd_state_.Error(); }
 
@@ -90,10 +90,11 @@ class RmEpsilonState {
   struct ElementHash {
    public:
     size_t operator()(const Element &element) const {
-      static constexpr auto prime0 = 7853;
-      static constexpr auto prime1 = 7867;
-      return element.nextstate + element.ilabel * prime0 +
-             element.olabel * prime1;
+      static constexpr size_t prime0 = 7853;
+      static constexpr size_t prime1 = 7867;
+      return static_cast<size_t>(element.nextstate) +
+             static_cast<size_t>(element.ilabel) * prime0 +
+             static_cast<size_t>(element.olabel) * prime1;
     }
   };
 
@@ -121,9 +122,9 @@ class RmEpsilonState {
   std::stack<StateId> eps_queue_;  // Queue used to visit the epsilon-closure.
   std::vector<bool> visited_;      // True if the state has been visited.
   std::forward_list<StateId> visited_states_;  // List of visited states.
-  std::vector<Arc> arcs_;                      // Arcs of state being expanded
-  Weight final_;       // Final weight of state being expanded.
-  StateId expand_id_;  // Unique ID for each call to Expand
+  std::vector<Arc> arcs_;                      // Arcs of state being expanded.
+  Weight final_weight_;  // Final weight of state being expanded.
+  StateId expand_id_;    // Unique ID for each call to Expand
 
   RmEpsilonState(const RmEpsilonState &) = delete;
   RmEpsilonState &operator=(const RmEpsilonState &) = delete;
@@ -131,7 +132,7 @@ class RmEpsilonState {
 
 template <class Arc, class Queue>
 void RmEpsilonState<Arc, Queue>::Expand(typename Arc::StateId source) {
-  final_ = Weight::Zero();
+  final_weight_ = Weight::Zero();
   arcs_.clear();
   sd_state_.ShortestDistance(source);
   if (sd_state_.Error()) return;
@@ -139,7 +140,9 @@ void RmEpsilonState<Arc, Queue>::Expand(typename Arc::StateId source) {
   while (!eps_queue_.empty()) {
     const auto state = eps_queue_.top();
     eps_queue_.pop();
-    while (visited_.size() <= state) visited_.push_back(false);
+    if (static_cast<decltype(state)>(visited_.size()) <= state) {
+      visited_.resize(state + 1, false);
+    }
     if (visited_[state]) continue;
     visited_[state] = true;
     visited_states_.push_front(state);
@@ -148,14 +151,17 @@ void RmEpsilonState<Arc, Queue>::Expand(typename Arc::StateId source) {
       auto arc = aiter.Value();
       arc.weight = Times((*distance_)[state], arc.weight);
       if (eps_filter_(arc)) {
-        while (visited_.size() <= arc.nextstate) visited_.push_back(false);
+        if (static_cast<decltype(arc.nextstate)>(visited_.size()) <=
+            arc.nextstate) {
+          visited_.resize(arc.nextstate + 1, false);
+        }
         if (!visited_[arc.nextstate]) eps_queue_.push(arc.nextstate);
       } else {
         const Element element(arc.ilabel, arc.olabel, arc.nextstate);
-        auto insert_result = element_map_.insert(
-            std::make_pair(element, std::make_pair(expand_id_, arcs_.size())));
+        auto insert_result = element_map_.emplace(
+            element, std::make_pair(expand_id_, arcs_.size()));
         if (insert_result.second) {
-          arcs_.push_back(arc);
+          arcs_.push_back(std::move(arc));
         } else {
           if (insert_result.first->second.first == expand_id_) {
             auto &weight = arcs_[insert_result.first->second.second].weight;
@@ -163,12 +169,13 @@ void RmEpsilonState<Arc, Queue>::Expand(typename Arc::StateId source) {
           } else {
             insert_result.first->second.first = expand_id_;
             insert_result.first->second.second = arcs_.size();
-            arcs_.push_back(arc);
+            arcs_.push_back(std::move(arc));
           }
         }
       }
     }
-    final_ = Plus(final_, Times((*distance_)[state], fst_.Final(state)));
+    final_weight_ =
+        Plus(final_weight_, Times((*distance_)[state], fst_.Final(state)));
   }
   while (!visited_states_.empty()) {
     visited_[visited_states_.front()] = false;
@@ -184,18 +191,17 @@ void RmEpsilonState<Arc, Queue>::Expand(typename Arc::StateId source) {
 // such epsilon transitions. This version modifies its input. It allows fine
 // control via the options argument; see below for a simpler interface.
 //
-// Thei distance vector will be used to hold the shortest distances during the
+// The distance vector will be used to hold the shortest distances during the
 // epsilon-closure computation. The state queue discipline and convergence delta
 // are taken in the options argument.
 template <class Arc, class Queue>
 void RmEpsilon(MutableFst<Arc> *fst,
                std::vector<typename Arc::Weight> *distance,
                const RmEpsilonOptions<Arc, Queue> &opts) {
-  using Label = typename Arc::Label;
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
   if (fst->Start() == kNoStateId) return;
-  // noneps_in[s] will be set to true iff s'admits a non-epsilon incoming
+  // noneps_in[s] will be set to true iff s admits a non-epsilon incoming
   // transition or is the start state.
   std::vector<bool> noneps_in(fst->NumStates(), false);
   noneps_in[fst->Start()] = true;
@@ -310,7 +316,7 @@ template <class Arc>
 void RmEpsilon(MutableFst<Arc> *fst, bool connect = true,
                typename Arc::Weight weight_threshold = Arc::Weight::Zero(),
                typename Arc::StateId state_threshold = kNoStateId,
-               float delta = kDelta) {
+               float delta = kShortestDelta) {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
   std::vector<Weight> distance;
@@ -323,10 +329,11 @@ void RmEpsilon(MutableFst<Arc> *fst, bool connect = true,
 struct RmEpsilonFstOptions : CacheOptions {
   float delta;
 
-  explicit RmEpsilonFstOptions(const CacheOptions &opts, float delta = kDelta)
+  explicit RmEpsilonFstOptions(const CacheOptions &opts,
+                               float delta = kShortestDelta)
       : CacheOptions(opts), delta(delta) {}
 
-  explicit RmEpsilonFstOptions(float delta = kDelta) : delta(delta) {}
+  explicit RmEpsilonFstOptions(float delta = kShortestDelta) : delta(delta) {}
 };
 
 namespace internal {
@@ -429,7 +436,7 @@ class RmEpsilonFstImpl : public CacheImpl<Arc> {
     SetFinal(s, rmeps_state_.Final());
     auto &arcs = rmeps_state_.Arcs();
     while (!arcs.empty()) {
-      PushArc(s, arcs.back());
+      PushArc(s, std::move(arcs.back()));
       arcs.pop_back();
     }
     SetArcs(s);
@@ -490,12 +497,12 @@ class RmEpsilonFst : public ImplToFst<internal::RmEpsilonFstImpl<A>> {
       : ImplToFst<Impl>(std::make_shared<Impl>(fst, opts)) {}
 
   // See Fst<>::Copy() for doc.
-  RmEpsilonFst(const RmEpsilonFst<Arc> &fst, bool safe = false)
+  RmEpsilonFst(const RmEpsilonFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
   // Get a copy of this RmEpsilonFst. See Fst<>::Copy() for further doc.
-  RmEpsilonFst<Arc> *Copy(bool safe = false) const override {
-    return new RmEpsilonFst<Arc>(*this, safe);
+  RmEpsilonFst *Copy(bool safe = false) const override {
+    return new RmEpsilonFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -544,4 +551,4 @@ using StdRmEpsilonFst = RmEpsilonFst<StdArc>;
 
 }  // namespace fst
 
-#endif  // FST_LIB_RMEPSILON_H_
+#endif  // FST_RMEPSILON_H_
