@@ -3,12 +3,14 @@
 //
 // Function to remove of final states that have epsilon-only input arcs.
 
-#ifndef FST_LIB_RMFINALEPSILON_H_
-#define FST_LIB_RMFINALEPSILON_H_
+#ifndef FST_RMFINALEPSILON_H_
+#define FST_RMFINALEPSILON_H_
 
 #include <unordered_set>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/connect.h>
 #include <fst/mutable-fst.h>
 
@@ -77,4 +79,4 @@ void RmFinalEpsilon(MutableFst<Arc> *fst) {
 
 }  // namespace fst
 
-#endif  // FST_LIB_RMFINALEPSILON_H_
+#endif  // FST_RMFINALEPSILON_H_
