@@ -7,8 +7,8 @@
 #include <memory>
 #include <utility>
 
+#include <fst/types.h>
 #include <fst/fstlib.h>
-#include <fst/script/arg-packs.h>
 #include <fst/script/fst-class.h>
 
 // Scripting API support for ArcIterator.
@@ -25,12 +25,12 @@ namespace script {
 class ArcIteratorImplBase {
  public:
   virtual bool Done() const = 0;
-  virtual uint32 Flags() const = 0;
+  virtual uint8 Flags() const = 0;
   virtual void Next() = 0;
   virtual size_t Position() const = 0;
   virtual void Reset() = 0;
   virtual void Seek(size_t a) = 0;
-  virtual void SetFlags(uint32 flags, uint32 mask) = 0;
+  virtual void SetFlags(uint8 flags, uint8 mask) = 0;
   virtual ArcClass Value() const = 0;
   virtual ~ArcIteratorImplBase() {}
 };
@@ -44,7 +44,7 @@ class ArcIteratorClassImpl : public ArcIteratorImplBase {
 
   bool Done() const final { return aiter_.Done(); }
 
-  uint32 Flags() const final { return aiter_.Flags(); }
+  uint8 Flags() const final { return aiter_.Flags(); }
 
   void Next() final { aiter_.Next(); }
 
@@ -54,9 +54,7 @@ class ArcIteratorClassImpl : public ArcIteratorImplBase {
 
   void Seek(size_t a) final { aiter_.Seek(a); }
 
-  void SetFlags(uint32 flags, uint32 mask) final {
-    aiter_.SetFlags(flags, mask);
-  }
+  void SetFlags(uint8 flags, uint8 mask) final { aiter_.SetFlags(flags, mask); }
 
   // This is returned by value because it has not yet been constructed, and
   // is likely to participate in return-value optimization.
@@ -71,7 +69,7 @@ class ArcIteratorClassImpl : public ArcIteratorImplBase {
 class ArcIteratorClass;
 
 using InitArcIteratorClassArgs =
-    args::Package<const FstClass &, int64, ArcIteratorClass *>;
+    std::tuple<const FstClass &, int64, ArcIteratorClass *>;
 
 // Untemplated user-facing class holding a templated pimpl.
 class ArcIteratorClass {
@@ -84,7 +82,7 @@ class ArcIteratorClass {
 
   bool Done() const { return impl_->Done(); }
 
-  uint32 Flags() const { return impl_->Flags(); }
+  uint8 Flags() const { return impl_->Flags(); }
 
   void Next() { impl_->Next(); }
 
@@ -94,7 +92,7 @@ class ArcIteratorClass {
 
   void Seek(size_t a) { impl_->Seek(a); }
 
-  void SetFlags(uint32 flags, uint32 mask) { impl_->SetFlags(flags, mask); }
+  void SetFlags(uint8 flags, uint8 mask) { impl_->SetFlags(flags, mask); }
 
   ArcClass Value() const { return impl_->Value(); }
 
@@ -107,8 +105,9 @@ class ArcIteratorClass {
 
 template <class Arc>
 void InitArcIteratorClass(InitArcIteratorClassArgs *args) {
-  const Fst<Arc> &fst = *(args->arg1.GetFst<Arc>());
-  args->arg3->impl_.reset(new ArcIteratorClassImpl<Arc>(fst, args->arg2));
+  const Fst<Arc> &fst = *std::get<0>(*args).GetFst<Arc>();
+  std::get<2>(*args)->impl_.reset(
+      new ArcIteratorClassImpl<Arc>(fst, std::get<1>(*args)));
 }
 
 // Mutable arc iterators.
@@ -123,15 +122,14 @@ class MutableArcIteratorImplBase : public ArcIteratorImplBase {
 
 // Templated implementation.
 template <class Arc>
-class MutableArcIteratorClassImpl
-    : public MutableArcIteratorImplBase {
+class MutableArcIteratorClassImpl : public MutableArcIteratorImplBase {
  public:
   explicit MutableArcIteratorClassImpl(MutableFst<Arc> *fst, int64 s)
       : aiter_(fst, s) {}
 
   bool Done() const final { return aiter_.Done(); }
 
-  uint32 Flags() const final { return aiter_.Flags(); }
+  uint8 Flags() const final { return aiter_.Flags(); }
 
   void Next() final { aiter_.Next(); }
 
@@ -141,13 +139,9 @@ class MutableArcIteratorClassImpl
 
   void Seek(size_t a) final { aiter_.Seek(a); }
 
-  void SetFlags(uint32 flags, uint32 mask) final {
-    aiter_.SetFlags(flags, mask);
-  }
+  void SetFlags(uint8 flags, uint8 mask) final { aiter_.SetFlags(flags, mask); }
 
-  void SetValue(const Arc &arc) { aiter_.SetValue(arc); }
-
-  void SetValue(const ArcClass &ac) final { aiter_.SetValue(ac.GetArc<Arc>()); }
+  void SetValue(const ArcClass &ac) final { SetValue(ac.GetArc<Arc>()); }
 
   // This is returned by value because it has not yet been constructed, and
   // is likely to participate in return-value optimization.
@@ -156,13 +150,15 @@ class MutableArcIteratorClassImpl
   ~MutableArcIteratorClassImpl() override {}
 
  private:
+  void SetValue(const Arc &arc) { aiter_.SetValue(arc); }
+
   MutableArcIterator<MutableFst<Arc>> aiter_;
 };
 
 class MutableArcIteratorClass;
 
 using InitMutableArcIteratorClassArgs =
-    args::Package<MutableFstClass *, int64, MutableArcIteratorClass *>;
+    std::tuple<MutableFstClass *, int64, MutableArcIteratorClass *>;
 
 // Untemplated user-facing class holding a templated pimpl.
 class MutableArcIteratorClass {
@@ -175,7 +171,7 @@ class MutableArcIteratorClass {
 
   bool Done() const { return impl_->Done(); }
 
-  uint32 Flags() const { return impl_->Flags(); }
+  uint8 Flags() const { return impl_->Flags(); }
 
   void Next() { impl_->Next(); }
 
@@ -185,7 +181,7 @@ class MutableArcIteratorClass {
 
   void Seek(size_t a) { impl_->Seek(a); }
 
-  void SetFlags(uint32 flags, uint32 mask) { impl_->SetFlags(flags, mask); }
+  void SetFlags(uint8 flags, uint8 mask) { impl_->SetFlags(flags, mask); }
 
   void SetValue(const ArcClass &ac) { impl_->SetValue(ac); }
 
@@ -201,9 +197,9 @@ class MutableArcIteratorClass {
 
 template <class Arc>
 void InitMutableArcIteratorClass(InitMutableArcIteratorClassArgs *args) {
-  MutableFst<Arc> *fst = args->arg1->GetMutableFst<Arc>();
-  args->arg3->impl_.reset(new MutableArcIteratorClassImpl<Arc>(fst,
-                                                               args->arg2));
+  MutableFst<Arc> *fst = std::get<0>(*args)->GetMutableFst<Arc>();
+  std::get<2>(*args)->impl_.reset(
+      new MutableArcIteratorClassImpl<Arc>(fst, std::get<1>(*args)));
 }
 
 }  // namespace script
