@@ -5,7 +5,6 @@
 #define FST_SCRIPT_INVERT_H_
 
 #include <fst/invert.h>
-#include <fst/script/arg-packs.h>
 #include <fst/script/fst-class.h>
 
 namespace fst {
@@ -13,8 +12,7 @@ namespace script {
 
 template <class Arc>
 void Invert(MutableFstClass *fst) {
-  MutableFst<Arc> *typed_fst = fst->GetMutableFst<Arc>();
-  Invert(typed_fst);
+  Invert(fst->GetMutableFst<Arc>());
 }
 
 void Invert(MutableFstClass *fst);
