@@ -4,6 +4,8 @@
 #ifndef FST_SCRIPT_ISOMORPHIC_H_
 #define FST_SCRIPT_ISOMORPHIC_H_
 
+#include <tuple>
+
 #include <fst/isomorphic.h>
 #include <fst/script/arg-packs.h>
 #include <fst/script/fst-class.h>
@@ -12,15 +14,15 @@ namespace fst {
 namespace script {
 
 using IsomorphicInnerArgs =
-    args::Package<const FstClass &, const FstClass &, float>;
+    std::tuple<const FstClass &, const FstClass &, float>;
 
-using IsomorphicArgs = args::WithReturnValue<bool, IsomorphicInnerArgs>;
+using IsomorphicArgs = WithReturnValue<bool, IsomorphicInnerArgs>;
 
 template <class Arc>
 void Isomorphic(IsomorphicArgs *args) {
-  const Fst<Arc> &fst1 = *(args->args.arg1.GetFst<Arc>());
-  const Fst<Arc> &fst2 = *(args->args.arg2.GetFst<Arc>());
-  args->retval = Isomorphic(fst1, fst2, args->args.arg3);
+  const Fst<Arc> &fst1 = *std::get<0>(args->args).GetFst<Arc>();
+  const Fst<Arc> &fst2 = *std::get<1>(args->args).GetFst<Arc>();
+  args->retval = Isomorphic(fst1, fst2, std::get<2>(args->args));
 }
 
 bool Isomorphic(const FstClass &fst1, const FstClass &fst2,
