@@ -4,26 +4,27 @@
 #ifndef FST_SCRIPT_MINIMIZE_H_
 #define FST_SCRIPT_MINIMIZE_H_
 
+#include <tuple>
+
 #include <fst/minimize.h>
-#include <fst/script/arg-packs.h>
 #include <fst/script/fst-class.h>
 
 namespace fst {
 namespace script {
 
 using MinimizeArgs =
-    args::Package<MutableFstClass *, MutableFstClass *, float, bool>;
+    std::tuple<MutableFstClass *, MutableFstClass *, float, bool>;
 
 template <class Arc>
 void Minimize(MinimizeArgs *args) {
-  MutableFst<Arc> *ofst1 = args->arg1->GetMutableFst<Arc>();
-  MutableFst<Arc> *ofst2 = args->arg2 ?
-                           args->arg2->GetMutableFst<Arc>() : nullptr;
-  Minimize(ofst1, ofst2, args->arg3, args->arg4);
+  MutableFst<Arc> *ofst1 = std::get<0>(*args)->GetMutableFst<Arc>();
+  MutableFst<Arc> *ofst2 =
+      std::get<1>(*args) ? std::get<1>(*args)->GetMutableFst<Arc>() : nullptr;
+  Minimize(ofst1, ofst2, std::get<2>(*args), std::get<3>(*args));
 }
 
 void Minimize(MutableFstClass *ofst1, MutableFstClass *ofst2 = nullptr,
-              float delta = kDelta, bool allow_nondet = false);
+              float delta = kShortestDelta, bool allow_nondet = false);
 
 }  // namespace script
 }  // namespace fst
