@@ -28,7 +28,7 @@
 //     a single struct. The template structs in arg-packs.h provide a handy
 //     way to do this. In Foo's case, that might look like this:
 //
-//       using FooArgs = args::Package<const FstClass &, MutableFstClass *>;
+//       using FooArgs = std::pair<const FstClass &, MutableFstClass *>;
 //
 //     Note: this package of args is going to be passed by non-const pointer.
 //
@@ -38,8 +38,8 @@
 //       template<class Arc>
 //       void Foo(FooArgs *args) {
 //          // Pulls out the actual, arc-templated FSTs.
-//          const Fst<Arc> &ifst = args->arg1.GetFst<Arc>();
-//          MutableFst<Arc> *ofst = args->arg2->GetMutableFst<Arc>();
+//          const Fst<Arc> &ifst = std::get<0>(*args).GetFst<Arc>();
+//          MutableFst<Arc> *ofst = std::get<1>(*args)->GetMutableFst<Arc>();
 //          // Actually perform Foo on ifst and ofst.
 //       }
 //
@@ -59,13 +59,16 @@
 //  provided you also have:
 //
 //  4) A registration for your new operation, on the arc types you care about.
-//     This can be provided easily by the REGISTER_FST_OPERATION macro in
-//     operations.h:
+//     This can be provided easily by the REGISTER_FST_OPERATION macro:
 //
 //       REGISTER_FST_OPERATION(Foo, StdArc, FooArgs);
 //       REGISTER_FST_OPERATION(Foo, MyArc, FooArgs);
 //       // .. etc
 //
+//     You can also use REGISTER_FST_OPERATION_3ARCS macro to register an
+//     operation for StdArc, LogArc, and Log64Arc:
+//
+//       REGISTER_FST_OPERATION_3ARCS(Foo, FooArcs);
 //
 //  That's it! Now when you call Foo(const FstClass &, MutableFstClass *),
 //  it dispatches (in #3) via the Apply<> function to the correct
@@ -75,20 +78,16 @@
 #ifndef FST_SCRIPT_SCRIPT_IMPL_H_
 #define FST_SCRIPT_SCRIPT_IMPL_H_
 
-//
 // This file contains general-purpose templates which are used in the
 // implementation of the operations.
-//
 
 #include <string>
 #include <utility>
 
-#include <fst/generic-register.h>
-#include <fst/script/arg-packs.h>
-#include <fst/script/fst-class.h>
-
 #include <fst/types.h>
 #include <fst/log.h>
+#include <fst/generic-register.h>
+#include <fst/script/fst-class.h>
 
 namespace fst {
 namespace script {
@@ -101,29 +100,25 @@ enum RandArcSelection {
 
 // A generic register for operations with various kinds of signatures.
 // Needed since every function signature requires a new registration class.
-// The std::pair<string, string> is understood to be the operation name and arc
-// type; subclasses (or typedefs) need only provide the operation signature.
-
+// The std::pair<std::string, std::string> is understood to be the operation
+// name and arc type; subclasses (or typedefs) need only provide the operation
+// signature.
 template <class OperationSignature>
 class GenericOperationRegister
-    : public GenericRegister<std::pair<string, string>, OperationSignature,
+    : public GenericRegister<std::pair<std::string, std::string>,
+                             OperationSignature,
                              GenericOperationRegister<OperationSignature>> {
  public:
-  void RegisterOperation(const string &operation_name, const string &arc_type,
-                         OperationSignature op) {
-    this->SetEntry(std::make_pair(operation_name, arc_type), op);
-  }
-
-  OperationSignature GetOperation(const string &operation_name,
-                                  const string &arc_type) {
+  OperationSignature GetOperation(const std::string &operation_name,
+                                  const std::string &arc_type) {
     return this->GetEntry(std::make_pair(operation_name, arc_type));
   }
 
  protected:
-  string ConvertKeyToSoFilename(
-      const std::pair<string, string> &key) const final {
+  std::string ConvertKeyToSoFilename(
+      const std::pair<std::string, std::string> &key) const final {
     // Uses the old-style FST for now.
-    string legal_type(key.second);  // The arc type.
+    std::string legal_type(key.second);  // The arc type.
     ConvertToLegalCSymbol(&legal_type);
     return legal_type + "-arc.so";
   }
@@ -145,44 +140,73 @@ struct Operation {
   // The register (hash) type.
   using Register = GenericOperationRegister<OpType>;
 
-  // The register-er type
+  // The register-er type.
   using Registerer = GenericRegisterer<Register>;
 };
 
 // Macro for registering new types of operations.
-
 #define REGISTER_FST_OPERATION(Op, Arc, ArgPack)               \
   static fst::script::Operation<ArgPack>::Registerer       \
       arc_dispatched_operation_##ArgPack##Op##Arc##_registerer \
-      (std::make_pair(#Op, Arc::Type()), Op<Arc>)
+      ({#Op, Arc::Type()}, Op<Arc>)
+
+// A macro that calls REGISTER_FST_OPERATION for widely-used arc types.
+#define REGISTER_FST_OPERATION_3ARCS(Op, ArgPack) \
+  REGISTER_FST_OPERATION(Op, StdArc, ArgPack);    \
+  REGISTER_FST_OPERATION(Op, LogArc, ArgPack);    \
+  REGISTER_FST_OPERATION(Op, Log64Arc, ArgPack)
 
 // Template function to apply an operation by name.
-
 template <class OpReg>
-void Apply(const string &op_name, const string &arc_type,
+void Apply(const std::string &op_name, const std::string &arc_type,
            typename OpReg::ArgPack *args) {
-  typename OpReg::Register *reg = OpReg::Register::GetRegister();
-  typename OpReg::OpType op = reg->GetOperation(op_name, arc_type);
+  const auto op =
+      OpReg::Register::GetRegister()->GetOperation(op_name, arc_type);
   if (!op) {
-    FSTERROR() << "No operation found for " << op_name << " on "
-               << "arc type " << arc_type;
+    FSTERROR() << op_name << ": No operation found on arc type " << arc_type;
     return;
   }
   op(args);
 }
 
-// Helper that logs to ERROR if the arc types of m and n don't match. The
-// op_name is used to construct the error message.
+namespace internal {
+
+// Helper that logs to ERROR if the arc types of m and n don't match,
+// assuming that both m and n implement .ArcType(). The op_name argument is
+// used to construct the error message.
 template <class M, class N>
-bool ArcTypesMatch(const M &m, const N &n, const string &op_name) {
+bool ArcTypesMatch(const M &m, const N &n, const std::string &op_name) {
   if (m.ArcType() != n.ArcType()) {
-    FSTERROR() << "Arguments with non-matching arc types passed to "
-               << op_name << ":\t" << m.ArcType() << " and " << n.ArcType();
+    FSTERROR() << op_name << ": Arguments with non-matching arc types "
+               << m.ArcType() << " and " << n.ArcType();
     return false;
   }
   return true;
 }
 
+// From untyped to typed weights.
+template <class Weight>
+void CopyWeights(const std::vector<WeightClass> &weights,
+                 std::vector<Weight> *typed_weights) {
+  typed_weights->clear();
+  typed_weights->reserve(weights.size());
+  for (const auto &weight : weights) {
+    typed_weights->emplace_back(*weight.GetWeight<Weight>());
+  }
+}
+
+// From typed to untyped weights.
+template <class Weight>
+void CopyWeights(const std::vector<Weight> &typed_weights,
+                 std::vector<WeightClass> *weights) {
+  weights->clear();
+  weights->reserve(typed_weights.size());
+  for (const auto &typed_weight : typed_weights) {
+    weights->emplace_back(typed_weight);
+  }
+}
+
+}  // namespace internal
 }  // namespace script
 }  // namespace fst
 
