@@ -11,11 +11,11 @@
 namespace fst {
 namespace script {
 
-using VerifyArgs = args::WithReturnValue<bool, const FstClass &>;
+using VerifyArgs = WithReturnValue<bool, const FstClass &>;
 
 template <class Arc>
 void Verify(VerifyArgs *args) {
-  const Fst<Arc> &fst = *(args->args.GetFst<Arc>());
+  const Fst<Arc> &fst = *args->args.GetFst<Arc>();
   args->retval = Verify(fst);
 }
 
