@@ -8,13 +8,14 @@
 #ifndef FST_SET_WEIGHT_H_
 #define FST_SET_WEIGHT_H_
 
-#include <cstdlib>
-
 #include <algorithm>
 #include <list>
+#include <random>
 #include <string>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/union-weight.h>
 #include <fst/weight.h>
 
@@ -34,19 +35,21 @@ constexpr char kSetSeparator = '_';  // Label separator in sets.
 // treats all non-Zero() elements as equivalent (with Zero() ==
 // UnivSet()), useful for algorithms that don't really depend on the
 // detailed sets.
-enum SetType { SET_INTERSECT_UNION = 0,
-               SET_UNION_INTERSECT = 1,
-               SET_INTERSECT_UNION_RESTRICT = 2,
-               SET_BOOLEAN = 3 };
+enum SetType {
+  SET_INTERSECT_UNION = 0,
+  SET_UNION_INTERSECT = 1,
+  SET_INTERSECT_UNION_RESTRICT = 2,
+  SET_BOOLEAN = 3
+};
 
 template <class>
 class SetWeightIterator;
 
 // Set semiring of integral labels.
-template <typename Label_, SetType S = SET_INTERSECT_UNION>
+template <typename L, SetType S = SET_INTERSECT_UNION>
 class SetWeight {
  public:
-  using Label = Label_;
+  using Label = L;
   using ReverseWeight = SetWeight<Label, S>;
   using Iterator = SetWeightIterator<SetWeight>;
   friend class SetWeightIterator<SetWeight>;
@@ -58,7 +61,7 @@ class SetWeight {
 
   // Input should be positive, sorted and unique.
   template <typename Iterator>
-  SetWeight(const Iterator &begin, const Iterator &end) {
+  SetWeight(const Iterator begin, const Iterator end) {
     for (auto iter = begin; iter != end; ++iter) PushBack(*iter);
   }
 
@@ -68,11 +71,13 @@ class SetWeight {
 
   template <SetType S2>
   explicit SetWeight(const SetWeight<Label, S2> &w)
-    : first_(w.first_), rest_(w.rest_) {}
+      : first_(w.first_), rest_(w.rest_) {}
 
   template <SetType S2>
   explicit SetWeight(SetWeight<Label, S2> &&w)
-    : first_(w.first_), rest_(std::move(w.rest_)) { w.Clear(); }
+      : first_(w.first_), rest_(std::move(w.rest_)) {
+    w.Clear();
+  }
 
   template <SetType S2>
   SetWeight &operator=(const SetWeight<Label, S2> &w) {
@@ -102,15 +107,15 @@ class SetWeight {
     return *no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type = new string(
-        S == SET_UNION_INTERSECT
-        ? "union_intersect_set"
-        : (S == SET_INTERSECT_UNION
-           ? "intersect_union_set"
-           : (S == SET_INTERSECT_UNION_RESTRICT
-              ? "restricted_set_intersect_union"
-              : "boolean_set")));
+  static const std::string &Type() {
+    static const std::string *const type =
+        new std::string(S == SET_UNION_INTERSECT
+                            ? "union_intersect_set"
+                            : (S == SET_INTERSECT_UNION
+                                   ? "intersect_union_set"
+                                   : (S == SET_INTERSECT_UNION_RESTRICT
+                                          ? "restricted_set_intersect_union"
+                                          : "boolean_set")));
     return *type;
   }
 
@@ -222,7 +227,6 @@ class SetWeightIterator {
   typename decltype(Weight::rest_)::const_iterator iter_;
 };
 
-
 // SetWeight member functions follow that require SetWeightIterator
 
 template <typename Label, SetType S>
@@ -312,8 +316,7 @@ inline bool operator!=(const SetWeight<Label, S> &w1,
 
 template <typename Label, SetType S>
 inline bool ApproxEqual(const SetWeight<Label, S> &w1,
-                        const SetWeight<Label, S> &w2,
-                        float delta = kDelta) {
+                        const SetWeight<Label, S> &w2, float delta = kDelta) {
   return w1 == w2;
 }
 
@@ -339,7 +342,7 @@ inline std::ostream &operator<<(std::ostream &strm,
 template <typename Label, SetType S>
 inline std::istream &operator>>(std::istream &strm,
                                 SetWeight<Label, S> &weight) {
-  string str;
+  std::string str;
   strm >> str;
   using Weight = SetWeight<Label, S>;
   if (str == "EmptySet") {
@@ -362,9 +365,8 @@ inline std::istream &operator>>(std::istream &strm,
 }
 
 template <typename Label, SetType S>
-inline SetWeight<Label, S> Union(
-    const SetWeight<Label, S> &w1,
-    const SetWeight<Label, S> &w2) {
+inline SetWeight<Label, S> Union(const SetWeight<Label, S> &w1,
+                                 const SetWeight<Label, S> &w2) {
   using Weight = SetWeight<Label, S>;
   using Iterator = typename SetWeight<Label, S>::Iterator;
   if (!w1.Member() || !w2.Member()) return Weight::NoWeight();
@@ -396,9 +398,8 @@ inline SetWeight<Label, S> Union(
 }
 
 template <typename Label, SetType S>
-inline SetWeight<Label, S> Intersect(
-    const SetWeight<Label, S> &w1,
-    const SetWeight<Label, S> &w2) {
+inline SetWeight<Label, S> Intersect(const SetWeight<Label, S> &w1,
+                                     const SetWeight<Label, S> &w2) {
   using Weight = SetWeight<Label, S>;
   using Iterator = typename SetWeight<Label, S>::Iterator;
   if (!w1.Member() || !w2.Member()) return Weight::NoWeight();
@@ -426,9 +427,8 @@ inline SetWeight<Label, S> Intersect(
 }
 
 template <typename Label, SetType S>
-inline SetWeight<Label, S> Difference(
-    const SetWeight<Label, S> &w1,
-    const SetWeight<Label, S> &w2) {
+inline SetWeight<Label, S> Difference(const SetWeight<Label, S> &w1,
+                                      const SetWeight<Label, S> &w2) {
   using Weight = SetWeight<Label, S>;
   using Iterator = typename SetWeight<Label, S>::Iterator;
   if (!w1.Member() || !w2.Member()) return Weight::NoWeight();
@@ -457,9 +457,8 @@ inline SetWeight<Label, S> Difference(
 
 // Default: Plus = Intersect.
 template <typename Label, SetType S>
-inline SetWeight<Label, S> Plus(
-    const SetWeight<Label, S> &w1,
-    const SetWeight<Label, S> &w2) {
+inline SetWeight<Label, S> Plus(const SetWeight<Label, S> &w1,
+                                const SetWeight<Label, S> &w2) {
   return Intersect(w1, w2);
 }
 
@@ -505,9 +504,8 @@ inline SetWeight<Label, SET_BOOLEAN> Plus(
 
 // Default: Times = Union.
 template <typename Label, SetType S>
-inline SetWeight<Label, S> Times(
-    const SetWeight<Label, S> &w1,
-    const SetWeight<Label, S> &w2) {
+inline SetWeight<Label, S> Times(const SetWeight<Label, S> &w1,
+                                 const SetWeight<Label, S> &w2) {
   return Union(w1, w2);
 }
 
@@ -584,7 +582,8 @@ class WeightGenerate<SetWeight<Label, S>> {
  public:
   using Weight = SetWeight<Label, S>;
 
-  explicit WeightGenerate(bool allow_zero = true,
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true,
                           size_t alphabet_size = kNumRandomWeights,
                           size_t max_set_length = kNumRandomWeights)
       : allow_zero_(allow_zero),
@@ -592,11 +591,14 @@ class WeightGenerate<SetWeight<Label, S>> {
         max_set_length_(max_set_length) {}
 
   Weight operator()() const {
-    const size_t n = rand() % (max_set_length_ + allow_zero_);  // NOLINT
+    const int n = std::uniform_int_distribution<>(
+        0, max_set_length_ + allow_zero_ - 1)(rand_);
     if (allow_zero_ && n == max_set_length_) return Weight::Zero();
     std::vector<Label> labels;
-    for (size_t i = 0; i < n; ++i) {
-      labels.push_back(rand() % alphabet_size_ + 1);  // NOLINT
+    labels.reserve(n);
+    for (int i = 0; i < n; ++i) {
+      labels.push_back(
+          std::uniform_int_distribution<>(0, alphabet_size_)(rand_));
     }
     std::sort(labels.begin(), labels.end());
     const auto labels_end = std::unique(labels.begin(), labels.end());
@@ -605,11 +607,9 @@ class WeightGenerate<SetWeight<Label, S>> {
   }
 
  private:
-  // Permits Zero() and zero divisors.
+  mutable std::mt19937_64 rand_;
   const bool allow_zero_;
-  // Alphabet size for random weights.
   const size_t alphabet_size_;
-  // Number of alternative random weights.
   const size_t max_set_length_;
 };
 
