@@ -3,10 +3,11 @@
 //
 // Functions to find shortest paths in an FST.
 
-#ifndef FST_LIB_SHORTEST_PATH_H_
-#define FST_LIB_SHORTEST_PATH_H_
+#ifndef FST_SHORTEST_PATH_H_
+#define FST_SHORTEST_PATH_H_
 
 #include <functional>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -33,15 +34,18 @@ struct ShortestPathOptions
                       // shortest distance from the initial state.
   bool first_path;    // Single shortest path stops after finding the first
                       // path to a final state; that path is the shortest path
-                      // only when using the ShortestFirstQueue and
-                      // only when all the weights in the FST are between
-                      // One() and Zero() according to NaturalLess.
+                      // only when:
+                      // (1) using the ShortestFirstQueue with all the weights
+                      // in the FST being between One() and Zero() according to
+                      // NaturalLess or when
+                      // (2) using the NaturalAStarQueue with an admissible
+                      // and consistent estimate.
   Weight weight_threshold;  // Pruning weight threshold.
   StateId state_threshold;  // Pruning state threshold.
 
   ShortestPathOptions(Queue *queue, ArcFilter filter, int32 nshortest = 1,
                       bool unique = false, bool has_distance = false,
-                      float delta = kDelta, bool first_path = false,
+                      float delta = kShortestDelta, bool first_path = false,
                       Weight weight_threshold = Weight::Zero(),
                       StateId state_threshold = kNoStateId)
       : ShortestDistanceOptions<Arc, Queue, ArcFilter>(queue, filter,
@@ -85,7 +89,7 @@ void SingleShortestPathBacktrace(
       aiter.Seek(parent[d].second);
       auto arc = aiter.Value();
       arc.nextstate = d_p;
-      ofst->AddArc(s_p, arc);
+      ofst->AddArc(s_p, std::move(arc));
     }
   }
   ofst->SetStart(s_p);
@@ -125,6 +129,37 @@ void SingleShortestTree(
       kFstProperties);
 }
 
+// Implements the stopping criterion when ShortestPathOptions::first_path
+// is set to true:
+//   operator()(s, d, f) == true
+//   iff every successful path through state 's' has a cost greater or equal
+//   to 'f' under the assumption that 'd' is the shortest distance to state 's'.
+// Correct when using the ShortestFirstQueue with all the weights in the FST
+// being between One() and Zero() according to NaturalLess
+template <typename S, typename W, typename Queue>
+struct FirstPathSelect {
+  FirstPathSelect(const Queue &) {}
+  bool operator()(S s, W d, W f) const { return f == Plus(d, f); }
+};
+
+// Specialisation for A*.
+// Correct when the estimate is admissible and consistent.
+template <typename S, typename W, typename Estimate>
+class FirstPathSelect<S, W, NaturalAStarQueue<S, W, Estimate>> {
+ public:
+  using Queue = NaturalAStarQueue<S, W, Estimate>;
+
+  FirstPathSelect(const Queue &state_queue)
+      : estimate_(state_queue.GetCompare().GetEstimate()) {}
+
+  bool operator()(S s, W d, W f) const {
+    return f == Plus(Times(d, estimate_(s)), f);
+  }
+
+ private:
+  const Estimate &estimate_;
+};
+
 // Shortest-path algorithm. It builds the output mutable FST so that it contains
 // the shortest path in the input FST; distance returns the shortest distances
 // from the source state to each state in the input FST, and the options struct
@@ -146,6 +181,9 @@ bool SingleShortestPath(
     std::vector<std::pair<typename Arc::StateId, size_t>> *parent) {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
+  static_assert(IsPath<Weight>::value, "Weight must have path property.");
+  static_assert((Weight::Properties() & kRightSemiring) == kRightSemiring,
+                "Weight must be right distributive.");
   parent->clear();
   *f_parent = kNoStateId;
   if (ifst.Start() == kNoStateId) return true;
@@ -156,19 +194,13 @@ bool SingleShortestPath(
   auto f_distance = Weight::Zero();
   distance->clear();
   state_queue->Clear();
-  if ((Weight::Properties() & (kPath | kRightSemiring)) !=
-      (kPath | kRightSemiring)) {
-    FSTERROR() << "SingleShortestPath: Weight needs to have the path"
-               << " property and be right distributive: " << Weight::Type();
-    return false;
-  }
   while (distance->size() < source) {
     distance->push_back(Weight::Zero());
     enqueued.push_back(false);
-    parent->push_back(std::make_pair(kNoStateId, kNoArc));
+    parent->emplace_back(kNoStateId, kNoArc);
   }
   distance->push_back(Weight::One());
-  parent->push_back(std::make_pair(kNoStateId, kNoArc));
+  parent->emplace_back(kNoStateId, kNoArc);
   state_queue->Enqueue(source);
   enqueued.push_back(true);
   while (!state_queue->Empty()) {
@@ -178,7 +210,9 @@ bool SingleShortestPath(
     const auto sd = (*distance)[s];
     // If we are using a shortest queue, no other path is going to be shorter
     // than f_distance at this point.
-    if (opts.first_path && final_seen && f_distance == Plus(f_distance, sd)) {
+    using FirstPath = FirstPathSelect<StateId, Weight, Queue>;
+    if (opts.first_path && final_seen &&
+        FirstPath(*state_queue)(s, sd, f_distance)) {
       break;
     }
     if (ifst.Final(s) != Weight::Zero()) {
@@ -195,7 +229,7 @@ bool SingleShortestPath(
       while (distance->size() <= arc.nextstate) {
         distance->push_back(Weight::Zero());
         enqueued.push_back(false);
-        parent->push_back(std::make_pair(kNoStateId, kNoArc));
+        parent->emplace_back(kNoStateId, kNoArc);
       }
       auto &nd = (*distance)[arc.nextstate];
       const auto weight = Times(sd, arc.weight);
@@ -267,8 +301,8 @@ class ShortestPathCompare {
 //
 // The n-shortest paths are the n-lowest weight paths w.r.t. the natural
 // semiring order. The single path that can be read from the ith of at most n
-// transitions leaving the initial state of the the input FST is the ith
-// shortest path. Disregarding the initial state and initial transitions, the
+// transitions leaving the initial state of the input FST is the ith shortest
+// path. Disregarding the initial state and initial transitions, the
 // n-shortest paths, in fact, form a tree rooted at the single final state.
 //
 // The weights need to be left and right distributive (kSemiring) and have the
@@ -286,30 +320,23 @@ class ShortestPathCompare {
 // The algorithm relies on the shortest-distance algorithm. There are some
 // issues with the pseudo-code as written in the paper (viz., line 11).
 //
-// IMPLEMENTATION NOTE: The input FST can be a delayed FST and and at any state
-// in its expansion the values of distance vector need only be defined at that
-// time for the states that are known to exist.
+// IMPLEMENTATION NOTE: The input FST can be a delayed FST and at any state in
+// its expansion the values of distance vector need only be defined at that time
+// for the states that are known to exist.
 template <class Arc, class RevArc>
 void NShortestPath(const Fst<RevArc> &ifst, MutableFst<Arc> *ofst,
                    const std::vector<typename Arc::Weight> &distance,
-                   int32 nshortest, float delta = kDelta,
+                   int32 nshortest, float delta = kShortestDelta,
                    typename Arc::Weight weight_threshold = Arc::Weight::Zero(),
                    typename Arc::StateId state_threshold = kNoStateId) {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
   using Pair = std::pair<StateId, Weight>;
+  static_assert((Weight::Properties() & kPath) == kPath,
+                "Weight must have path property.");
+  static_assert((Weight::Properties() & kSemiring) == kSemiring,
+                "Weight must be distributive.");
   if (nshortest <= 0) return;
-  // TODO(kbg): Make this a compile-time static_assert once:
-  // 1) All weight properties are made constexpr for all weight types.
-  // 2) We have a pleasant way to "deregister" this operation for non-path
-  //    semirings so an informative error message is produced. The best
-  //    solution will probably involve some kind of SFINAE magic.
-  if ((Weight::Properties() & (kPath | kSemiring)) != (kPath | kSemiring)) {
-    FSTERROR() << "NShortestPath: Weight needs to have the "
-               << "path property and be distributive: " << Weight::Type();
-    ofst->SetProperties(kError, kError);
-    return;
-  }
   ofst->DeleteStates();
   ofst->SetInputSymbols(ifst.InputSymbols());
   ofst->SetOutputSymbols(ifst.OutputSymbols());
@@ -318,7 +345,7 @@ void NShortestPath(const Fst<RevArc> &ifst, MutableFst<Arc> *ofst,
   // (s, w). The vector pairs maps each state in ofst to the corresponding
   // pair maps states in ofst to the corresponding pair (s, w).
   std::vector<Pair> pairs;
-  // The supefinal state is denoted by kNoStateId. The distance from the
+  // The superfinal state is denoted by kNoStateId. The distance from the
   // superfinal state to the final state is semiring One, so
   // `distance[kNoStateId]` is not needed.
   const ShortestPathCompare<StateId, Weight> compare(pairs, distance,
@@ -332,9 +359,9 @@ void NShortestPath(const Fst<RevArc> &ifst, MutableFst<Arc> *ofst,
   }
   ofst->SetStart(ofst->AddState());
   const auto final_state = ofst->AddState();
-  ofst->SetFinal(final_state, Weight::One());
+  ofst->SetFinal(final_state);
   while (pairs.size() <= final_state) {
-    pairs.push_back(std::make_pair(kNoStateId, Weight::Zero()));
+    pairs.emplace_back(kNoStateId, Weight::Zero());
   }
   pairs[final_state] = std::make_pair(ifst.Start(), Weight::One());
   std::vector<StateId> heap;
@@ -360,9 +387,7 @@ void NShortestPath(const Fst<RevArc> &ifst, MutableFst<Arc> *ofst,
     }
     while (r.size() <= p.first + 1) r.push_back(0);
     ++r[p.first + 1];
-    if (p.first == kNoStateId) {
-      ofst->AddArc(ofst->Start(), Arc(0, 0, Weight::One(), state));
-    }
+    if (p.first == kNoStateId) ofst->AddArc(ofst->Start(), Arc(0, 0, state));
     if ((p.first == kNoStateId) && (r[p.first + 1] == nshortest)) break;
     if (r[p.first + 1] > nshortest) continue;
     if (p.first == kNoStateId) continue;
@@ -372,9 +397,9 @@ void NShortestPath(const Fst<RevArc> &ifst, MutableFst<Arc> *ofst,
       Arc arc(rarc.ilabel, rarc.olabel, rarc.weight.Reverse(), rarc.nextstate);
       const auto weight = Times(p.second, arc.weight);
       const auto next = ofst->AddState();
-      pairs.push_back(std::make_pair(arc.nextstate, weight));
+      pairs.emplace_back(arc.nextstate, weight);
       arc.nextstate = state;
-      ofst->AddArc(next, arc);
+      ofst->AddArc(next, std::move(arc));
       heap.push_back(next);
       std::push_heap(heap.begin(), heap.end(), compare);
     }
@@ -382,7 +407,7 @@ void NShortestPath(const Fst<RevArc> &ifst, MutableFst<Arc> *ofst,
     if (final_weight != Weight::Zero()) {
       const auto weight = Times(p.second, final_weight);
       const auto next = ofst->AddState();
-      pairs.push_back(std::make_pair(kNoStateId, weight));
+      pairs.emplace_back(kNoStateId, weight);
       ofst->AddArc(next, Arc(0, 0, final_weight, state));
       heap.push_back(next);
       std::push_heap(heap.begin(), heap.end(), compare);
@@ -422,7 +447,9 @@ void NShortestPath(const Fst<RevArc> &ifst, MutableFst<Arc> *ofst,
 //
 // The algorithm relies on the shortest-distance algorithm. There are some
 // issues with the pseudo-code as written in the paper (viz., line 11).
-template <class Arc, class Queue, class ArcFilter>
+template <class Arc, class Queue, class ArcFilter,
+          typename std::enable_if<IsPath<typename Arc::Weight>::value>::type * =
+              nullptr>
 void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
                   std::vector<typename Arc::Weight> *distance,
                   const ShortestPathOptions<Arc, Queue, ArcFilter> &opts) {
@@ -441,17 +468,6 @@ void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
     return;
   }
   if (opts.nshortest <= 0) return;
-  // TODO(kbg): Make this a compile-time static_assert once:
-  // 1) All weight properties are made constexpr for all weight types.
-  // 2) We have a pleasant way to "deregister" this operation for non-path
-  //    semirings so an informative error message is produced. The best
-  //    solution will probably involve some kind of SFINAE magic.
-  if ((Weight::Properties() & (kPath | kSemiring)) != (kPath | kSemiring)) {
-    FSTERROR() << "ShortestPath: Weight needs to have the "
-               << "path property and be distributive: " << Weight::Type();
-    ofst->SetProperties(kError, kError);
-    return;
-  }
   if (!opts.has_distance) {
     ShortestDistance(ifst, distance, opts);
     if (distance->size() == 1 && !(*distance)[0].Member()) {
@@ -480,8 +496,8 @@ void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
                             opts.weight_threshold, opts.state_threshold);
   } else {
     std::vector<Weight> ddistance;
-    DeterminizeFstOptions<RevArc> dopts(opts.delta);
-    DeterminizeFst<RevArc> dfst(rfst, distance, &ddistance, dopts);
+    const DeterminizeFstOptions<RevArc> dopts(opts.delta);
+    const DeterminizeFst<RevArc> dfst(rfst, distance, &ddistance, dopts);
     internal::NShortestPath(dfst, ofst, ddistance, opts.nshortest, opts.delta,
                             opts.weight_threshold, opts.state_threshold);
   }
@@ -489,6 +505,17 @@ void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
   distance->erase(distance->begin());
 }
 
+template <class Arc, class Queue, class ArcFilter,
+          typename std::enable_if<!IsPath<typename Arc::Weight>::value>::type
+              * = nullptr>
+void ShortestPath(const Fst<Arc> &, MutableFst<Arc> *ofst,
+                  std::vector<typename Arc::Weight> *,
+                  const ShortestPathOptions<Arc, Queue, ArcFilter> &) {
+  FSTERROR() << "ShortestPath: Weight needs to have the "
+             << "path property and be distributive: " << Arc::Weight::Type();
+  ofst->SetProperties(kError, kError);
+}
+
 // Shortest-path algorithm: simplified interface. See above for a version that
 // allows finer control. The output mutable FST contains the n-shortest paths
 // in the input FST. The queue discipline is automatically selected. When unique
@@ -496,24 +523,25 @@ void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
 //
 // The n-shortest paths are the n-lowest weight paths w.r.t. the natural
 // semiring order. The single path that can be read from the ith of at most n
-// transitions leaving the initial state of the ouput FST is the ith best path.
+// transitions leaving the initial state of the output FST is the ith best path.
 // The weights need to be right distributive and have the path (kPath) property.
 template <class Arc>
 void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
                   int32 nshortest = 1, bool unique = false,
                   bool first_path = false,
                   typename Arc::Weight weight_threshold = Arc::Weight::Zero(),
-                  typename Arc::StateId state_threshold = kNoStateId) {
+                  typename Arc::StateId state_threshold = kNoStateId,
+                  float delta = kShortestDelta) {
   using StateId = typename Arc::StateId;
   std::vector<typename Arc::Weight> distance;
   AnyArcFilter<Arc> arc_filter;
   AutoQueue<StateId> state_queue(ifst, &distance, arc_filter);
   const ShortestPathOptions<Arc, AutoQueue<StateId>, AnyArcFilter<Arc>> opts(
-      &state_queue, arc_filter, nshortest, unique, false, kDelta, first_path,
+      &state_queue, arc_filter, nshortest, unique, false, delta, first_path,
       weight_threshold, state_threshold);
   ShortestPath(ifst, ofst, &distance, opts);
 }
 
 }  // namespace fst
 
-#endif  // FST_LIB_SHORTEST_PATH_H_
+#endif  // FST_SHORTEST_PATH_H_
