@@ -3,13 +3,14 @@
 //
 // Classes for representing the mapping between state tuples and state IDs.
 
-#ifndef FST_LIB_STATE_TABLE_H_
-#define FST_LIB_STATE_TABLE_H_
+#ifndef FST_STATE_TABLE_H_
+#define FST_STATE_TABLE_H_
 
 #include <deque>
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/bi-table.h>
@@ -106,7 +107,7 @@ class CompactHashStateTable
 // passed a fingerprint functor that should fingerprint tuples uniquely to an
 // integer that can used as a vector index. Normally, VectorStateTable
 // constructs the fingerprint functor. Alternately, the user can pass this
-// objert, in which case the table takes ownership.
+// object, in which case the table takes ownership.
 template <class T, class FP>
 class VectorStateTable : public VectorBiTable<typename T::StateId, T, FP> {
  public:
@@ -118,7 +119,7 @@ class VectorStateTable : public VectorBiTable<typename T::StateId, T, FP> {
   using VectorBiTable<StateId, StateTuple, FP>::Size;
   using VectorBiTable<StateId, StateTuple, FP>::Fingerprint;
 
-  explicit VectorStateTable(FP *fingerprint = nullptr, size_t table_size = 0)
+  explicit VectorStateTable(const FP &fingerprint = FP(), size_t table_size = 0)
       : VectorBiTable<StateId, StateTuple, FP>(fingerprint, table_size) {}
 
   StateId FindState(const StateTuple &tuple) { return FindId(tuple); }
@@ -145,8 +146,9 @@ class VectorHashStateTable
   using VectorHashBiTable<StateId, StateTuple, Select, FP, H>::Fingerprint;
   using VectorHashBiTable<StateId, StateTuple, Select, FP, H>::Hash;
 
-  VectorHashStateTable(Select *select, FP *fingerprint, H *hash,
-                       size_t vector_size = 0, size_t tuple_size = 0)
+  VectorHashStateTable(const Select &select, const FP &fingerprint,
+                       const H &hash, size_t vector_size = 0,
+                       size_t tuple_size = 0)
       : VectorHashBiTable<StateId, StateTuple, Select, FP, H>(
             select, fingerprint, hash, vector_size, tuple_size) {}
 
@@ -195,7 +197,7 @@ class ErasableStateTable : public ErasableBiTable<typename T::StateId, T, H> {
 //   // Looks up a tuple by state ID.
 //   const ComposeStateTuple<StateId> &Tuple(StateId s) const;
 //
-//   // The number of of stored tuples.
+//   // The number of stored tuples.
 //   StateId Size() const;
 //
 //   // Return true if error was encountered.
@@ -252,7 +254,9 @@ class DefaultComposeStateTuple {
   }
 
   size_t Hash() const {
-    return StateId1() + StateId2() * 7853 + GetFilterState().Hash() * 7867;
+    return static_cast<size_t>(StateId1()) +
+           static_cast<size_t>(StateId2()) * 7853u +
+           GetFilterState().Hash() * 7867u;
   }
 
  private:
@@ -260,7 +264,7 @@ class DefaultComposeStateTuple {
   FilterState fs_;  // State of composition filter.
 };
 
-// Specialization for TrivialFilterState that does not explicitely store the
+// Specialization for TrivialFilterState that does not explicitly store the
 // filter state since it is always the unique non-blocking state.
 template <typename S>
 class DefaultComposeStateTuple<S, TrivialFilterState> {
@@ -268,8 +272,7 @@ class DefaultComposeStateTuple<S, TrivialFilterState> {
   using StateId = S;
   using FilterState = TrivialFilterState;
 
-  DefaultComposeStateTuple()
-      : state_pair_(kNoStateId, kNoStateId) {}
+  DefaultComposeStateTuple() : state_pair_(kNoStateId, kNoStateId) {}
 
   DefaultComposeStateTuple(StateId s1, StateId s2, const FilterState &)
       : state_pair_(s1, s2) {}
@@ -287,7 +290,7 @@ class DefaultComposeStateTuple<S, TrivialFilterState> {
     return (&x == &y) || (x.state_pair_ == y.state_pair_);
   }
 
-  size_t Hash() const { return StateId1() + StateId2() * 7853; }
+  size_t Hash() const { return StateId1() + StateId2() * size_t{7853}; }
 
  private:
   std::pair<StateId, StateId> state_pair_;
@@ -338,7 +341,7 @@ class ComposeFingerprint {
   ComposeFingerprint(StateId nstates1, StateId nstates2)
       : mult1_(nstates1), mult2_(nstates1 * nstates2) {}
 
-  size_t operator()(const StateTuple &tuple) {
+  size_t operator()(const StateTuple &tuple) const {
     return tuple.StateId1() + tuple.StateId2() * mult1_ +
            tuple.GetFilterState().Hash() * mult2_;
   }
@@ -376,13 +379,13 @@ class ProductComposeStateTable
 
   ProductComposeStateTable(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
                            size_t table_size = 0)
-      : StateTable(new ComposeFingerprint<StateTuple>(CountStates(fst1),
-                                                      CountStates(fst2)),
+      : StateTable(ComposeFingerprint<StateTuple>(CountStates(fst1),
+                                                  CountStates(fst2)),
                    table_size) {}
 
   ProductComposeStateTable(
       const ProductComposeStateTable<Arc, StateTuple> &table)
-      : StateTable(new ComposeFingerprint<StateTuple>(table.Fingerprint())) {}
+      : StateTable(ComposeFingerprint<StateTuple>(table.Fingerprint())) {}
 
   constexpr bool Error() const { return false; }
 
@@ -392,7 +395,7 @@ class ProductComposeStateTable
 };
 
 // A vector-backed table over composition tuples which can be used when the
-// first FST is a string (i.e., satisfies kStringProperties) and the second is
+// first FST is a string (i.e., satisfies kString property) and the second is
 // deterministic and epsilon-free. It should be used with a composition filter
 // that creates at most one filter state per tuple under these conditions (e.g.,
 // SequenceComposeFilter or MatchComposeFilter).
@@ -489,4 +492,4 @@ class ErasableComposeStateTable
 
 }  // namespace fst
 
-#endif  // FST_LIB_STATE_TABLE_H_
+#endif  // FST_STATE_TABLE_H_
