@@ -3,15 +3,16 @@
 //
 // String weight set and associated semiring operation definitions.
 
-#ifndef FST_LIB_STRING_WEIGHT_H_
-#define FST_LIB_STRING_WEIGHT_H_
-
-#include <cstdlib>
+#ifndef FST_STRING_WEIGHT_H_
+#define FST_STRING_WEIGHT_H_
 
 #include <list>
+#include <random>
 #include <string>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/product-weight.h>
 #include <fst/union-weight.h>
 #include <fst/weight.h>
@@ -41,44 +42,43 @@ template <class>
 class StringWeightReverseIterator;
 
 // String semiring: (longest_common_prefix/suffix, ., Infinity, Epsilon)
-template <typename Label_, StringType S = STRING_LEFT>
+template <typename L, StringType S = STRING_LEFT>
 class StringWeight {
  public:
-  using Label = Label_;
-  using Self = StringWeight<Label, S>;
+  using Label = L;
   using ReverseWeight = StringWeight<Label, ReverseStringType(S)>;
-  using Iterator = StringWeightIterator<Self>;
-  using ReverseIterator = StringWeightReverseIterator<Self>;
+  using Iterator = StringWeightIterator<StringWeight>;
+  using ReverseIterator = StringWeightReverseIterator<StringWeight>;
 
-  friend class StringWeightIterator<Self>;
-  friend class StringWeightReverseIterator<Self>;
+  friend class StringWeightIterator<StringWeight>;
+  friend class StringWeightReverseIterator<StringWeight>;
 
   StringWeight() {}
 
   template <typename Iterator>
-  StringWeight(const Iterator &begin, const Iterator &end) {
+  StringWeight(const Iterator begin, const Iterator end) {
     for (auto iter = begin; iter != end; ++iter) PushBack(*iter);
   }
 
   explicit StringWeight(Label label) { PushBack(label); }
 
-  static const Self &Zero() {
-    static const Self *const zero = new Self(Label(kStringInfinity));
+  static const StringWeight &Zero() {
+    static const auto *const zero = new StringWeight(Label(kStringInfinity));
     return *zero;
   }
 
-  static const Self &One() {
-    static const Self *const one = new Self();
+  static const StringWeight &One() {
+    static const auto *const one = new StringWeight();
     return *one;
   }
 
-  static const Self &NoWeight() {
-    static const Self *const no_weight = new Self(Label(kStringBad));
+  static const StringWeight &NoWeight() {
+    static const auto *const no_weight = new StringWeight(Label(kStringBad));
     return *no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type = new string(
+  static const std::string &Type() {
+    static const std::string *const type = new std::string(
         S == STRING_LEFT
             ? "left_string"
             : (S == STRING_RIGHT ? "right_string" : "restricted_string"));
@@ -93,7 +93,7 @@ class StringWeight {
 
   size_t Hash() const;
 
-  Self Quantize(float delta = kDelta) const { return *this; }
+  StringWeight Quantize(float delta = kDelta) const { return *this; }
 
   ReverseWeight Reverse() const;
 
@@ -312,7 +312,7 @@ inline std::ostream &operator<<(std::ostream &strm,
 template <typename Label, StringType S>
 inline std::istream &operator>>(std::istream &strm,
                                 StringWeight<Label, S> &weight) {
-  string str;
+  std::string str;
   strm >> str;
   using Weight = StringWeight<Label, S>;
   if (str == "Infinity") {
@@ -335,8 +335,8 @@ inline std::istream &operator>>(std::istream &strm,
 }
 
 // Default is for the restricted string semiring. String equality is required
-// (for non-Zero() input. The restriction is used (e.g., in determinization) to
-// ensure the input is functional.
+// (for non-Zero() input). The restriction is used (e.g., in determinization)
+// to ensure the input is functional.
 template <typename Label, StringType S>
 inline StringWeight<Label, S> Plus(const StringWeight<Label, S> &w1,
                                    const StringWeight<Label, S> &w2) {
@@ -469,7 +469,7 @@ inline StringWeight<Label, STRING_LEFT> Divide(
     const StringWeight<Label, STRING_LEFT> &w2, DivideType divide_type) {
   if (divide_type != DIVIDE_LEFT) {
     FSTERROR() << "StringWeight::Divide: Only left division is defined "
-               << "for the left string semiring";
+               << "for the left std::string semiring";
     return StringWeight<Label, STRING_LEFT>::NoWeight();
   }
   return DivideLeft(w1, w2);
@@ -482,7 +482,7 @@ inline StringWeight<Label, STRING_RIGHT> Divide(
     const StringWeight<Label, STRING_RIGHT> &w2, DivideType divide_type) {
   if (divide_type != DIVIDE_RIGHT) {
     FSTERROR() << "StringWeight::Divide: Only right division is defined "
-               << "for the right string semiring";
+               << "for the right std::string semiring";
     return StringWeight<Label, STRING_RIGHT>::NoWeight();
   }
   return DivideRight(w1, w2);
@@ -496,29 +496,32 @@ class WeightGenerate<StringWeight<Label, S>> {
  public:
   using Weight = StringWeight<Label, S>;
 
-  explicit WeightGenerate(bool allow_zero = true,
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true,
                           size_t alphabet_size = kNumRandomWeights,
                           size_t max_string_length = kNumRandomWeights)
-      : allow_zero_(allow_zero),
+      : rand_(seed),
+        allow_zero_(allow_zero),
         alphabet_size_(alphabet_size),
         max_string_length_(max_string_length) {}
 
   Weight operator()() const {
-    size_t n = rand() % (max_string_length_ + allow_zero_);  // NOLINT
+    const int n = std::uniform_int_distribution<>(
+        0, max_string_length_ + allow_zero_)(rand_);
     if (allow_zero_ && n == max_string_length_) return Weight::Zero();
-    std::vector<Label> labels(n);
-    for (size_t i = 0; i < n; ++i) {
-      labels.push_back(rand() % alphabet_size_ + 1);  // NOLINT
+    std::vector<Label> labels;
+    labels.reserve(n);
+    for (int i = 0; i < n; ++i) {
+      labels.push_back(
+          std::uniform_int_distribution<>(1, alphabet_size_)(rand_));
     }
     return Weight(labels.begin(), labels.end());
   }
 
  private:
-  // Permits Zero() and zero divisors.
+  mutable std::mt19937_64 rand_;
   const bool allow_zero_;
-  // Alphabet size for random weights.
   const size_t alphabet_size_;
-  // Number of alternative random weights.
   const size_t max_string_length_;
 };
 
@@ -565,7 +568,7 @@ struct GallicWeight
 
   GallicWeight(SW w1, W w2) : ProductWeight<SW, W>(w1, w2) {}
 
-  explicit GallicWeight(const string &s, int *nread = nullptr)
+  explicit GallicWeight(const std::string &s, int *nread = nullptr)
       : ProductWeight<SW, W>(s, nread) {}
 
   explicit GallicWeight(const ProductWeight<SW, W> &w)
@@ -586,8 +589,8 @@ struct GallicWeight
     return no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type = new string(
+  static const std::string &Type() {
+    static const std::string *const type = new std::string(
         G == GALLIC_LEFT
             ? "left_gallic"
             : (G == GALLIC_RIGHT
@@ -650,7 +653,9 @@ class WeightGenerate<GallicWeight<Label, W, G>>
   using Generate = WeightGenerate<
       ProductWeight<StringWeight<Label, GallicStringType(G)>, W>>;
 
-  explicit WeightGenerate(bool allow_zero = true) : generate_(allow_zero) {}
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true)
+      : generate_(seed, allow_zero) {}
 
   Weight operator()() const { return Weight(generate_()); }
 
@@ -721,7 +726,7 @@ struct GallicWeight<Label, W, GALLIC>
 
   GallicWeight(SW w1, W w2) : UW(GW(w1, w2)) {}
 
-  explicit GallicWeight(const string &str, int *nread = nullptr)
+  explicit GallicWeight(const std::string &str, int *nread = nullptr)
       : UW(str, nread) {}
 
   static const GallicWeight<Label, W, GALLIC> &Zero() {
@@ -739,8 +744,8 @@ struct GallicWeight<Label, W, GALLIC>
     return no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type = new string("gallic");
+  static const std::string &Type() {
+    static const std::string *const type = new std::string("gallic");
     return *type;
   }
 
@@ -794,7 +799,9 @@ class WeightGenerate<GallicWeight<Label, W, GALLIC>>
       WeightGenerate<UnionWeight<GallicWeight<Label, W, GALLIC_RESTRICT>,
                                  GallicUnionWeightOptions<Label, W>>>;
 
-  explicit WeightGenerate(bool allow_zero = true) : generate_(allow_zero) {}
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true)
+      : generate_(seed, allow_zero) {}
 
   Weight operator()() const { return Weight(generate_()); }
 
@@ -804,4 +811,4 @@ class WeightGenerate<GallicWeight<Label, W, GALLIC>>
 
 }  // namespace fst
 
-#endif  // FST_LIB_STRING_WEIGHT_H_
+#endif  // FST_STRING_WEIGHT_H_
