@@ -3,8 +3,8 @@
 //
 // Topological sort of FSTs.
 
-#ifndef FST_LIB_TOPSORT_H_
-#define FST_LIB_TOPSORT_H_
+#ifndef FST_TOPSORT_H_
+#define FST_TOPSORT_H_
 
 #include <memory>
 #include <vector>
@@ -14,7 +14,6 @@
 #include <fst/fst.h>
 #include <fst/statesort.h>
 
-
 namespace fst {
 
 // DFS visitor class to return topological ordering.
@@ -30,7 +29,7 @@ class TopOrderVisitor {
       : order_(order), acyclic_(acyclic) {}
 
   void InitVisit(const Fst<Arc> &fst) {
-    finish_.reset(new std::vector<StateId>());
+    finish_ = fst::make_unique<std::vector<StateId>>();
     *acyclic_ = true;
   }
 
@@ -92,4 +91,4 @@ bool TopSort(MutableFst<Arc> *fst) {
 
 }  // namespace fst
 
-#endif  // FST_LIB_TOPSORT_H_
+#endif  // FST_TOPSORT_H_
