@@ -3,12 +3,16 @@
 //
 // Functions and classes to compute the union of two FSTs.
 
-#ifndef FST_LIB_UNION_H_
-#define FST_LIB_UNION_H_
+#ifndef FST_UNION_H_
+#define FST_UNION_H_
 
 #include <algorithm>
+#include <utility>
 #include <vector>
 
+#include <fst/types.h>
+
+#include <fst/expanded-fst.h>
 #include <fst/mutable-fst.h>
 #include <fst/rational.h>
 
@@ -29,9 +33,6 @@ namespace fst {
 // FST.
 template <class Arc>
 void Union(MutableFst<Arc> *fst1, const Fst<Arc> &fst2) {
-  using Label = typename Arc::Label;
-  using StateId = typename Arc::StateId;
-  using Weight = typename Arc::Weight;
   // Checks for symbol table compatibility.
   if (!CompatSymbols(fst1->InputSymbols(), fst2.InputSymbols()) ||
       !CompatSymbols(fst1->OutputSymbols(), fst2.OutputSymbols())) {
@@ -41,7 +42,8 @@ void Union(MutableFst<Arc> *fst1, const Fst<Arc> &fst2) {
     return;
   }
   const auto numstates1 = fst1->NumStates();
-  const bool initial_acyclic1 = fst1->Properties(kInitialAcyclic, true);
+  const bool initial_acyclic1 =
+      fst1->Properties(kInitialAcyclic, false) == kInitialAcyclic;
   const auto props1 = fst1->Properties(kFstProperties, false);
   const auto props2 = fst2.Properties(kFstProperties, false);
   const auto start2 = fst2.Start();
@@ -61,7 +63,7 @@ void Union(MutableFst<Arc> *fst1, const Fst<Arc> &fst2) {
     for (ArcIterator<Fst<Arc>> aiter(fst2, s2); !aiter.Done(); aiter.Next()) {
       auto arc = aiter.Value();  // Copy intended.
       arc.nextstate += numstates1;
-      fst1->AddArc(s1, arc);
+      fst1->AddArc(s1, std::move(arc));
     }
   }
   const auto start1 = fst1->Start();
@@ -71,16 +73,25 @@ void Union(MutableFst<Arc> *fst1, const Fst<Arc> &fst2) {
     return;
   }
   if (initial_acyclic1) {
-    fst1->AddArc(start1, Arc(0, 0, Weight::One(), start2 + numstates1));
+    fst1->AddArc(start1, Arc(0, 0, start2 + numstates1));
   } else {
     const auto nstart1 = fst1->AddState();
     fst1->SetStart(nstart1);
-    fst1->AddArc(nstart1, Arc(0, 0, Weight::One(), start1));
-    fst1->AddArc(nstart1, Arc(0, 0, Weight::One(), start2 + numstates1));
+    fst1->AddArc(nstart1, Arc(0, 0, start1));
+    fst1->AddArc(nstart1, Arc(0, 0, start2 + numstates1));
   }
   fst1->SetProperties(UnionProperties(props1, props2), kFstProperties);
 }
 
+// Same as the above but can handle arbitrarily many right-hand-side FSTs,
+// preallocating the states.
+template <class Arc>
+void Union(MutableFst<Arc> *fst1, const std::vector<const Fst<Arc> *> &fsts2) {
+  // We add 1 just in case fst1 has an initial cycle.
+  fst1->ReserveStates(1 + fst1->NumStates() + CountStates(fsts2));
+  for (const auto *fst2 : fsts2) Union(fst1, *fst2);
+}
+
 // Computes the union of two FSTs, modifying the RationalFst argument.
 template <class Arc>
 void Union(RationalFst<Arc> *fst1, const Fst<Arc> &fst2) {
@@ -120,12 +131,12 @@ class UnionFst : public RationalFst<A> {
   }
 
   // See Fst<>::Copy() for doc.
-  UnionFst(const UnionFst<Arc> &fst, bool safe = false)
+  UnionFst(const UnionFst &fst, bool safe = false)
       : RationalFst<Arc>(fst, safe) {}
 
   // Gets a copy of this UnionFst. See Fst<>::Copy() for further doc.
-  UnionFst<Arc> *Copy(bool safe = false) const override {
-    return new UnionFst<Arc>(*this, safe);
+  UnionFst *Copy(bool safe = false) const override {
+    return new UnionFst(*this, safe);
   }
 
  private:
@@ -155,4 +166,4 @@ using StdUnionFst = UnionFst<StdArc>;
 
 }  // namespace fst
 
-#endif  // FST_LIB_UNION_H_
+#endif  // FST_UNION_H_
