@@ -3,8 +3,8 @@
 //
 // Queue-dependent visitation of finite-state transducers. See also dfs-visit.h.
 
-#ifndef FST_LIB_VISIT_H_
-#define FST_LIB_VISIT_H_
+#ifndef FST_VISIT_H_
+#define FST_VISIT_H_
 
 
 #include <fst/arcfilter.h>
@@ -272,8 +272,10 @@ class PartialCopyVisitor : public CopyVisitor<A> {
 
   PartialCopyVisitor(MutableFst<Arc> *ofst, StateId maxvisit,
                      bool copy_grey = true, bool copy_black = true)
-      : CopyVisitor<A>(ofst), maxvisit_(maxvisit),
-        copy_grey_(copy_grey), copy_black_(copy_black) {}
+      : CopyVisitor<A>(ofst),
+        maxvisit_(maxvisit),
+        copy_grey_(copy_grey),
+        copy_black_(copy_black) {}
 
   void InitVisit(const Fst<A> &ifst) {
     CopyVisitor<A>::InitVisit(ifst);
@@ -318,4 +320,4 @@ class PartialCopyVisitor : public CopyVisitor<A> {
 
 }  // namespace fst
 
-#endif  // FST_LIB_VISIT_H_
+#endif  // FST_VISIT_H_
