@@ -10,7 +10,7 @@ namespace fst {
 
 SymbolTable *MergeSymbolTable(const SymbolTable &left, const SymbolTable &right,
                               bool *right_relabel_output) {
-  // MergeSymbolTable detects several special cases.  It will return a reference
+  // MergeSymbolTable detects several special cases  It will return a reference
   // copied version of SymbolTable of left or right if either symbol table is
   // a superset of the other.
   std::unique_ptr<SymbolTable> merged(
@@ -19,13 +19,13 @@ SymbolTable *MergeSymbolTable(const SymbolTable &left, const SymbolTable &right,
   bool left_has_all = true;
   bool right_has_all = true;
   bool relabel = false;
-  for (SymbolTableIterator liter(left); !liter.Done(); liter.Next()) {
-    merged->AddSymbol(liter.Symbol(), liter.Value());
+  for (const auto &litem : left) {
+    merged->AddSymbol(litem.Symbol(), litem.Label());
     if (right_has_all) {
-      int64 key = right.Find(liter.Symbol());
+      int64 key = right.Find(litem.Symbol());
       if (key == -1) {
         right_has_all = false;
-      } else if (!relabel && key != liter.Value()) {
+      } else if (!relabel && key != litem.Label()) {
         relabel = true;
       }
     }
@@ -34,37 +34,36 @@ SymbolTable *MergeSymbolTable(const SymbolTable &left, const SymbolTable &right,
     if (right_relabel_output) *right_relabel_output = relabel;
     return right.Copy();
   }
-  // add all symbols we can from right symbol table
-  std::vector<string> conflicts;
-  for (SymbolTableIterator riter(right); !riter.Done(); riter.Next()) {
-    int64 key = merged->Find(riter.Symbol());
+  // Adds all symbols we can from right symbol table.
+  std::vector<std::string> conflicts;
+  for (const auto &ritem : right) {
+    int64 key = merged->Find(ritem.Symbol());
     if (key != -1) {
-      // Symbol already exists, maybe with different value
-      if (key != riter.Value()) relabel = true;
+      // Symbol already exists, maybe with different value.
+      if (key != ritem.Label()) relabel = true;
       continue;
     }
-    // Symbol doesn't exist from left
+    // Symbol doesn't exist from left.
     left_has_all = false;
-    if (!merged->Find(riter.Value()).empty()) {
-      // we can't add this where we want to, add it later, in order
-      conflicts.push_back(riter.Symbol());
+    if (!merged->Find(ritem.Label()).empty()) {
+      // We can't add this where we want to, add it later, in order.
+      conflicts.push_back(ritem.Symbol());
       continue;
     }
-    // there is a hole and we can add this symbol with its id
-    merged->AddSymbol(riter.Symbol(), riter.Value());
+    // There is a hole and we can add this symbol with its ID.
+    merged->AddSymbol(ritem.Symbol(), ritem.Label());
   }
   if (right_relabel_output) *right_relabel_output = relabel;
   if (left_has_all) return left.Copy();
-  // Add all symbols that conflicted, in order
+  // Adds all symbols that conflicted, in order.
   for (const auto &conflict : conflicts) merged->AddSymbol(conflict);
   return merged.release();
 }
 
 SymbolTable *CompactSymbolTable(const SymbolTable &syms) {
-  std::map<int64, string> sorted;
-  SymbolTableIterator stiter(syms);
-  for (; !stiter.Done(); stiter.Next()) {
-    sorted[stiter.Value()] = stiter.Symbol();
+  std::map<int64, std::string> sorted;
+  for (const auto &stitem : syms) {
+    sorted[stitem.Label()] = stitem.Symbol();
   }
   auto *compact = new SymbolTable(syms.Name() + "_compact");
   int64 newkey = 0;
@@ -72,41 +71,41 @@ SymbolTable *CompactSymbolTable(const SymbolTable &syms) {
   return compact;
 }
 
-SymbolTable *FstReadSymbols(const string &filename, bool input_symbols) {
-  std::ifstream in(filename, std::ios_base::in | std::ios_base::binary);
+SymbolTable *FstReadSymbols(const std::string &source, bool input_symbols) {
+  std::ifstream in(source, std::ios_base::in | std::ios_base::binary);
   if (!in) {
-    LOG(ERROR) << "FstReadSymbols: Can't open file " << filename;
+    LOG(ERROR) << "FstReadSymbols: Can't open file " << source;
     return nullptr;
   }
   FstHeader hdr;
-  if (!hdr.Read(in, filename)) {
-    LOG(ERROR) << "FstReadSymbols: Couldn't read header from " << filename;
+  if (!hdr.Read(in, source)) {
+    LOG(ERROR) << "FstReadSymbols: Couldn't read header from " << source;
     return nullptr;
   }
   if (hdr.GetFlags() & FstHeader::HAS_ISYMBOLS) {
-    std::unique_ptr<SymbolTable> isymbols(SymbolTable::Read(in, filename));
+    std::unique_ptr<SymbolTable> isymbols(SymbolTable::Read(in, source));
     if (isymbols == nullptr) {
       LOG(ERROR) << "FstReadSymbols: Couldn't read input symbols from "
-                 << filename;
+                 << source;
       return nullptr;
     }
     if (input_symbols) return isymbols.release();
   }
   if (hdr.GetFlags() & FstHeader::HAS_OSYMBOLS) {
-    std::unique_ptr<SymbolTable> osymbols(SymbolTable::Read(in, filename));
+    std::unique_ptr<SymbolTable> osymbols(SymbolTable::Read(in, source));
     if (osymbols == nullptr) {
       LOG(ERROR) << "FstReadSymbols: Couldn't read output symbols from "
-                 << filename;
+                 << source;
       return nullptr;
     }
     if (!input_symbols) return osymbols.release();
   }
-  LOG(ERROR) << "FstReadSymbols: The file " << filename
+  LOG(ERROR) << "FstReadSymbols: The file " << source
              << " doesn't contain the requested symbols";
   return nullptr;
 }
 
-bool AddAuxiliarySymbols(const string &prefix, int64 start_label,
+bool AddAuxiliarySymbols(const std::string &prefix, int64 start_label,
                          int64 nlabels, SymbolTable *syms) {
   for (int64 i = 0; i < nlabels; ++i) {
     auto index = i + start_label;
