@@ -5,6 +5,8 @@
 
 #include <fst/symbol-table.h>
 
+#include <fst/flags.h>
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fstream>
@@ -17,50 +19,137 @@ DEFINE_string(fst_field_separator, "\t ",
 
 namespace fst {
 
-// Maximum line length in textual symbols file.
-static constexpr int kLineLen = 8096;
-
-// Identifies stream data as a symbol table (and its endianity).
-static constexpr int32 kSymbolTableMagicNumber = 2125658996;
-
 SymbolTableTextOptions::SymbolTableTextOptions(bool allow_negative_labels)
     : allow_negative_labels(allow_negative_labels),
       fst_field_separator(FLAGS_fst_field_separator) {}
 
 namespace internal {
 
+// Maximum line length in textual symbols file.
+const int kLineLen = 8096;
+
+// Identifies stream data as a symbol table (and its endianity).
+static constexpr int32 kSymbolTableMagicNumber = 2125658996;
+
+constexpr int64 DenseSymbolMap::kEmptyBucket;
+
+DenseSymbolMap::DenseSymbolMap()
+    : str_hash_(),
+      buckets_(1 << 4, kEmptyBucket),
+      hash_mask_(buckets_.size() - 1) {}
+
+std::pair<int64, bool> DenseSymbolMap::InsertOrFind(KeyType key) {
+  static constexpr float kMaxOccupancyRatio = 0.75;  // Grows when 75% full.
+  if (Size() >= kMaxOccupancyRatio * buckets_.size()) {
+    Rehash(buckets_.size() * 2);
+  }
+  size_t idx = GetHash(key);
+  while (buckets_[idx] != kEmptyBucket) {
+    const auto stored_value = buckets_[idx];
+    if (symbols_[stored_value] == key) return {stored_value, false};
+    idx = (idx + 1) & hash_mask_;
+  }
+  const auto next = Size();
+  buckets_[idx] = next;
+  symbols_.emplace_back(key);
+  return {next, true};
+}
+
+int64 DenseSymbolMap::Find(KeyType key) const {
+  size_t idx = str_hash_(key) & hash_mask_;
+  while (buckets_[idx] != kEmptyBucket) {
+    const auto stored_value = buckets_[idx];
+    if (symbols_[stored_value] == key) return stored_value;
+    idx = (idx + 1) & hash_mask_;
+  }
+  return buckets_[idx];
+}
+
+void DenseSymbolMap::Rehash(size_t num_buckets) {
+  buckets_.resize(num_buckets);
+  hash_mask_ = buckets_.size() - 1;
+  std::fill(buckets_.begin(), buckets_.end(), kEmptyBucket);
+  for (size_t i = 0; i < Size(); ++i) {
+    size_t idx = GetHash(symbols_[i]);
+    while (buckets_[idx] != kEmptyBucket) {
+      idx = (idx + 1) & hash_mask_;
+    }
+    buckets_[idx] = i;
+  }
+}
+
+void DenseSymbolMap::RemoveSymbol(size_t idx) {
+  symbols_.erase(symbols_.begin() + idx);
+  Rehash(buckets_.size());
+}
+
+void DenseSymbolMap::ShrinkToFit() { symbols_.shrink_to_fit(); }
+
+void MutableSymbolTableImpl::AddTable(const SymbolTable &table) {
+  for (const auto &item : table) {
+    AddSymbol(item.Symbol());
+  }
+}
+
+std::unique_ptr<SymbolTableImplBase> ConstSymbolTableImpl::Copy() const {
+  LOG(FATAL) << "ConstSymbolTableImpl can't be copied";
+  return nullptr;
+}
+
+int64 ConstSymbolTableImpl::AddSymbol(SymbolType symbol, int64 key) {
+  LOG(FATAL) << "ConstSymbolTableImpl does not support AddSymbol";
+  return kNoSymbol;
+}
+
+int64 ConstSymbolTableImpl::AddSymbol(SymbolType symbol) {
+  return AddSymbol(symbol, kNoSymbol);
+}
+
+void ConstSymbolTableImpl::RemoveSymbol(int64 key) {
+  LOG(FATAL) << "ConstSymbolTableImpl does not support RemoveSymbol";
+}
+
+void ConstSymbolTableImpl::SetName(const std::string &new_name) {
+  LOG(FATAL) << "ConstSymbolTableImpl does not support SetName";
+}
+
+void ConstSymbolTableImpl::AddTable(const SymbolTable &table) {
+  LOG(FATAL) << "ConstSymbolTableImpl does not support AddTable";
+}
+
 SymbolTableImpl *SymbolTableImpl::ReadText(std::istream &strm,
-                                           const string &filename,
+                                           const std::string &source,
                                            const SymbolTableTextOptions &opts) {
-  std::unique_ptr<SymbolTableImpl> impl(new SymbolTableImpl(filename));
+  std::unique_ptr<SymbolTableImpl> impl(new SymbolTableImpl(source));
   int64 nline = 0;
   char line[kLineLen];
+  const auto separator = opts.fst_field_separator + "\n";
   while (!strm.getline(line, kLineLen).fail()) {
     ++nline;
     std::vector<char *> col;
-    auto separator = opts.fst_field_separator + "\n";
-    SplitToVector(line, separator.c_str(), &col, true);
+    SplitString(line, separator.c_str(), &col, true);
     if (col.empty()) continue;  // Empty line.
     if (col.size() != 2) {
       LOG(ERROR) << "SymbolTable::ReadText: Bad number of columns ("
                  << col.size() << "), "
-                 << "file = " << filename << ", line = " << nline << ":<"
-                 << line << ">";
+                 << "file = " << source << ", line = " << nline << ":<" << line
+                 << ">";
       return nullptr;
     }
     const char *symbol = col[0];
     const char *value = col[1];
     char *p;
     const auto key = strtoll(value, &p, 10);
-    if (p < value + strlen(value) ||
-       (!opts.allow_negative_labels && key < 0) || key == -1) {
+    if (*p != '\0' || (!opts.allow_negative_labels && key < 0) ||
+        key == kNoSymbol) {
       LOG(ERROR) << "SymbolTable::ReadText: Bad non-negative integer \""
                  << value << "\", "
-                 << "file = " << filename << ", line = " << nline;
+                 << "file = " << source << ", line = " << nline;
       return nullptr;
     }
     impl->AddSymbol(symbol, key);
   }
+  impl->ShrinkToFit();
   return impl.release();
 }
 
@@ -76,7 +165,7 @@ void SymbolTableImpl::MaybeRecomputeCheckSum() const {
   }
   // Calculates the original label-agnostic checksum.
   CheckSummer check_sum;
-  for (size_t i = 0; i < symbols_.size(); ++i) {
+  for (size_t i = 0; i < symbols_.Size(); ++i) {
     const auto &symbol = symbols_.GetSymbol(i);
     check_sum.Update(symbol.data(), symbol.size());
     check_sum.Update("", 1);
@@ -89,7 +178,7 @@ void SymbolTableImpl::MaybeRecomputeCheckSum() const {
     line << symbols_.GetSymbol(i) << '\t' << i;
     labeled_check_sum.Update(line.str().data(), line.str().size());
   }
-  using citer = map<int64, int64>::const_iterator;
+  using citer = std::map<int64, int64>::const_iterator;
   for (citer it = key_map_.begin(); it != key_map_.end(); ++it) {
     // TODO(tombagby, 2013-11-22) This line maintains a bug that ignores
     // negative labels in the checksum that too many tests rely on.
@@ -102,22 +191,34 @@ void SymbolTableImpl::MaybeRecomputeCheckSum() const {
   check_sum_finalized_ = true;
 }
 
-int64 SymbolTableImpl::AddSymbol(const string &symbol, int64 key) {
-  if (key == -1) return key;
-  const std::pair<int64, bool> &insert_key = symbols_.InsertOrFind(symbol);
+std::string SymbolTableImpl::Find(int64 key) const {
+  int64 idx = key;
+  if (key < 0 || key >= dense_key_limit_) {
+    const auto it = key_map_.find(key);
+    if (it == key_map_.end()) return "";
+    idx = it->second;
+  }
+  if (idx < 0 || idx >= symbols_.Size()) return "";
+  return symbols_.GetSymbol(idx);
+}
+
+int64 SymbolTableImpl::AddSymbol(SymbolType symbol, int64 key) {
+  if (key == kNoSymbol) return key;
+  const auto insert_key = symbols_.InsertOrFind(symbol);
   if (!insert_key.second) {
-    auto key_already = GetNthKey(insert_key.first);
+    const auto key_already = GetNthKey(insert_key.first);
     if (key_already == key) return key;
     VLOG(1) << "SymbolTable::AddSymbol: symbol = " << symbol
             << " already in symbol_map_ with key = " << key_already
             << " but supplied new key = " << key << " (ignoring new key)";
     return key_already;
   }
-  if (key == (symbols_.size() - 1) && key == dense_key_limit_) {
+  if (key + 1 == static_cast<int64>(symbols_.Size()) &&
+      key == dense_key_limit_) {
     ++dense_key_limit_;
   } else {
     idx_key_.push_back(key);
-    key_map_[key] = symbols_.size() - 1;
+    key_map_[key] = symbols_.Size() - 1;
   }
   if (key >= available_key_) available_key_ = key + 1;
   check_sum_finalized_ = false;
@@ -134,7 +235,7 @@ void SymbolTableImpl::RemoveSymbol(const int64 key) {
     idx = iter->second;
     key_map_.erase(iter);
   }
-  if (idx < 0 || idx >= symbols_.size()) return;
+  if (idx < 0 || idx >= static_cast<int64>(symbols_.Size())) return;
   symbols_.RemoveSymbol(idx);
   // Removed one symbol, all indexes > idx are shifted by -1.
   for (auto &k : key_map_) {
@@ -147,8 +248,8 @@ void SymbolTableImpl::RemoveSymbol(const int64 key) {
       key_map_[i] = i - 1;
     }
     // Moves existing values in idx_key to new place.
-    idx_key_.resize(symbols_.size() - new_dense_key_limit);
-    for (int64 i = symbols_.size(); i >= dense_key_limit_; --i) {
+    idx_key_.resize(symbols_.Size() - new_dense_key_limit);
+    for (int64 i = symbols_.Size(); i >= dense_key_limit_; --i) {
       idx_key_[i - new_dense_key_limit - 1] = idx_key_[i - dense_key_limit_];
     }
     // Adds indexes for previously dense keys.
@@ -158,7 +259,7 @@ void SymbolTableImpl::RemoveSymbol(const int64 key) {
     dense_key_limit_ = new_dense_key_limit;
   } else {
     // Remove entry for removed index in idx_key.
-    for (int64 i = idx - dense_key_limit_; i < idx_key_.size() - 1; ++i) {
+    for (size_t i = idx - dense_key_limit_; i + 1 < idx_key_.size(); ++i) {
       idx_key_[i] = idx_key_[i + 1];
     }
     idx_key_.pop_back();
@@ -167,14 +268,14 @@ void SymbolTableImpl::RemoveSymbol(const int64 key) {
 }
 
 SymbolTableImpl *SymbolTableImpl::Read(std::istream &strm,
-                                       const SymbolTableReadOptions &opts) {
+                                       const SymbolTableReadOptions &) {
   int32 magic_number = 0;
   ReadType(strm, &magic_number);
   if (strm.fail()) {
     LOG(ERROR) << "SymbolTable::Read: Read failed";
     return nullptr;
   }
-  string name;
+  std::string name;
   ReadType(strm, &name);
   std::unique_ptr<SymbolTableImpl> impl(new SymbolTableImpl(name));
   ReadType(strm, &impl->available_key_);
@@ -184,7 +285,7 @@ SymbolTableImpl *SymbolTableImpl::Read(std::istream &strm,
     LOG(ERROR) << "SymbolTable::Read: Read failed";
     return nullptr;
   }
-  string symbol;
+  std::string symbol;
   int64 key;
   impl->check_sum_finalized_ = false;
   for (int64 i = 0; i < size; ++i) {
@@ -196,6 +297,7 @@ SymbolTableImpl *SymbolTableImpl::Read(std::istream &strm,
     }
     impl->AddSymbol(symbol, key);
   }
+  impl->ShrinkToFit();
   return impl.release();
 }
 
@@ -203,12 +305,15 @@ bool SymbolTableImpl::Write(std::ostream &strm) const {
   WriteType(strm, kSymbolTableMagicNumber);
   WriteType(strm, name_);
   WriteType(strm, available_key_);
-  int64 size = symbols_.size();
+  const int64 size = symbols_.Size();
   WriteType(strm, size);
-  for (int64 i = 0; i < size; ++i) {
-    auto key = (i < dense_key_limit_) ? i : idx_key_[i - dense_key_limit_];
+  for (int64 i = 0; i < dense_key_limit_; ++i) {
     WriteType(strm, symbols_.GetSymbol(i));
-    WriteType(strm, key);
+    WriteType(strm, i);
+  }
+  for (const auto &p : key_map_) {
+    WriteType(strm, symbols_.GetSymbol(p.second));
+    WriteType(strm, p.first);
   }
   strm.flush();
   if (strm.fail()) {
@@ -218,14 +323,35 @@ bool SymbolTableImpl::Write(std::ostream &strm) const {
   return true;
 }
 
+void SymbolTableImpl::ShrinkToFit() { symbols_.ShrinkToFit(); }
+
 }  // namespace internal
 
-constexpr int64 SymbolTable::kNoSymbol;
+SymbolTable *SymbolTable::ReadText(const std::string &source,
+                                   const SymbolTableTextOptions &opts) {
+  std::ifstream strm(source, std::ios_base::in);
+  if (!strm.good()) {
+    LOG(ERROR) << "SymbolTable::ReadText: Can't open file: " << source;
+    return nullptr;
+  }
+  return ReadText(strm, source, opts);
+}
 
-void SymbolTable::AddTable(const SymbolTable &table) {
-  MutateCheck();
-  for (SymbolTableIterator iter(table); !iter.Done(); iter.Next()) {
-    impl_->AddSymbol(iter.Symbol());
+bool SymbolTable::Write(const std::string &source) const {
+  if (!source.empty()) {
+    std::ofstream strm(source,
+                             std::ios_base::out | std::ios_base::binary);
+    if (!strm) {
+      LOG(ERROR) << "SymbolTable::Write: Can't open file: " << source;
+      return false;
+    }
+    if (!Write(strm)) {
+      LOG(ERROR) << "SymbolTable::Write: Write failed: " << source;
+      return false;
+    }
+    return true;
+  } else {
+    return Write(std::cout);
   }
 }
 
@@ -236,102 +362,36 @@ bool SymbolTable::WriteText(std::ostream &strm,
     return false;
   }
   bool once_only = false;
-  for (SymbolTableIterator iter(*this); !iter.Done(); iter.Next()) {
+  for (const auto &item : *this) {
     std::ostringstream line;
-    if (iter.Value() < 0 && !opts.allow_negative_labels && !once_only) {
+    if (item.Label() < 0 && !opts.allow_negative_labels && !once_only) {
       LOG(WARNING) << "Negative symbol table entry when not allowed";
       once_only = true;
     }
-    line << iter.Symbol() << opts.fst_field_separator[0] << iter.Value()
+    line << item.Symbol() << opts.fst_field_separator[0] << item.Label()
          << '\n';
     strm.write(line.str().data(), line.str().length());
   }
   return true;
 }
 
-namespace internal {
-
-DenseSymbolMap::DenseSymbolMap()
-    : empty_(-1), buckets_(1 << 4), hash_mask_(buckets_.size() - 1) {
-  std::uninitialized_fill(buckets_.begin(), buckets_.end(), empty_);
-}
-
-DenseSymbolMap::DenseSymbolMap(const DenseSymbolMap &x)
-    : empty_(-1),
-      symbols_(x.symbols_.size()),
-      buckets_(x.buckets_),
-      hash_mask_(x.hash_mask_) {
-  for (size_t i = 0; i < symbols_.size(); ++i) {
-    const auto sz = strlen(x.symbols_[i]) + 1;
-    auto *cpy = new char[sz];
-    memcpy(cpy, x.symbols_[i], sz);
-    symbols_[i] = cpy;
-  }
-}
-
-DenseSymbolMap::~DenseSymbolMap() {
-  for (size_t i = 0; i < symbols_.size(); ++i) delete[] symbols_[i];
-}
-
-std::pair<int64, bool> DenseSymbolMap::InsertOrFind(const string &key) {
-  static constexpr float kMaxOccupancyRatio = 0.75;  // Grows when 75% full.
-  if (symbols_.size() >= kMaxOccupancyRatio * buckets_.size()) {
-    Rehash(buckets_.size() * 2);
-  }
-  size_t idx = str_hash_(key) & hash_mask_;
-  while (buckets_[idx] != empty_) {
-    const auto stored_value = buckets_[idx];
-    if (!strcmp(symbols_[stored_value], key.c_str())) {
-      return std::make_pair(stored_value, false);
+bool SymbolTable::WriteText(const std::string &source) const {
+  if (!source.empty()) {
+    std::ofstream strm(source);
+    if (!strm) {
+      LOG(ERROR) << "SymbolTable::WriteText: Can't open file: " << source;
+      return false;
     }
-    idx = (idx + 1) & hash_mask_;
-  }
-  auto next = symbols_.size();
-  buckets_[idx] = next;
-  symbols_.push_back(NewSymbol(key));
-  return std::make_pair(next, true);
-}
-
-int64 DenseSymbolMap::Find(const string &key) const {
-  size_t idx = str_hash_(key) & hash_mask_;
-  while (buckets_[idx] != empty_) {
-    const auto stored_value = buckets_[idx];
-    if (!strcmp(symbols_[stored_value], key.c_str())) {
-      return stored_value;
+    if (!WriteText(strm, SymbolTableTextOptions())) {
+      LOG(ERROR) << "SymbolTable::WriteText: Write failed: " << source;
+      return false;
     }
-    idx = (idx + 1) & hash_mask_;
-  }
-  return buckets_[idx];
-}
-
-void DenseSymbolMap::Rehash(size_t num_buckets) {
-  buckets_.resize(num_buckets);
-  hash_mask_ = buckets_.size() - 1;
-  std::uninitialized_fill(buckets_.begin(), buckets_.end(), empty_);
-  for (size_t i = 0; i < symbols_.size(); ++i) {
-    size_t idx = str_hash_(string(symbols_[i])) & hash_mask_;
-    while (buckets_[idx] != empty_) {
-      idx = (idx + 1) & hash_mask_;
-    }
-    buckets_[idx] = i;
+    return true;
+  } else {
+    return WriteText(std::cout, SymbolTableTextOptions());
   }
 }
 
-const char *DenseSymbolMap::NewSymbol(const string &sym) {
-  auto num = sym.size() + 1;
-  auto newstr = new char[num];
-  memcpy(newstr, sym.c_str(), num);
-  return newstr;
-}
-
-void DenseSymbolMap::RemoveSymbol(size_t idx) {
-  delete[] symbols_[idx];
-  symbols_.erase(symbols_.begin() + idx);
-  Rehash(buckets_.size());
-}
-
-}  // namespace internal
-
 bool CompatSymbols(const SymbolTable *syms1, const SymbolTable *syms2,
                    bool warning) {
   // Flag can explicitly override this check.
@@ -349,14 +409,15 @@ bool CompatSymbols(const SymbolTable *syms1, const SymbolTable *syms2,
   }
 }
 
-void SymbolTableToString(const SymbolTable *table, string *result) {
+void SymbolTableToString(const SymbolTable *table, std::string *result) {
   std::ostringstream ostrm;
   table->Write(ostrm);
   *result = ostrm.str();
 }
 
-SymbolTable *StringToSymbolTable(const string &str) {
+SymbolTable *StringToSymbolTable(const std::string &str) {
   std::istringstream istrm(str);
   return SymbolTable::Read(istrm, SymbolTableReadOptions());
 }
+
 }  // namespace fst
