@@ -1,31 +1,28 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#include <istream>
-#include <string>
-
 #include <fst/script/compile.h>
-#include <fst/script/fst-class.h>
+
 #include <fst/script/script-impl.h>
 
 namespace fst {
 namespace script {
 
-// 1
-void CompileFst(std::istream &istrm, const string &source, const string &dest,
-                const string &fst_type, const string &arc_type,
-                const SymbolTable *isyms, const SymbolTable *osyms,
-                const SymbolTable *ssyms, bool accep, bool ikeep, bool okeep,
-                bool nkeep, bool allow_negative_labels) {
+void CompileFst(std::istream &istrm, const std::string &source,
+                const std::string &dest, const std::string &fst_type,
+                const std::string &arc_type, const SymbolTable *isyms,
+                const SymbolTable *osyms, const SymbolTable *ssyms, bool accep,
+                bool ikeep, bool okeep, bool nkeep,
+                bool allow_negative_labels) {
   std::unique_ptr<FstClass> fst(
       CompileFstInternal(istrm, source, fst_type, arc_type, isyms, osyms, ssyms,
                          accep, ikeep, okeep, nkeep, allow_negative_labels));
   fst->Write(dest);
 }
 
-// 2
-FstClass *CompileFstInternal(std::istream &istrm, const string &source,
-                             const string &fst_type, const string &arc_type,
+FstClass *CompileFstInternal(std::istream &istrm, const std::string &source,
+                             const std::string &fst_type,
+                             const std::string &arc_type,
                              const SymbolTable *isyms, const SymbolTable *osyms,
                              const SymbolTable *ssyms, bool accep, bool ikeep,
                              bool okeep, bool nkeep,
@@ -37,10 +34,8 @@ FstClass *CompileFstInternal(std::istream &istrm, const string &source,
   return args.retval;
 }
 
-// This registers 2; 1 does not require registration.
-REGISTER_FST_OPERATION(CompileFstInternal, StdArc, CompileFstArgs);
-REGISTER_FST_OPERATION(CompileFstInternal, LogArc, CompileFstArgs);
-REGISTER_FST_OPERATION(CompileFstInternal, Log64Arc, CompileFstArgs);
+// This registers form 2; 1 does not require registration.
+REGISTER_FST_OPERATION_3ARCS(CompileFstInternal, CompileFstArgs);
 
 }  // namespace script
 }  // namespace fst
