@@ -1,8 +1,8 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#include <fst/script/fst-class.h>
 #include <fst/script/compose.h>
+
 #include <fst/script/script-impl.h>
 
 namespace fst {
@@ -19,9 +19,7 @@ void Compose(const FstClass &ifst1, const FstClass &ifst2,
   Apply<Operation<ComposeArgs>>("Compose", ifst1.ArcType(), &args);
 }
 
-REGISTER_FST_OPERATION(Compose, StdArc, ComposeArgs);
-REGISTER_FST_OPERATION(Compose, LogArc, ComposeArgs);
-REGISTER_FST_OPERATION(Compose, Log64Arc, ComposeArgs);
+REGISTER_FST_OPERATION_3ARCS(Compose, ComposeArgs);
 
 }  // namespace script
 }  // namespace fst
