@@ -1,8 +1,8 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#include <fst/script/fst-class.h>
 #include <fst/script/equal.h>
+
 #include <fst/script/script-impl.h>
 
 namespace fst {
@@ -16,9 +16,7 @@ bool Equal(const FstClass &fst1, const FstClass &fst2, float delta) {
   return args.retval;
 }
 
-REGISTER_FST_OPERATION(Equal, StdArc, EqualArgs);
-REGISTER_FST_OPERATION(Equal, LogArc, EqualArgs);
-REGISTER_FST_OPERATION(Equal, Log64Arc, EqualArgs);
+REGISTER_FST_OPERATION_3ARCS(Equal, EqualArgs);
 
 }  // namespace script
 }  // namespace fst
