@@ -1,15 +1,15 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#include <fst/script/fst-class.h>
-#include <fst/script/script-impl.h>
 #include <fst/script/synchronize.h>
 
+#include <fst/script/script-impl.h>
+
 namespace fst {
 namespace script {
 
 void Synchronize(const FstClass &ifst, MutableFstClass *ofst) {
-  if (!ArcTypesMatch(ifst, *ofst, "Synchronize")) {
+  if (!internal::ArcTypesMatch(ifst, *ofst, "Synchronize")) {
     ofst->SetProperties(kError, kError);
     return;
   }
@@ -17,9 +17,7 @@ void Synchronize(const FstClass &ifst, MutableFstClass *ofst) {
   Apply<Operation<SynchronizeArgs>>("Synchronize", ifst.ArcType(), &args);
 }
 
-REGISTER_FST_OPERATION(Synchronize, StdArc, SynchronizeArgs);
-REGISTER_FST_OPERATION(Synchronize, LogArc, SynchronizeArgs);
-REGISTER_FST_OPERATION(Synchronize, Log64Arc, SynchronizeArgs);
+REGISTER_FST_OPERATION_3ARCS(Synchronize, SynchronizeArgs);
 
 }  // namespace script
 }  // namespace fst
