@@ -1,10 +1,10 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#include <fst/script/fst-class.h>
-#include <fst/script/script-impl.h>
 #include <fst/script/topsort.h>
 
+#include <fst/script/script-impl.h>
+
 namespace fst {
 namespace script {
 
@@ -14,9 +14,7 @@ bool TopSort(MutableFstClass *fst) {
   return args.retval;
 }
 
-REGISTER_FST_OPERATION(TopSort, StdArc, TopSortArgs);
-REGISTER_FST_OPERATION(TopSort, LogArc, TopSortArgs);
-REGISTER_FST_OPERATION(TopSort, Log64Arc, TopSortArgs);
+REGISTER_FST_OPERATION_3ARCS(TopSort, TopSortArgs);
 
 }  // namespace script
 }  // namespace fst
