@@ -1,10 +1,10 @@
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
-#include <fst/script/fst-class.h>
-#include <fst/script/script-impl.h>
 #include <fst/script/verify.h>
 
+#include <fst/script/script-impl.h>
+
 namespace fst {
 namespace script {
 
@@ -14,9 +14,7 @@ bool Verify(const FstClass &fst) {
   return args.retval;
 }
 
-REGISTER_FST_OPERATION(Verify, StdArc, VerifyArgs);
-REGISTER_FST_OPERATION(Verify, LogArc, VerifyArgs);
-REGISTER_FST_OPERATION(Verify, Log64Arc, VerifyArgs);
+REGISTER_FST_OPERATION_3ARCS(Verify, VerifyArgs);
 
 }  // namespace script
 }  // namespace fst
